//
//  AppDelegate.swift
//  JobThai
//
//  Created by ForAlL on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import ReachabilitySwift
import Fabric
import Crashlytics
import IQKeyboardManagerSwift
import Alamofire
//import Firebase

let applicationDelegate = UIApplication.sharedApplication().delegate as! AppDelegate
let appLanguageKey: String = "AppLanguage"
let hasSetLang : String = "hasSetLang"
var reachability: Reachability?
var screenType : DisplaySizeType = DisplaySizeType.SCREEN_UNKNOWN

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?
    var appSetting: JTMApplicationSetting?
    let numFormatter:NSNumberFormatter = NSNumberFormatter()
    var isTryingAutoLogin = false
    
    var checkTokenStatusTimer: NSTimer?
    let TIME_TO_RECHECK_TOKEN: Double = 60
    
    
    func application(application: UIApplication, didFinishLaunchingWithOptions launchOptions: [NSObject: AnyObject]?) -> Bool {
        // Override point for customization after application launch.
        
//        // Use Firebase library to configure APIs
//        FIRApp.configure()
        
        Fabric.sharedSDK().debug = true
        Fabric.with([Crashlytics.self])
        
        numFormatter.numberStyle = .DecimalStyle
        appSetting = JTMApplicationSetting()
        self.getApplicationLanguage()
        self.checkNetworkReachability()
        self.checkDeviceSize()
        
        // IQKeyboardManager
        IQKeyboardManager.sharedManager().enable = true
        IQKeyboardManager.sharedManager().enableAutoToolbar = false
        
        // Google Analytic
        // Configure tracker from GoogleService-Info.plist.
        var configureError:NSError?
        GGLContext.sharedInstance().configureWithError(&configureError)
        assert(configureError == nil, "Error configuring Google services: \(configureError)")
        
        // Optional: configure GAI options.
        let gai = GAI.sharedInstance()
        gai.trackUncaughtExceptions = true  // report uncaught exceptions
//        gai.logger.logLevel = GAILogLevel.Verbose  // remove before app release
        
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(updateTimeOutTokenTimer(_:)), name: JTM_LOGIN_STATE_DID_CHANGE, object: nil)
        
        // Initial Push Notification
        self.registerForPushNotifications(application)
        
        return true
    }
    
    // MARK:- Register Push Notification
    func application(application: UIApplication, didRegisterUserNotificationSettings notificationSettings: UIUserNotificationSettings) {
        if notificationSettings.types != .None {
            application.registerForRemoteNotifications()
        }
    }
    
    // MARK:- Gen Device token
    func application(application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: NSData) {
        let tokenChars = UnsafePointer<CChar>(deviceToken.bytes)
        var tokenString = ""
        
        for i in 0..<deviceToken.length {
            tokenString += String(format: "%02.2hhx", arguments: [tokenChars[i]])
        }
        
        print("Device Token:", tokenString)
    }
    
    // MARK:- Register Push Notification Fialed
    func application(application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: NSError) {
        print("Failed to register:", error)
    }
    
    //MARK:- Push Notiication
    func registerForPushNotifications(application: UIApplication) {
        let notificationSettings = UIUserNotificationSettings(
            forTypes: [.Badge, .Sound, .Alert], categories: nil)
        application.registerUserNotificationSettings(notificationSettings)
    }
    
    func applicationWillResignActive(application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(application: UIApplication) {
        // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    }
    
    func applicationDidBecomeActive(application: UIApplication) {
//         Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
        
//        self.tryAutoLogin()
        self.openLaunchScreen()
    }

    func applicationWillTerminate(application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }

    //Custom
//    func tryAutoLogin() {
//        //it should if(token not expire){use token to get user model} else{reconnect login service}
//        var controller = self.window?.rootViewController as? JTMCustomLaunchScreenViewController
//        
//        if(controller == nil){
//            if launchScreenView != nil {
//                controller = launchScreenView
//            }
//        }else{
//            launchScreenView = controller
//        }
//        
//        let userCredential = appSetting?.userCredential
//        let isLogin = (appSetting?.isLogin)!
//        
//        if (userCredential?.token != "" && !isLogin && !isTryingAutoLogin) {
//            
//            isTryingAutoLogin = true
//            
//            print(userCredential?.token)
//            let param = JTMLoginViewController.getAutoLogInParameter((userCredential?.token)!)
//            
//            JTMLoginViewController.connectServiceLogIn(controller!, parameter: param, redirectType: .AUTO_LOGIN, completion: { () -> Void in
//                controller!.pushToMainViewController()
//                NSNotificationCenter.defaultCenter().postNotificationName(JTM_FINISHED_AUTO_LOG_IN, object: nil)
//                self.isTryingAutoLogin = false
//            })
//    
//            JTMLoginViewController.connectServiceLogIn(controller!,
//                username: (userCredential?.username)!,
//                password: (userCredential?.password)!,
//                redirectType: .AUTO_LOGIN) {
//                    () -> Void in
//                    controller!.pushToMainViewController()
//                    NSNotificationCenter.defaultCenter().postNotificationName(JTM_FINISHED_AUTO_LOG_IN, object: nil)
//                    self.isTryingAutoLogin = false
//            }
//            
//        } else {
//            controller?.pushToMainViewController()
//        }
//    }
    
    func pushToViewController(view : UIViewController){
        applicationDelegate.window?.rootViewController?.presentViewController(view, animated: false, completion: nil)
    }
    
    func getApplicationLanguage() {
        let appLang = NSUserDefaults.standardUserDefaults().integerForKey(appLanguageKey)
        
        let isHasSetLang = NSUserDefaults.standardUserDefaults().boolForKey(hasSetLang)
        
        if !isHasSetLang {
            let deviceLang = NSLocale.currentLocale().objectForKey(NSLocaleLanguageCode) as! String
            appSetting?.currentLanguage = (deviceLang == "th") ? appLanguage.THAI : appLanguage.ENGLISH
            setApplicationLanguage((appSetting?.currentLanguage)!)
            NSUserDefaults.standardUserDefaults().setBool(true, forKey: hasSetLang)
            
        } else {
            appSetting?.currentLanguage = (appLanguage(rawValue: appLang) == .ENGLISH) ? appLanguage.ENGLISH : appLanguage.THAI
        }
       
    }
    
    func setApplicationLanguage(language:appLanguage) {
        NSUserDefaults.standardUserDefaults().setInteger(language.rawValue, forKey: appLanguageKey)
        NSUserDefaults.standardUserDefaults().synchronize();
        appSetting?.currentLanguage = language
        NSNotificationCenter.defaultCenter().postNotificationName("NOTI_LANGUAGE_CHANGE", object: nil)
    }
    
    func getApplicationLanguagePath() -> String {
        let appLang = NSUserDefaults.standardUserDefaults().integerForKey(appLanguageKey)
        let stringAppLang = (appLanguage(rawValue: appLang) == .ENGLISH) ? "en" : "th"
        let path:String = NSBundle.mainBundle().pathForResource(stringAppLang, ofType: "lproj")!
        return path
    }
    
    func getLocalizeStringForKey(key:String) -> String {
        let path:String = self.getApplicationLanguagePath()
        return (NSBundle.init(path: path)?.localizedStringForKey(key, value: "", table: nil))!
    }
    
    func checkNetworkReachability() {
       
        do {
            reachability = try Reachability.reachabilityForInternetConnection()
        } catch {
            print("Unable to create Reachability")
            return
        }
        
        NSNotificationCenter.defaultCenter().addObserver(self,
            selector: #selector(AppDelegate.reachabilityChanged(_:)),
            name: ReachabilityChangedNotification,
            object: reachability)
        
        do {
            try reachability!.startNotifier()
        } catch {
            print("Unable to start notifier")
        }
    }
    
    func reachabilityChanged(note: NSNotification) {
        
        let reachability = note.object as! Reachability
        
        if reachability.isReachable() {
            print("Reachable")
            appSetting?.isNetworkReachable = true
//            tryAutoLogin()
            self.openLaunchScreen()
        } else {
            print("Not reachable")
            appSetting?.isNetworkReachable = false
        }
    }
    
    func isDeviceScreenLessThan375Points() -> Bool {
        let SCREEN_LARGE_WIDTH : CGFloat = 375
        if(UIScreen.mainScreen().bounds.width < SCREEN_LARGE_WIDTH){
                return true
        }
        return false
    }
    
    
    func checkDeviceSize() {
        if (DEFAULT_FRAME_WIDTH <= 320) && (DEFAULT_SCREEN_HEIGHT <= 480){
            screenType = DisplaySizeType.SCREEN_3_5_INCH
            
        } else if (DEFAULT_FRAME_WIDTH == 320) && (DEFAULT_SCREEN_HEIGHT == 568){
            screenType = DisplaySizeType.SCREEN_4_0_INCH
            
        } else if (DEFAULT_FRAME_WIDTH == 375) && (DEFAULT_SCREEN_HEIGHT == 667){
            screenType = DisplaySizeType.SCREEN_4_7_INCH
            
        } else if (DEFAULT_FRAME_WIDTH >= 414) && (DEFAULT_SCREEN_HEIGHT >= 736){
            screenType = DisplaySizeType.SCREEN_5_5_INCH
        } else {
            screenType = DisplaySizeType.SCREEN_UNKNOWN
        }
    }
    
//    //MARK: Manage Map Tile
//    func callServiceTileMap() {
//        let serviceController = JTMServiceController()
//        let url = getServiceMapTileSource()
//        
//        serviceController.requestService(JTMBaseViewController(), urlString: url, params: nil, isCanLoadParallelToOther: false, isPresentLoadingIndicator: false)
//            { (response, status) -> Void in
//                if (status == .CONNECTION_SUCCESS) {
//                    let jsonObject = response!.result.value as? NSMutableDictionary
//                    self.parseTileMapJson(jsonObject)
//                }
//        }
//    }
//    
//    func parseTileMapJson(jsonObject : NSMutableDictionary?) {
//        let urlTileSource = jsonObject?.objectForKey("urlTileSource") as? String
//        let startMapRange = jsonObject?.objectForKey("startRange") as? Int
//        let stopMapRange  = jsonObject?.objectForKey("stopRange") as? Int
//        let maxZoom       = jsonObject?.objectForKey("maxZoom") as? Float
////        let minZoom       = jsonObject?.objectForKey("minZoom") as? Float
//        
//        if urlTileSource != nil && startMapRange != nil && stopMapRange != nil && maxZoom != nil {
//            checkMapTileSourceUrl(urlTileSource, startRange: startMapRange!, stopRange: stopMapRange!, maxZoom: maxZoom!)
//        }
//    }
//    
//    func checkMapTileSourceUrl(newUrl : String? , startRange : Int , stopRange : Int , maxZoom : Float , minZoom : Float = 1) {
//        let saveUrl = applicationDelegate.appSetting?.getMapTileSourceUrl()
//        let (rangeStart , rangeStop) = (applicationDelegate.appSetting?.getMapTileRange())!
//        if saveUrl != newUrl || rangeStart != startRange || rangeStop != stopRange {
//            TNMapView.clearMapTileCache()
//            applicationDelegate.appSetting?.saveMapTileSourceUrl(newUrl!)
//            applicationDelegate.appSetting?.saveMapTileRange(startRange, stop: stopRange)
//        }
//        
//        TNMapView.urlTnTileSource   = newUrl
//        TNMapView.startMapRange     = startRange
//        TNMapView.stopMapRange      = stopRange
//        TNMapView.maxZoom           = maxZoom
////        TNMapView.minZoom           = minZoom
//    }
    
    //MARK: Present Launch Screen
    func openLaunchScreen() {
        if let controller = self.window?.rootViewController as? JTMCustomLaunchScreenViewController {
            controller.pushToMainViewController()
        }
    }
    
    // MARK :- Google Analytics
    func trackGoogleAnalyticEvent(category : String, action : String, label : String) {
        print("==== trackGoogleAnalytic  Action ====")
        print("category : \(category)")
        print("action : \(action)")
        
        let language = (applicationDelegate.appSetting?.currentLanguage == .THAI) ? "_TH" : "_EN"
        
        let labelValue = (label == "TH" || label == "EN") ? label : label + language
        print("labelValue :: \(labelValue)")
        
        let tracker = GAI.sharedInstance().defaultTracker
        //        tracker.set(kGAIScreenName, value: screen)
        
        let builder = GAIDictionaryBuilder.createEventWithCategory(category, action: action, label: labelValue, value: 1)
        tracker.send(builder.build() as [NSObject : AnyObject])
    }
    
    func trackGoogleAnalyticScreen(screen : String) {
        print("==== trackGoogleAnalytic  Screen ====")
        print("screen : \(screen)")
        let tracker = GAI.sharedInstance().defaultTracker
        tracker.set(kGAIScreenName, value: screen)
        let builder = GAIDictionaryBuilder.createScreenView()
        tracker.send(builder.build() as [NSObject : AnyObject])
    }
    //MARK:- Token Time Out
    func updateTimeOutTokenTimer(notification: NSNotification) {
        let userData = notification.object as? UserModel
        if (userData != nil) {
            startCheckTokenStatusTimer()
        } else {
            stopCheckTokenStatusTimer()
        }
    }
    
    func startCheckTokenStatusTimer() {
        if (checkTokenStatusTimer == nil) {
            print("start check token")
            checkTokenStatusTimer = NSTimer.scheduledTimerWithTimeInterval(TIME_TO_RECHECK_TOKEN, target: self, selector: #selector(callServiceCheckTokenTimeOut), userInfo: nil, repeats: true)
        }
    }
    
    func stopCheckTokenStatusTimer() {
        print("stop check token")
        checkTokenStatusTimer?.invalidate()
        checkTokenStatusTimer = nil
    }
    
    func callServiceCheckTokenTimeOut() {
        let serviceController = JTMServiceController()
        let url = getServiceCheckTokenTimeOut()
        let param: [String:AnyObject]? = [
            "token" : (appSetting?.userData?.token)!
        ]
        
        serviceController.requestBackgroundService(url, params: param) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let responseJSON = response!.result.value as! NSMutableDictionary
                
                if (responseJSON["status"] as! String == "ok") {
                    print("success")
                } else {
                    print("timeout")
                    JTMLoginViewController.connectServiceLogout(nil, redirectType: .STAY_IN_CONTROLLER, completion: { () -> Void in
                        print("out")
                        NSNotificationCenter.defaultCenter().postNotificationName(JTM_TOKEN_TIME_OUT, object: nil)
                    })
                }
            }
        }
    }
    
    func getDeviceUUID() -> String {
        if let uniqueID = NSUserDefaults.standardUserDefaults().valueForKey(JTM_UNIQUE_ID_KEY) {
            return uniqueID as! String
        } else {
            let uniqueID = NSUUID().UUIDString
            NSUserDefaults.standardUserDefaults().setValue(uniqueID, forKey: JTM_UNIQUE_ID_KEY)
            NSUserDefaults.standardUserDefaults().synchronize()
            return uniqueID
        }
    }
}

