//
//  JTMApplicationSetting.swift
//  JobThai
//
//  Created by ForAlL on 1/13/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import KeychainSwift

class JTMApplicationSetting: NSObject {
    
    var isLogin:Bool = false
    var currentLanguage:appLanguage = .THAI
    var isNetworkReachable:Bool = false
    var userData:UserModel?
    var userCredential:UserCredentialModel?
//    var httpHeader : [String:String]!
    
    //SearchFilter
    var searchFilterJobCategoryList = [SearchFilterCategory]()
    var searchFilterLocationList = [SearchFilterCategory]()
    var searchFilterTransportationList = [SearchFilterCategoryTransportation]()
    var searchFilterIndustrialList = [SearchFilterCategory]()
    
    var searchFilterJobCategoryLastUpdate: Int = 0
    var searchFilterLocationLastUpdate: Int = 0
    var searchFilterTransportationLastUpdate: Int = 0
    var searchFilterIndustrialLastUpdate: Int = 0
    
    override init() {
        userCredential = UserCredentialModel().getStoredCredential()
//        httpHeader = [String:String]()
//        httpHeader["Accept"] = "application/json"
    }
    
    //MARK:- Search Suggestion
    func getSearchSuggestionListFromUserDefault() -> [String] {
        let suggestionList = NSUserDefaults.standardUserDefaults().valueForKey("searchSuggestionList") as? [String]
        return (suggestionList == nil) ? [String]() : suggestionList!
    }
    
    func setSearchSuggestionListToUserDefault(suggestionList:[String]) {
        NSUserDefaults.standardUserDefaults().setValue(suggestionList, forKey: "searchSuggestionList")
        NSUserDefaults.standardUserDefaults().synchronize()
    }
    
    func getSearchSuggestionLastUpdateFromUserDefault() -> Int {
        let lastUpdate = NSUserDefaults.standardUserDefaults().valueForKey("searchSuggestionLastUpdate") as? Int
        return (lastUpdate == nil) ? 0 : lastUpdate!
    }
    
    func setSearchSuggestionLastUpdateToUserDefault(lastUpdate:Int) {
        NSUserDefaults.standardUserDefaults().setValue(lastUpdate, forKey: "searchSuggestionLastUpdate")
        NSUserDefaults.standardUserDefaults().synchronize()
    }
    
    //MARK:- Search Filter List
    func removeAllSaveFilterList() {
        searchFilterJobCategoryList.removeAll()
        searchFilterLocationList.removeAll()
        searchFilterTransportationList.removeAll()
        searchFilterIndustrialList.removeAll()
        
        searchFilterJobCategoryLastUpdate = 0
        searchFilterLocationLastUpdate = 0
        searchFilterTransportationLastUpdate = 0
        searchFilterIndustrialLastUpdate = 0
    }
    
    //Mark:- User Log in&out
    func saveUserData(userData:UserModel) {
        self.userData = userData
    }
    
    func removeUserData() {
        self.userData = nil
        self.userCredential!.removeUserCredential()
    }
    
    func changeUserLoginState(isLogin:Bool) {
        self.isLogin = isLogin
//        print("Login Click : \(self.isLogin)" )
//        if let userData = applicationDelegate.appSetting?.userData {
//            self.setResumeCodeInHeader(userData.resumeCode!)
//            print("token \(userData.token!)")
//        } else {
//            print("null token")
//            self.removeResume()
//        }
        
        NSNotificationCenter.defaultCenter().postNotificationName(JTM_LOGIN_STATE_DID_CHANGE, object: userData)
    }
    
    func setIsRememberMe(isRememberMe:Bool) {
        NSUserDefaults.standardUserDefaults().setBool(isRememberMe, forKey: JTM_IS_REMEMBER_USER_KEY)
        NSUserDefaults.standardUserDefaults().synchronize()
    }
    
    func getIsRememberMe()->Bool {
        if let isRememberMe = NSUserDefaults.standardUserDefaults().valueForKey(JTM_IS_REMEMBER_USER_KEY) {
            return isRememberMe as! Bool
        } else {
            return true
        }
    }
    
    //MARK: Get & Save Notice Read ID List
    func getNoticeReadIdList()->NSDictionary{
        if let noticeReadHistory : NSDictionary = NSUserDefaults.standardUserDefaults().objectForKey(JTM_NOTICE_READ_KEY) as? NSDictionary{
            return noticeReadHistory
        }
        else{
            return NSDictionary()
        }
        
    }
    
    func saveNoticeReadIdList(readIdList : NSMutableDictionary){
        NSUserDefaults.standardUserDefaults().setObject(readIdList, forKey: JTM_NOTICE_READ_KEY)
    }

    //Save & Get
    func getNoticeNewCount()->Int{
        if let count : Int = NSUserDefaults.standardUserDefaults().objectForKey(JTM_NOTICE_NEW_COUNT_KEY) as? Int{
            return count
        }
        else{
            return 0
        }
        
    }
    
    func saveNoticeNewCount(number : Int){
        NSUserDefaults.standardUserDefaults().setValue(number, forKey: JTM_NOTICE_NEW_COUNT_KEY)
    }
    
//    func setResumeCodeInHeader(resumeCode : Int) {
//        httpHeader["resumecode"] = "\(resumeCode)"
////        httpHeader["resumecode"]  = "\(5387740)"
//    }
    
//    func removeResume() {
//        httpHeader.removeValueForKey("resumecode")
//    }

    
    //Mark:- Favorite Job
    func callServiceAddFavoriteForJobCode(controller:JTMBaseViewController, jobCode:Int, completion: (() -> Void)?) {
        
        let serviceController = JTMServiceController()
        let url = getServiceFavoriteJob()
        let token = applicationDelegate.appSetting?.userData?.token
        let sessionId = applicationDelegate.appSetting?.userData?.sessionId
        
        let params : [String : AnyObject] = [
            "action"    : "add",
            "jobcode"   : jobCode,
            "sessionId" : sessionId!,
            "token"     : token!
        ]
        
        serviceController.requestService(controller, urlString: url, params: params) { (response, status) -> Void in
            switch status {
                case .CONNECTION_SUCCESS:
                    completion?()
                    NSNotificationCenter.defaultCenter().postNotification(NSNotification(name: NOTI_FAVORITE_LIST_CHANGED, object: nil))
                    break
                case .CONNECTION_FAIL:
                    controller.showAlertWithData(applicationDelegate.getLocalizeStringForKey("MESSAGE_CANT_ADD_FAVORITE"),
                                                message: nil,
                                                completionHandler: nil,
                                                actionList: nil)
                    break
                default:
                    break
            }
        }
    }
    
    func callServiceRemoveFavoriteForJobCode(controller:JTMBaseViewController, jobCode:Int, completion:(()->Void)?) {
        
        let serviceController = JTMServiceController()
        let url = getServiceFavoriteJob()
        let token = applicationDelegate.appSetting?.userData?.token
        let sessionId = applicationDelegate.appSetting?.userData?.sessionId
        let param : [String:AnyObject] = [
            "action"    : "remove",
            "jobcode"   : jobCode,
            "sessionId" : sessionId!,
            "token"     : token!
        ]
        
        serviceController.requestService(controller, urlString: url, params: param) { (response, status) -> Void in
            switch status {
            case .CONNECTION_SUCCESS:
                completion?()
                NSNotificationCenter.defaultCenter().postNotification(NSNotification(name: NOTI_FAVORITE_LIST_CHANGED, object: nil))
                break
            case .CONNECTION_FAIL:
                controller.showAlertWithData(applicationDelegate.getLocalizeStringForKey("MESSAGE_CANT_REMOVE_FAVORITE"),
                                            message: nil,
                                            completionHandler: nil,
                                            actionList: nil)
                break
            default:
                break
            }
        }
    }
    
    //MARK: Manage Map Tile Source
    func saveMapTileSourceUrl(url : String) {
        NSUserDefaults.standardUserDefaults().setValue(url, forKey: MAP_TILE_SOURCE_URL_KEY)
    }
    
    func getMapTileSourceUrl() -> String? {
        if let url = NSUserDefaults.standardUserDefaults().valueForKey(MAP_TILE_SOURCE_URL_KEY) {
            return url as? String
        }
        return nil
    }
    
    func saveMapTileRange(start : Int? , stop : Int?) {
        NSUserDefaults.standardUserDefaults().setValue(start, forKey: MAP_TILE_RANGE_START)
        NSUserDefaults.standardUserDefaults().setValue(stop, forKey: MAP_TILE_RANGE_STOP)
    }
    
    func getMapTileRange() -> (Int? ,Int?) {
        var rangeStart  : Int?
        var rangeStop   : Int?
        
        if let start = NSUserDefaults.standardUserDefaults().valueForKey(MAP_TILE_RANGE_START) {
            rangeStart = start as? Int
        }
        
        if let stop = NSUserDefaults.standardUserDefaults().valueForKey(MAP_TILE_RANGE_START) {
            rangeStop = stop as? Int
        }
        
        return (rangeStart , rangeStop)
        
    }
}

