//
//  JTMAppliedJobsViewController.swift
//  JobThai
//
//  Created by kanya on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMAppliedJobsViewController: JTMBaseMenuViewController, UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout  {

    @IBOutlet weak var appliedJobsCollectionView: UICollectionView!
    @IBOutlet weak var backToTopButton: UIButton!
    
    @IBOutlet weak var appliedPositionView: UIView!
    @IBOutlet weak var positionLabel: UILabel!
    @IBOutlet weak var prefixLabel: UILabel!
    @IBOutlet weak var remarkLabel: UILabel!
//    @IBOutlet weak var statusLabel: UILabel!
    
    // CONST
    let APPLIED_CELL_IDENTIFIER = "JTMAppliedJobCollectionViewCell"
    let APPLIED_FOOTER_CELL_IDENTIFIER = "JTMAppliedJobFooterCollectionViewCollectionViewCell"
    
    let WIDTH_MARGIN : CGFloat = 4
    let HEIGHT_CELL : CGFloat = 119
    let WIDTH_MARGIN_LOAD_MORE : CGFloat = 8
    let HEIGHT_CELL_AND_MARGIN : CGFloat = 121
    let INDEX_SHOW_BACK_TO_TOP : Int = 10
    let ALPHA_SHOW_BACK_TO_TOP : CGFloat = 0.7
    let ALPHA_HIDE_BACK_TO_TOP : CGFloat = 0
    let CELL_PADDING : CGFloat = 10
    let CELL_BETWEEN_PADDING : CGFloat = 0
    
    var noResultView : JTMResumeNoResultView?
    var currentPage = 1
    var appliedJobData : AppliedJobModel?
    var totalPage : Int?
    
    var numberOfItemsPerSection = 0
    var isLoadFirstService : Bool = false
    var isSelectedCell : Bool = false
    var indexSelected = -1
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.registerNib()
        
        self.noResultView = JTMResumeNoResultView(frame: self.view.bounds)
        self.noResultView?.setNoResultForAppliedJob()
        
        self.view.addSubview(self.noResultView!)
        
        //Add Quick Search Button to SearchResultView
        self.initQuickSearchNavigationItemButton()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        noResultView?.frame = self.view.bounds
//        if !self.isLoadFirstService {
//            self.noResultView?.hidden = true
//            self.appliedJobData?.appliedJobList.removeAll()
//            self.appliedJobsCollectionView.reloadData()
//        }
    }
    
    override func viewWillAppear(animated: Bool) {
        
        print("AppliedJobs")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_APPLIED_JOBS)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLIED_JOB, action: ACTION_VIEW, label: LABEL_VIEW_APPLY_JOB)
        
        initialData()
        super.viewWillAppear(animated)
    }
    
    override func viewDidDisappear(animated: Bool) {
        
        if !(self.isSelectedCell) {
            self.noResultView?.hidden = true
//            self.appliedJobData?.appliedJobList.removeAll()
            clearControllerData()
            self.appliedJobsCollectionView.reloadData()
        }
        
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
        isCanUseAfterLogInOnly = true
    }
    
    override func clearControllerData() {
        appliedJobData = nil
        isLoadFirstService = false
        isLoadingService = false
        currentPage = 1
        numberOfItemsPerSection = 0
    }
    
    //MARK:- Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()

        if (applicationDelegate.appSetting?.isLogin)!
            && ((appliedJobData == nil) || appliedJobsCollectionView.visibleCells().contains({$0.isKindOfClass(JTMAppliedJobFooterCollectionViewCollectionViewCell)})) {
            callService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (appliedJobData == nil) {
            setAlphaShowToView(view)
        }
    }
    
    func registerNib() {
        appliedJobsCollectionView.registerNib(UINib(nibName: APPLIED_CELL_IDENTIFIER, bundle: nil), forCellWithReuseIdentifier: APPLIED_CELL_IDENTIFIER)
        appliedJobsCollectionView.registerNib(UINib(nibName: APPLIED_FOOTER_CELL_IDENTIFIER, bundle: nil), forCellWithReuseIdentifier: APPLIED_FOOTER_CELL_IDENTIFIER)
    }
    
    func callService() {
        let serviceController = JTMServiceController()
        let token = applicationDelegate.appSetting?.userData?.token
        let sessionId = applicationDelegate.appSetting?.userData?.sessionId
        let params  : [String: AnyObject] = [
            "page"      : self.currentPage,
            "action"    : "list",
            "token"     : token!,
            "sessionId" : sessionId!
        ]
        
        var url = getServiceAppliedJob()
        url = url.stringByAddingPercentEncodingWithAllowedCharacters(NSCharacterSet.URLQueryAllowedCharacterSet())!
        
        if !self.isLoadFirstService {
            self.hideAppliedPosition()
        }
        
        serviceController.requestService(self, urlString: url, params: params, isPresentLoadingIndicator: !isLoadFirstService) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                if(!self.isLoadFirstService) {
                    self.parseJsonToModel(jsonObject!)
                    print((self.appliedJobData?.appliedJobList.count)!)
                    self.isLoadFirstService = true
                    self.setAppliedJobPosition((self.appliedJobData?.countAllAppliedJob)!)
                } else {
                    self.addAppliedJobList(jsonObject!)
                }
                self.appliedJobsCollectionView.reloadData()
                self.currentPage += 1
            }
        }
    }
    
    func parseJsonToModel(jsonResult : NSMutableDictionary) {
        self.appliedJobData = AppliedJobModel(result: jsonResult)
        self.totalPage = self.appliedJobData?.countPages
        setNumberOfItem()
        self.noResultView?.hidden = (self.appliedJobData?.countAllAppliedJob == 0) ? false : true
        if self.appliedJobData?.countAllAppliedJob == 0 {
            self.noResultView?.setNoResultForAppliedJob()
        }
    }
    
    func addAppliedJobList(jsonResult : NSMutableDictionary) {
        let appliedJobObject = AppliedJobModel(result: jsonResult)
        self.appliedJobData?.appliedJobList = (self.appliedJobData?.appliedJobList)! + appliedJobObject.appliedJobList
        
        setNumberOfItem()
    }
    
    func hideAppliedPosition() {
        self.appliedPositionView.hidden = true
        self.remarkLabel.hidden = true
    }
    
    func setAppliedJobPosition(position: Int) {
        self.positionLabel.text = applicationDelegate.numFormatter.stringFromNumber(position)
        self.remarkLabel.text = applicationDelegate.getLocalizeStringForKey("JOB_WILL_BE_AUTO_REMOVE")
        self.prefixLabel.text = applicationDelegate.getLocalizeStringForKey("PREFIX_POSITION")
//        self.statusLabel.text = applicationDelegate.getLocalizeStringForKey("STATUS_APPLIED_JOB")

        self.remarkLabel.hidden = false
        self.appliedPositionView.hidden = false
        self.prefixLabel.hidden = false
        
        // FOR UI TEST
        self.positionLabel.accessibilityLabel = self.positionLabel.text
    }
    
    func setNumberOfItem() {
        if self.appliedJobData == nil {
            self.numberOfItemsPerSection = 0
        }else if self.currentPage < self.totalPage {
            self.numberOfItemsPerSection = (self.appliedJobData?.appliedJobList.count)! + 1
        } else {
            self.numberOfItemsPerSection = (self.appliedJobData?.appliedJobList.count)!
        }
        
    }
    
    // Mark: SetHighlight Cell Seclected
//    func setHighlight() {
//        self.appliedJobsCollectionView?.selectItemAtIndexPath(NSIndexPath(forItem: indexSelected, inSection: 0), animated: false, scrollPosition: .None)
//    }
    
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_APPLIED_JOB")
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func initialData() {
        // Hide back to top button
        self.backToTopButton.alpha = self.ALPHA_HIDE_BACK_TO_TOP
        appliedJobsCollectionView.accessibilityIdentifier = ID_APPLIED_JOBS_COLLECTION_VIEW
        
        if (applicationDelegate.appSetting?.isLogin == false) {
            manageViewOnError(connectionErrorView!)
        } else if !(self.isSelectedCell) {
            // Initial Data
            self.currentPage = 1
            self.isLoadFirstService = false
            self.numberOfItemsPerSection = 0
            
            // Load Data
            callService()
        } else {
            self.isSelectedCell = false
        }
        
        
    }
    
    func onClickCellSubView(item : AppliedJobItem) {
        
        if (item.onlineStatus == "1") {
            let jobDetailStoryBoard = UIStoryboard(name: "JobDetail", bundle: nil)
            let jobDetail = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMJobDetailViewController") as? JTMJobDetailViewController
        
            jobDetail?.companyCode = item.companyCode!
            jobDetail?.jobCode = item.jobCode!
            jobDetail?.openPageType = .JOB_INFO_PAGE
            
            self.navigationController?.pushViewController(jobDetail!, animated: true)
        } else {
            self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_CLOSED_JOB"), message: nil, completionHandler: nil, actionList: nil)
        }
    }
    
    //Mark:- UIBarButton
    override func genUISearchMenuButton() -> UIBarButtonItem? {
        return nil
    }
    
    @IBAction func onClickBackToTopButton(sender: AnyObject) {
        appliedJobsCollectionView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
    }
    
    // MARK: - Scroll
    func scrollViewDidScroll(scrollView: UIScrollView) {
        let offset : CGPoint = scrollView.contentOffset;
        let bounds : CGRect = scrollView.bounds;
        let size : CGSize = scrollView.contentSize;
        let inset : UIEdgeInsets = scrollView.contentInset;
        let y : CGFloat = offset.y + bounds.size.height - inset.bottom;
        let h  : CGFloat = size.height;
        
        if(y > h - RELOAD_DISTANCE){
            if !isLoadingService && (self.currentPage <= self.totalPage) {
//                self.currentPage++
                callService()
            }
        }
        
//        let currentScrollOffset = appliedJobsCollectionView!.contentOffset.y + appliedJobsCollectionView!.frame.height
//        let cellOffset = HEIGHT_CELL * CGFloat(INDEX_SHOW_BACK_TO_TOP - 1) + (DEFAULT_RESUME_CELL_MARGIN/2) * CGFloat(INDEX_SHOW_BACK_TO_TOP)
//        
//        if(currentScrollOffset >= cellOffset) {
//            self.backToTopButton!.alpha = self.ALPHA_SHOW_BACK_TO_TOP
//        } else {
//            self.backToTopButton!.alpha = self.ALPHA_HIDE_BACK_TO_TOP
//        }
        
        
        let visibleIndex = appliedJobsCollectionView.indexPathsForVisibleItems()
        
        if visibleIndex.count > 0 {
            let arrayVisible = visibleIndex.sort {$0.row < $1.row}
            let index = arrayVisible.last!.row + 1
            
            if(index >= self.INDEX_SHOW_BACK_TO_TOP && scrollView.contentOffset.y != 0) {
                self.backToTopButton!.alpha = self.ALPHA_SHOW_BACK_TO_TOP
            } else {
                self.backToTopButton!.alpha = self.ALPHA_HIDE_BACK_TO_TOP
            }
        }
        
    }
    

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */
    
    
    
    // MARK: - CollectionView
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return self.numberOfItemsPerSection
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        if indexPath.row + 1 > self.appliedJobData?.appliedJobList.count {
            let cell = collectionView.dequeueReusableCellWithReuseIdentifier(APPLIED_FOOTER_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMAppliedJobFooterCollectionViewCollectionViewCell
            return cell!
        } else {
            let cell = collectionView.dequeueReusableCellWithReuseIdentifier(APPLIED_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMAppliedJobCollectionViewCell
            let itemCellData = self.appliedJobData?.appliedJobList[indexPath.row]
            cell?.setDataToCell(itemCellData!, sequence: indexPath.row)
            return cell!
        }
    }

    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
//        return indexPath.row + 1 > self.appliedJobData?.appliedJobList.count ? CGSize(width: collectionView.frame.width - self.WIDTH_MARGIN_LOAD_MORE, height: LOADMORE_CELL_HEIGHT) : CGSize(width: self.view.frame.width - self.WIDTH_MARGIN, height: self.HEIGHT_CELL)
//        return indexPath.row + 1 > self.appliedJobData?.appliedJobList.count ? CGSize(width: collectionView.frame.width - self.WIDTH_MARGIN_LOAD_MORE, height: LOADMORE_CELL_HEIGHT) : CGSize(width: self.view.frame.width - DEFAULT_RESUME_CELL_MARGIN, height: self.HEIGHT_CELL)
        
        let width = collectionView.frame.width
        
        if indexPath.row + 1 > self.appliedJobData?.appliedJobList.count {
            return CGSize(width: width, height: LOADMORE_CELL_HEIGHT)
        } else {
            return collectionView.ar_sizeForCellWithIdentifier(APPLIED_CELL_IDENTIFIER, indexPath: indexPath, fixedWidth: width, configuration: { ( cellItem) in
                let itemCellData = self.appliedJobData?.appliedJobList[indexPath.row]
                if let cell = cellItem as? JTMAppliedJobCollectionViewCell {
                    cell.setDataToCell(itemCellData!, sequence: indexPath.row)
                }
            })
            
        }
        
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAtIndex section: Int) -> CGFloat {
        return CELL_BETWEEN_PADDING
    }
    
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        indexSelected = indexPath.row
        
        if indexSelected + 1 <= self.appliedJobData?.appliedJobList.count {
            
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLIED_JOB, action: ACTION_CLICK, label: LABEL_JOB_DETAIL)
            
            self.onClickCellSubView((self.appliedJobData?.appliedJobList[indexPath.row])!)
            self.isSelectedCell = true
        }
    }
    
    func collectionView(collectionView: UICollectionView, didHighlightItemAtIndexPath indexPath: NSIndexPath) {
        if indexPath.row + 1 <= self.appliedJobData?.appliedJobList.count {
            let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMAppliedJobCollectionViewCell
            cell?.backgroundColor = JTM_HIGHLIGHT_COLOR
        }
    }
    
    func collectionView(collectionView: UICollectionView, didUnhighlightItemAtIndexPath indexPath: NSIndexPath) {
        if indexPath.row + 1 <= self.appliedJobData?.appliedJobList.count {
            let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMAppliedJobCollectionViewCell
            cell?.backgroundColor = UIColor.whiteColor()
        }
    }
    
}
