//
//  JTMBaseViewController.swift
//  JobThai
//
//  Created by ForAlL on 1/13/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import ReachabilitySwift
import MBProgressHUD
import MHVideoPhotoGallery
import ENSwiftSideMenu
import UICollectionView_ARDynamicHeightLayoutCell_Bell

class JTMBaseViewController: UIViewController {

    var isCanUseAfterLogInOnly : Bool = false
    var isIgnoreTokenTimeOutAlert : Bool = false
    var isLoadingService : Bool = false
    var isPresentedInOtherController : Bool = false //true in controller that presented in other ex: jobdetailpageview/ jobdetailinfo
    var isNeedToDimOnConnectionError : Bool = false //true if controller show bade UI on service connection error
    var isHasRetryButtonOnAlertConnectionError : Bool = true //false if don't have any retry connection logic
    var isPresentedBefore : Bool = false
    var loadingView : MBProgressHUD?
    var noInternetView : JTMNoResultView?
    var noInternetTabView : JTMNoNetworkHeaderTabView?
    var connectionErrorView : UIView?
    
    let VIEW_HIDE_ALPHA : CGFloat = 0
    let VIEW_SHOW_ALPHA : CGFloat = 1
    let NO_INTERNET_TAB_HEIGHT : CGFloat = 35
    
    var dimBackground : UIView = UIView()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setControllerFlags()
        initConnectionErrorView()
        initNoInterNetView()
        initBaseObserver()
        self.navigationItem.leftBarButtonItem = genUIBackButton()
        //setDefaultBackButton()
        
        //Setup NavBar if it is JTMPresentNavigation
        setUpPresentNavBar()
        
        // Set up Dim Background
        setUpDimBackground()
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(showDimBackground), name: OBSERVER_SHOW_DIM_BACKGROUND, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(hideDimBackground), name: OBSERVER_HIDE_DIM_BACKGROUND, object: nil)
        
        if let popGesture = self.navigationController?.interactivePopGestureRecognizer {
            popGesture.enabled = false
        }
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        initReachabilityObserver()
        updateViewForReachabilityChange()
        
        print("current Viewcontroller ::: \(self.navigationController?.visibleViewController)")
        
//        if self.checkHasSeenQuickGuide() {
//            self.presentQuickGuide()
//        }
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        
        if (!isPresentedBefore) {
            self.performSelector(#selector(setPresentedBeforeFlag), withObject: nil, afterDelay: 0.1)
        }
        
        if isCanUseAfterLogInOnly && !isIgnoreTokenTimeOutAlert {
            updateControllerAfterUserStatusChanged()
        }
        
//        noInternetView?.frame = self.view.bounds
        
    }
    
    override func viewWillDisappear(animated: Bool) {
        removeReachabilityObserver()
        isIgnoreTokenTimeOutAlert = false
    }
    
    func setControllerFlags() {
    }
    
    func clearControllerData() {
    }
    
    //MARK:- Observer
    //init reachability in viewwillappear and remove when go out => don't need controller not in window callservice on reachability change
    func initBaseObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(onLanguageChange), name: "NOTI_LANGUAGE_CHANGE", object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(applicationDidActive), name: UIApplicationDidBecomeActiveNotification, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(applicationDidEnterBackground), name: UIApplicationDidEnterBackgroundNotification, object: nil)
        
        if isCanUseAfterLogInOnly {
            NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(updateControllerAfterUserStatusChanged), name: JTM_TOKEN_TIME_OUT, object: nil)
        }
    }
    
    func initReachabilityObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(reachabilityChanged(_:)), name: ReachabilityChangedNotification, object: reachability)
    }
    
    func removeReachabilityObserver() {
        NSNotificationCenter.defaultCenter().removeObserver(self, name: ReachabilityChangedNotification, object: reachability)
    }
    
    //MARK:- Blank View
    func initConnectionErrorView() {
        connectionErrorView = UIView(frame: self.view.bounds)
        connectionErrorView?.backgroundColor = UIColor.whiteColor()
        connectionErrorView?.alpha = VIEW_SHOW_ALPHA
        connectionErrorView?.hidden = isPresentedInOtherController || !isNeedToDimOnConnectionError
        self.view.addSubview(connectionErrorView!)
    }
    
    //do while get response fail from service
    func onConnectionError(errorRequestUrl: String?) {
        print("error on \(errorRequestUrl)")
        showAlertConnectionError()
        manageViewOnError(connectionErrorView!)
    }
    
    //do while get response success from service
    func onConnectionSuccess() {
        connectionErrorView?.alpha = VIEW_HIDE_ALPHA
    }
    
    //do while user click "retry" button on alert connection error or internet just reachable
    //can set controller callservice in this method cause it called in viewwillappear
    func onRetryConnection() {
    }
    
    //do something after user tap "ok" button on alert connection error
    func onDismissAlertConnectionError() {
    }
    
    //alert connection error. override and make this method blank if not need controller present any alert on connection error
    func showAlertConnectionError() {
        let alert = UIAlertController(title: applicationDelegate.getLocalizeStringForKey("ALERT_CONNECTION_ERROR"),
                                    message: applicationDelegate.getLocalizeStringForKey("MESSAGE_CONNECTION_ERROR"),
                                    preferredStyle: UIAlertControllerStyle.Alert)
        
        let onClickDismissHandler = {(action:UIAlertAction) -> Void in
                                        self.onDismissAlertConnectionError()
                                    }
        
        let onClickRetryHandler = {(action:UIAlertAction) -> Void in
                                        self.onRetryConnection()
                                    }
        
        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("OK"), style: .Default, handler:onClickDismissHandler))
        
        //some controller isn't need to call service to show ui
        if isHasRetryButtonOnAlertConnectionError {
            alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("RETRY"), style: .Default, handler: onClickRetryHandler))
        }

        if (self.presentedViewController == nil) {
            self.presentViewController(alert, animated: true, completion: nil)
        }
    }
    
    //MARK:- For Network Reachability Change
    func initNoInterNetView() {
        let tabViewFrame = CGRect(x: 0, y: 0, width: self.view.frame.width, height: NO_INTERNET_TAB_HEIGHT)
        noInternetTabView = JTMNoNetworkHeaderTabView(frame: tabViewFrame)
        noInternetTabView?.alpha = VIEW_HIDE_ALPHA
        noInternetTabView?.hidden = isPresentedInOtherController
        self.view.addSubview(noInternetTabView!)

        var noInternetFrame = self.view.bounds
        noInternetFrame.size.height = noInternetFrame.height - NAVIGATION_BAR_AND_STATUS_BAR_HEIGHT
        print("noInternetFrame :: \(noInternetFrame)")
        noInternetView = JTMNoResultView(frame: noInternetFrame)
        noInternetView?.setNoResultForNoNetwork()
        noInternetView?.alpha = (applicationDelegate.appSetting?.isNetworkReachable)! ? VIEW_HIDE_ALPHA : VIEW_SHOW_ALPHA
        noInternetView?.hidden = isPresentedInOtherController
        self.view.addSubview(noInternetView!)
        
    }
    
    //do while recieve observer that reachability status jus changed
    func reachabilityChanged(note: NSNotification) {
        updateViewForReachabilityChange()
    }
    
    //update no internet view & tab view after reachability status just changed
    func updateViewForReachabilityChange() {
        if (applicationDelegate.appSetting?.isNetworkReachable)! {
            noInternetTabView?.alpha = VIEW_HIDE_ALPHA
            noInternetView?.alpha = VIEW_HIDE_ALPHA
//            connectionErrorView?.hidden = isPresentedInOtherController || !isNeedToDimOnConnectionError
            manageViewAfterInternetReachable()
        } else {
            self.view.bringSubviewToFront(noInternetTabView!)
//            self.view.bringSubviewToFront(noInternetView!)
            noInternetTabView?.alpha = VIEW_SHOW_ALPHA
//            connectionErrorView?.hidden = true
            manageViewAfterInternetNotReachable()
        }
    }
    
    //call onRetryConnection after internet just reachable, override this method if controller need something to do after internet reachable only
    func manageViewAfterInternetReachable() {
        onRetryConnection()
    }
    
    //call manageViewOnError after internet just unreachable, override this method if controller need something to do after internet unreachable only
    func manageViewAfterInternetNotReachable() {
        manageViewOnError(noInternetView!)
    }
    
    //MARK:- Manage Controller In Any Status
    //called when internet unreachabled or connect service get fail
    func manageViewOnError(view: UIView) {
        self.view.bringSubviewToFront(view)
    }
    
    func setAlphaShowToView(view: UIView) {
        view.alpha = VIEW_SHOW_ALPHA
    }
    
    func setPresentedBeforeFlag() {
        isPresentedBefore = true
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func onLanguageChange(){
        noInternetView?.setNoResultForNoNetwork()
        noInternetTabView?.setLabel()
    }
    
    // MARK:- String
    func replaceNewLineString(string : String) -> String {
        
        let replacedString = string.stringByReplacingOccurrencesOfString("\n", withString: "")
        
        return replacedString
    }
    
    // MARK:- Dim Background --------------------------------------------
    func setUpDimBackground() {
        dimBackground.frame = view.bounds
        dimBackground.autoresizingMask = [.FlexibleWidth, .FlexibleHeight]
        dimBackground.backgroundColor = UIColor.blackColor()
        dimBackground.alpha = 0.6
    }
    
    func hideDimBackground() {
        self.dimBackground.removeFromSuperview()
    }
    
    func showDimBackground() {
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_MENU_BAR, action: ACTION_VIEW, label: LABEL_VIEW_MENU)
        self.view.addSubview(dimBackground)
    }
    //---------------------------------------------------------------------
    
    
    func genUIBackButton() -> UIBarButtonItem {
        let btnName = UIButton()
        btnName.setImage(UIImage(named: "ic_back"), forState: .Normal)
        btnName.frame = CGRectMake(0, 0, 30, 30)
        btnName.addTarget(self, action: #selector(actionBack), forControlEvents: .TouchUpInside)
        btnName.accessibilityIdentifier = ID_NAV_BAR_BACK_BUTTON
        //.... Set Right/Left Bar Button item
        let leftBarButton = UIBarButtonItem()
        leftBarButton.customView = btnName
        
        return leftBarButton
    }
    
    func setDefaultBackButton() {
        self.navigationItem.backBarButtonItem = UIBarButtonItem(title:"", style:.Plain, target:nil, action:nil)
        self.navigationController?.navigationBar.tintColor = JTM_SEGMENT_TEXT_COLOR
    }

    func actionBack() {
        self.navigationController?.popViewControllerAnimated(true)
    }
    
    func alertActionHandler(alert : UIAlertAction){
//        print("Alert Action Handler")
    }
    
    func showAlertNoNetworkIsReachable() {
//        let alert = UIAlertController(title: applicationDelegate.getLocalizeStringForKey("ALERT_NO_INTERNET"),
//                                    messa ge: applicationDelegate.getLocalizeStringForKey("MESSAGE_NO_INTERNET"),
//                                preferredStyle: UIAlertControllerStyle.Alert)
//        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("OK"), style: .Default, handler:alertActionHandler))
//        
//        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    func showAlertNoNetworkIsReachable(completionHandler:((UIAlertAction) -> Void)) {
//        let alert = UIAlertController(title: applicationDelegate.getLocalizeStringForKey("ALERT_NO_INTERNET"),
//            message: applicationDelegate.getLocalizeStringForKey("MESSAGE_NO_INTERNET"),
//            preferredStyle: UIAlertControllerStyle.Alert)
//        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("OK"), style: .Default, handler:completionHandler))
//        
//        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    func showAlertWithData(title:String, message:String?, completionHandler:((UIAlertAction) -> Void)?, actionList:[UIAlertAction]?) {
        let alert = UIAlertController(title: title, message: message, preferredStyle: .Alert)
        let OKAction = UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("OK"), style: .Default, handler:completionHandler)
        alert.addAction(OKAction)
        alert.view.accessibilityIdentifier = "alert"

        if let otherActionList = actionList {
            for alertAction in otherActionList {
                alert.addAction(alertAction)
            }
        }
        
        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    func showLoading() {
//        let loadingNotification = MBProgressHUD.showHUDAddedTo(self.view, animated: true)
//        loadingNotification.mode = MBProgressHUDMode.Indeterminate
        
        if (loadingView == nil) {
            loadingView = MBProgressHUD(view: self.view)
            self.view.addSubview(loadingView!)
        }
        
        self.view.bringSubviewToFront(loadingView!)
        loadingView?.show(true)
    }
    
    func hideLoading() {
        loadingView?.hide(true)
//        MBProgressHUD.hideAllHUDsForView(self.view, animated: true)
    }
    
    func showLoadingWithBlankView() {
        connectionErrorView?.hidden = false
        setAlphaShowToView(connectionErrorView!)
        self.showLoading()
    }
    
    func hideLoadingWithBlankView() {
        self.connectionErrorView?.hidden = true
        self.connectionErrorView?.alpha = VIEW_HIDE_ALPHA
        self.hideLoading()
    }
    
    //Mark:- Gallery
    func openImageGallery(imageView:UIImageView, listImageUrl:[NSURL], presentIndex: Int? = 0) {
        
        var listGalleryItem = [MHGalleryItem]()
        
        for url in listImageUrl {
            let galleryItem = MHGalleryItem(URL: "\(url)", galleryType: MHGalleryType.Image)
            listGalleryItem.append(galleryItem)
        }
        
        let galleryController = JTMCustomMHGalleryController(presentationStyle: .ImageViewerNavigationBarShown) //MHGalleryController(presentationStyle: .ImageViewerNavigationBarShown)
        galleryController.delegate = self.navigationController?.delegate
        galleryController.galleryItems = listGalleryItem
        galleryController.presentationIndex = presentIndex!
        galleryController.finishedCallback = { (currentIndex:Int, image:UIImage!, controller:MHTransitionDismissMHGallery!, mode:MHGalleryViewMode) -> Void in
            galleryController.dismissViewControllerAnimated(true, completion: nil)
        }
        
        galleryController.UICustomization.barStyle = .Black
        galleryController.UICustomization.barButtonsTintColor = UIColor.whiteColor()
        galleryController.UICustomization.showOverView = false
        galleryController.UICustomization.showMHShareViewInsteadOfActivityViewController = false
        galleryController.UICustomization.setMHGalleryBackgroundColor(UIColor.blackColor(), forViewMode: MHGalleryViewMode.ImageViewerNavigationBarShown)
        
        galleryController.transitionCustomization.interactiveDismiss = true
//        galleryController.transitionCustomization.fixXValueForDismiss = false
//        galleryController.transitionCustomization.dismissWithScrollGestureOnFirstAndLastImage = true
    
        self.navigationController?.presentMHGalleryController(galleryController, animated: true, completion: nil)
    }
    
    //MARK: Alert for User Location
    //Alert Custom Not Allow
    func alertNoPermissionToAccess() {
        showDialogWithTitle(applicationDelegate.getLocalizeStringForKey("TITLE_ALERT_MESSAGE"), message: applicationDelegate.getLocalizeStringForKey("MESSAGE_ALERT_MESSAGE"), okTitle: applicationDelegate.getLocalizeStringForKey("OK"), settingTitle: applicationDelegate.getLocalizeStringForKey("SETTING_ALERT_MESSAGE_BUTTON"), withokAction: nil, settingAction: {
            UIApplication.sharedApplication().openURL(NSURL(string: UIApplicationOpenSettingsURLString)!)
        })
    }
    
    // Mark :: Alert Custom Open Location Service
    func alertNoOpenLocationService() {
        showDialogWithTitle(applicationDelegate.getLocalizeStringForKey("TITLE_ALERT_MESSAGE"), message: applicationDelegate.getLocalizeStringForKey("MESSAGE_ALERT_MESSAGE"), okTitle: applicationDelegate.getLocalizeStringForKey("OK"), settingTitle: applicationDelegate.getLocalizeStringForKey("SETTING_ALERT_MESSAGE_BUTTON"), withokAction: nil, settingAction: {
            UIApplication.sharedApplication().openURL(NSURL(string: "prefs:root=LOCATION_SERVICES")!)
        })
    }
    
    // Mark ::  Alert
    private func showDialogWithTitle(title: String, message: String, okTitle: String, settingTitle: String, withokAction okAction : (() -> Void)?, settingAction: (() -> Void)?) {
        dispatch_async(dispatch_get_main_queue()) {
            let alertController = UIAlertController(title: title, message: message, preferredStyle: .Alert)
            
            let okAction: UIAlertAction = UIAlertAction(title: okTitle, style: .Default) { _ in
                okAction?()
            }
            let settingAction: UIAlertAction = UIAlertAction(title: settingTitle, style: .Default) { _ in
                settingAction?()
            }
            
            alertController.addAction(settingAction)
            alertController.addAction(okAction)
            
            self.presentViewController(alertController, animated: true, completion: nil)
        }
    }

    //MARK: Observer for Check Application Active
    func applicationDidActive(){
    }
    
    //MARK: Observer for Check Application Enter Background
    func applicationDidEnterBackground() {
        
    }
    
    //MARK: PresentNav for gent Cancel and Back Button
    func setUpPresentNavBar(){
        if(self.navigationController?.isKindOfClass(JTMPresentNavigationViewController)) == true{
            setUpCancelButton()
            setUpBackButton()
            self.navigationController?.navigationBar.translucent = false
        }
    }
    
    func setUpCancelButton(){
        let cancelString = applicationDelegate.getLocalizeStringForKey("CANCEL")
        let rightBarButton = UIBarButtonItem(title: cancelString, style: UIBarButtonItemStyle.Plain , target: self, action: #selector(actionPresentNavCancel))
        rightBarButton.accessibilityIdentifier = ID_NAV_BAR_QUICK_SEARCH_CANCEL
        self.navigationItem.rightBarButtonItem = rightBarButton
        
        //set up color
        self.navigationItem.rightBarButtonItem?.tintColor = JTM_SEGMENT_TEXT_COLOR
    }
    
    func setUpBackButton(){
        if(self.navigationController?.viewControllers.count > 1){
            addBackButton()
        }else{
            removeBackButton()
        }
    }
    
    func addBackButton(){
        let btnName = UIButton()
        btnName.setImage(UIImage(named: "ic_back"), forState: .Normal)
        btnName.frame = CGRectMake(0, 0, 30, 30)
        btnName.addTarget(self, action: #selector(actionPresentNavBack), forControlEvents: .TouchUpInside)
        
        let leftBarButton = UIBarButtonItem(customView: btnName)
        self.navigationItem.leftBarButtonItem = leftBarButton
    }
    
    func removeBackButton(){
        self.navigationItem.leftBarButtonItem = nil
    }
    
    func actionPresentNavCancel(){
        self.navigationController?.dismissViewControllerAnimated(true, completion: nil)
    }
    
    func actionPresentNavBack(){
        if(self.navigationController?.viewControllers.count > 1){
            self.navigationController?.popViewControllerAnimated(true)
        }else{
            self.dismissViewControllerAnimated(true, completion: nil)
        }
        
    }
    
    //MARK:- Manage TimeOut
    func updateControllerAfterUserStatusChanged() {
        //not do anything on user login
        if (applicationDelegate.appSetting?.isLogin)! {
            return
        }
        
        //show alert while controller is visible only
        if checkIfControllerIsShowing() {
            showAlertWithData(applicationDelegate.getLocalizeStringForKey("TOKEN_TIMEOUT_TITLE"),
                message: applicationDelegate.getLocalizeStringForKey("TOKEN_TIMEOUT_MESSAGE"),
                completionHandler: { (alertAction) -> Void in
                    self.showLogInViewController(.DISMISS_CONTROLLER, completion: nil)
                }, actionList: nil)
        }
        
        self.clearControllerData()
    }

    func showLogInViewController(redirectType: LogInRedirectType , parentToBack : UIViewController? = nil, completion: (()->Void)? ) {
        let mainStoryboard = UIStoryboard(name: "Main", bundle: nil)
        let loginController = mainStoryboard.instantiateViewControllerWithIdentifier("JTMLoginViewController") as! JTMLoginViewController
        loginController.redirectType = redirectType
        loginController.completion = completion
        loginController.isParentNeedLogInStatus = isCanUseAfterLogInOnly
        loginController.parentToBack = parentToBack
        self.navigationController?.pushViewController(loginController, animated: true)
    }
    
    //MARK:- Check viewController is appear
    func checkIfControllerIsShowing() -> Bool {
        //true if controller is already load to memory & has uiwindow
        return self.isViewLoaded() && (self.view.window != nil)
    }
    
    // MARK:- Quick Guide
//    func presentQuickGuide() {
//        let quickGuideViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainQuickGuideViewController") as? JTMMainQuickGuideViewController
//        self.presentViewController(quickGuideViewController!, animated: true, completion: nil)
//        NSUserDefaults.standardUserDefaults().setBool(true, forKey: HAS_SEEN_QUICK_GUIDE)
//    }
//    
//    func checkHasSeenQuickGuide() -> Bool {
//        
//        var isPageToShow = false
//        let currentViewController = self.navigationController?.visibleViewController
//        
//        if currentViewController?.isKindOfClass(JTMHomeViewController) == true {
//            isPageToShow = true
//        } else if currentViewController?.isKindOfClass(JTMWebViewController) == true {
//            isPageToShow = true
//        } else if currentViewController?.isKindOfClass(JTMJobDetailViewController) == true {
//            isPageToShow = true
//        }
//        
//        return !NSUserDefaults.standardUserDefaults().boolForKey(HAS_SEEN_QUICK_GUIDE) && isPageToShow
//    }
}

//MARK : Extension QuickSearch on NavBar
extension JTMBaseViewController {
    
    func initQuickSearchNavigationItemButton() {
        if self.navigationController?.isKindOfClass(JTMPresentNavigationViewController) == false{
            self.navigationItem.rightBarButtonItem = genUIQuickSearchMenuButton()
        }
    }
    
    func genUIQuickSearchMenuButton() -> UIBarButtonItem? {
        let btnName = UIButton()
        btnName.setImage(UIImage(named: "ic_search"), forState: .Normal)
        btnName.frame = CGRectMake(0, 0, 25, 25)
        btnName.addTarget(self, action: #selector(JTMBaseViewController.actionQuickSearachNavBar), forControlEvents: .TouchUpInside)
        btnName.accessibilityIdentifier = ID_NAV_BAR_QUICK_SEARCH_BUTTON
        
//        Set Right/Left Bar Button item
        let rightBarButton = UIBarButtonItem()
        rightBarButton.customView = btnName
        
//        let searchBarButton = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.Search, target: self, action: #selector(actionQuickSearachNavBar))
//        searchBarButton.accessibilityIdentifier = ID_NAV_BAR_QUICK_SEARCH_BUTTON
        
        return rightBarButton //searchBarButton
    }
    
    func actionQuickSearachNavBar() {
        print("Click Search")
        hideSideMenuView()
        openMainSearchFilter()
        self.view.endEditing(true)
    }
    
    func openMainSearchFilter() {
        
        print("openSearchFilter() -- Base")
        let searchFilterStoryboard: UIStoryboard = UIStoryboard(name: "SearchFilter",bundle: nil)
        let searchFilterviewController:UIViewController = searchFilterStoryboard.instantiateViewControllerWithIdentifier("JTMSearchFilterTypeViewController")
        let quickSearchNavController = JTMPresentNavigationViewController(rootViewController : searchFilterviewController)
        self.presentViewController(quickSearchNavController, animated: true, completion: nil)
    }
}
