//
//  JTMBrowseJobViewController.swift
//  JobThai
//
//  Created by weerapon on 1/19/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMBrowseJobViewController : JTMBaseViewController , JTMBrowseJobsModeJobsViewDelegate, JTMBrowseJobCareersViewDelagate {
    
    @IBOutlet weak var modeJobsView : JTMBrowseJobsModeJobsView?
    @IBOutlet weak var careerJobsView : JTMBrowseJobCareersView?
    @IBOutlet weak var headerJobsView : JTMBrowseJobsHeaderView?
    @IBOutlet weak var headerViewHeightConstraint: NSLayoutConstraint!
    @IBOutlet weak var modeJobViewHeightConstraint: NSLayoutConstraint!
    @IBOutlet weak var careerJobViewHeightConstraint: NSLayoutConstraint!
    @IBOutlet weak var ScrollViewContentHeightConstraint: NSLayoutConstraint!

    var browseJobsData : BrowseJobMainPageModel?
    var browseJobsCurrentIndex : Int = -1
    
    var listFirstLevelViewControllers: NSMutableDictionary?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.listFirstLevelViewControllers = NSMutableDictionary()
        self.modeJobsView?.delegate = self
        self.careerJobsView?.delegate = self
        connectService()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("BrowseJob viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_MAIN_BROWSE_JOB)
        
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_BROWSE_JOBS")
        
        clearActive()
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- On Retry Connection
    override func onRetryConnection() {
        super.onRetryConnection()
        if (browseJobsData == nil) {
            connectService()
        }
    }
    
    //MARK:- Manage View On Connection Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        
        if (browseJobsData == nil) {
            setAlphaShowToView(view)
        }
    }
    
    func clearActive() {
        let selectedList = careerJobsView?.browseJobCareersCollection?.indexPathsForSelectedItems()
        if(selectedList?.count > 0) {
            for indexPath in (selectedList)! {
                careerJobsView!.deSelectItem(indexPath)
            }
        }
        let selectedList2 = modeJobsView?.modeJobsCollection?.indexPathsForSelectedItems()
        if(selectedList2?.count > 0) {
            for indexPath in (selectedList2)! {
                modeJobsView!.deSelectItem(indexPath)
            }
        }
    }
    
    static func openBrowseJobViewController(storyboard : UIStoryboard, navigationController : UINavigationController, type : typeBrowseJob, code : String? = nil, searchParams : [String:AnyObject]?, searchMode : ModeJobType = .NONE, title : String? = nil, parent : JTMBrowseJobViewController? = nil){
        
        var controller:UIViewController?
        switch(type){
            case .HOME:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMBrowseJobViewController")
                break
            case .FIRST_LEVEL:
                if(parent != nil){
                    let listViewControllers:NSMutableDictionary = (parent?.listFirstLevelViewControllers)!
                    controller = listViewControllers.objectForKey(searchMode.rawValue) as? UIViewController
                    if(controller == nil){
                        controller =  storyboard.instantiateViewControllerWithIdentifier("JTMBrowseJobFirstLevelViewController")

                        listViewControllers.setObject(controller!, forKey: searchMode.rawValue)
                    }
                    let firstLevelController:JTMBrowseJobFirstLevelViewController = controller as! JTMBrowseJobFirstLevelViewController
                    let selectedItem = firstLevelController.collectionView?.indexPathsForSelectedItems()
                    firstLevelController.setModeTypeAndReloadService(searchMode,loadService:searchMode != firstLevelController.jobType)
                    if(selectedItem?.count > 0) {
                        for indexPath in (selectedItem)! {
                            firstLevelController.deSelectItem(indexPath)
                        }
                    }
                }else{
                    controller =  storyboard.instantiateViewControllerWithIdentifier("JTMBrowseJobFirstLevelViewController")
                    let firstLevelController:JTMBrowseJobFirstLevelViewController = controller as! JTMBrowseJobFirstLevelViewController
                    let selectedItem = firstLevelController.collectionView?.indexPathsForSelectedItems()
                    firstLevelController.setModeTypeAndReloadService(searchMode)
                    if(selectedItem?.count > 0) {
                        for indexPath in (selectedItem)! {
                            firstLevelController.deSelectItem(indexPath)
                        }
                    }
                }
                break
            case .SECOND_LEVEL:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMBrowseJobSecondLevelViewController")
                let secondLevelController:JTMBrowseJobSecondLevelViewController = controller as! JTMBrowseJobSecondLevelViewController
                secondLevelController.loadModelBrowseJobSecondLevel(searchMode, code: code!)
                break
            case .TRANSPORTATION:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMTransportaionViewController")
                break
            case .BUSROUTE:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMBusRouteViewController")
                break
            case .COMPANIES:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMCompaniesViewController")
                break
            case .SEARCH_RESULT:
                controller =  storyboard.instantiateViewControllerWithIdentifier("JTMMainSearchResultViewController")
                let searchResultController = controller as! JTMMainSearchResultViewController
                searchResultController.params = searchParams
                searchResultController.isOpenFromSearchMap = (searchMode == .BUSROUTE || searchMode == .TRANSPORTATION) ? true : false
                break
        }
        
        if(controller != nil){
            if(title != nil){
                controller?.title = title;
            }
            navigationController.pushViewController(controller!, animated: true)
        }
    }
    
    //Connect Service & Manage Model
    func connectService() {
        
        let serviceController = JTMServiceController()
        let url = getServiceBrowseJobMain()
        
        serviceController.requestService(self, urlString: url, params: nil) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.convertJsonToModel(jsonObject!)
                
                // Create SubView
                self.createSubViewBrowseJobs()
                self.updateUI()
            }
        }
    }
    
    func convertJsonToModel(resultDict : NSMutableDictionary) {
        let result = resultDict.objectForKey("browsejobs") as! NSMutableDictionary
        browseJobsData = BrowseJobMainPageModel(resultDict: result)
    }
    
    func updateUI() {
        modeJobViewHeightConstraint.constant = (modeJobsView?.systemLayoutSizeFittingSize(UILayoutFittingExpandedSize).height)!
        careerJobViewHeightConstraint.constant = (careerJobsView?.systemLayoutSizeFittingSize(UILayoutFittingExpandedSize).height)!
        ScrollViewContentHeightConstraint.constant = headerViewHeightConstraint.constant + modeJobViewHeightConstraint.constant + careerJobViewHeightConstraint.constant
    }
    
    //Sub View
    func createSubViewBrowseJobs() {
        setUIHeader()
        setUIModeJobs()
        setUICareerJobs()
    }
    
    func setUIHeader() {
        headerJobsView?.setDataWithModel(browseJobsData!)
    }
    
    func setUIModeJobs() {
        modeJobsView?.setDataModeJobs((browseJobsData?.modeJobsList)!)
    }
    
    func setUICareerJobs() {
        careerJobsView?.setModelCareers((browseJobsData?.careersList)!)
        careerJobsView?.delegate = self
    }
    
    //JTMBrowseJobsModeJobsViewDelegate
    func selectBrowseJobsModeItem(browseJobModeView:JTMBrowseJobsModeJobsView, item : BrowseJobModeModel){
        
        print("item.type :: \(item.type!)")
        let selectedList = careerJobsView?.browseJobCareersCollection?.indexPathsForSelectedItems()
        if(selectedList?.count > 0) {
            for indexPath in (selectedList)! {
                careerJobsView!.deSelectItem(indexPath)
            }
        }
        var type:typeBrowseJob = .HOME
        let title : String? = item.getTilte((applicationDelegate.appSetting?.currentLanguage)!)
        var param: [String:AnyObject] = [:]
        
        switch(item.type!){
        case .BUSINESS:
            type = .FIRST_LEVEL
            print("Industry Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_INDUSTRY)
            break
        case .CATEGORY:
            type = .FIRST_LEVEL
            print("Category Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_CATEGORY)
            break;
        case .LOCATION:
            type = .FIRST_LEVEL
            print("Province Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_PROVINCE)
            break
        case .INDUSTRIAL:
            type = .FIRST_LEVEL
            print("Industrial Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_INDUSTRIAL_ESTATE)
            break;
        case .BUSROUTE:
            type = .BUSROUTE
            print("Bus Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_BUS_ROUTE)
            break
        case .TRANSPORTATION:
            type = .TRANSPORTATION
            print("Metro Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_METRO_SEARCH)
            break
        case .COMPANIES:
            type = .COMPANIES
            print("Companies Selected")
//            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_COMPANIES)
            break
        case .OVERSEAS:
            type = .SEARCH_RESULT
            param = [
                PARAMETER_JOB_TYPE_KEY      : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER,
                PARAMETER_LOCATION_KEY      : DEFAULT_PARAMETER_OVERSEAS
            ]
            
                //PARAMETER_DEFAULT + PARAMETER_OVERSEAS
            break
        default:
            break
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_BROWSE_JOB, action: ACTION_CLICK, label: getLabelNameBrowseJob(item.titleEn!))
        
        JTMBrowseJobViewController.openBrowseJobViewController(self.storyboard!, navigationController: self.navigationController!, type: type , code: item.code,searchParams: param, title: title, searchMode:item.type!,parent:self)
    }
    
    //JTMBrowseJobCareersView delegate
    func onBrowseJobCareersClick(browseJobCareers : JTMBrowseJobCareersView, careerData : BrowseJobCareerJobModel){
        
        print("select Career :: \(careerData.nameEn)")
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_BROWSE_JOB, action: ACTION_CLICK, label: getLabelNameBrowseJob(careerData.nameEn!))
        
        let selectedList = modeJobsView?.modeJobsCollection?.indexPathsForSelectedItems()
        if(selectedList?.count > 0) {
            for indexPath in (selectedList)! {
                modeJobsView!.deSelectItem(indexPath)
            }
        }
        let params = [
            PARAMETER_JOB_TYPE_KEY      : careerData.code!,
            PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER
        ]
        
        //PARAMETER_JOB_TYPE + careerData.code! + PARAMETER_SUB_JOB_TYPE + "All"
        JTMBrowseJobViewController.openBrowseJobViewController(self.storyboard!, navigationController: self.navigationController!, type: .SEARCH_RESULT , code: careerData.code, searchParams:  params)
    }
    
    //Language change
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_BROWSE_JOBS")
        headerJobsView?.reloadData()
        modeJobsView?.textTitle?.text = applicationDelegate.getLocalizeStringForKey("MODE_JOB_TITLE")
        careerJobsView!.titleBrowseJobCareer?.text = applicationDelegate.getLocalizeStringForKey("CAREER_TITLE")
    }
}

