//
//  JTMBrowseJobsHomeView.swift
//  JobThai
//
//  Created by kanya on 1/18/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMBrowseJobsHomeView: UIView, UITableViewDataSource, UITableViewDelegate {
    
    var delegate: JTMBrowseJobHomeViewDelagate?
    var regionList: [BrowseJobHomeRegionsModel]? = [BrowseJobHomeRegionsModel]()
    var specialRegionList: [BrowseJobHomeRegionsModel]? = [BrowseJobHomeRegionsModel]()
    var timer:NSTimer?
    
    @IBOutlet weak var totalJobView:UIView!
    @IBOutlet weak var totalJobTitleLabel: JTMLabel!
    @IBOutlet weak var totalJobLabel: JTMLabel!
    @IBOutlet weak var positionLabel: JTMLabel!
    @IBOutlet weak var regionJobTableView: UITableView!
    @IBOutlet weak var specialRegionTableView: UITableView!
    
    ///initial
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupUi()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupUi()
    }
    
    func setupUi(){
        let view    = loadViewFromNib()
        view.frame  = bounds
        view.autoresizingMask = [.FlexibleWidth , .FlexibleHeight]
        view.backgroundColor = JTM_BROWSEJOB_HOME_BG_COLOR
        
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(onClickNextButton))
        tapGesture.numberOfTapsRequired = 1
        totalJobView.addGestureRecognizer(tapGesture)
        
        addSubview(view)
        
        self.totalJobTitleLabel.text = ""
        self.totalJobLabel.text = ""
        self.positionLabel.text = ""
        
        self.setUpRegionJobTableView()
        self.setUpSpecialRegionTableView()
    }
    
    func loadViewFromNib() -> UIView {
        let nib     = UINib(nibName: "JTMHomeBrowseJobsView", bundle: nil)
        let view    = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        
        return view
    }
    
    //set model & data
    
    func reloadViewData() {
        self.setUpTotalJobView()
        self.regionJobTableView.reloadData()
        self.specialRegionTableView.reloadData()
    }
    
    func setRegionModel(browseJobsHomeObject: BrowseJobsHomeModel) {
        
        self.setUpTotalJobView(browseJobsHomeObject.countAllPosition!)
        
        regionList = browseJobsHomeObject.regions
        self.regionJobTableView.reloadData()
        self.scrollToCenter(regionJobTableView)
        
        specialRegionList = browseJobsHomeObject.specialRegions
        self.specialRegionTableView.reloadData()
        self.scrollToCenter(specialRegionTableView)
    }
    
    func setUpTotalJobView() {
        self.totalJobTitleLabel.text = applicationDelegate.getLocalizeStringForKey("TOTAL_JOB_IN_THAI")
        self.positionLabel.text = applicationDelegate.getLocalizeStringForKey("JOB_UNIT")
    }
    
    func setUpTotalJobView(totalJob: Int) {
        self.totalJobTitleLabel.text = applicationDelegate.getLocalizeStringForKey("TOTAL_JOB_IN_THAI")
        self.totalJobLabel.text = applicationDelegate.numFormatter.stringFromNumber(totalJob)
        self.positionLabel.text = applicationDelegate.getLocalizeStringForKey("JOB_UNIT")
    }
    
    func setUpRegionJobTableView() {
        self.setupTableView(regionJobTableView)
    }
    
    func setUpSpecialRegionTableView() {
        self.setupTableView(specialRegionTableView)
    }
    
    func setupTableView(tableView:UITableView) {
        tableView.dataSource        = self
        tableView.delegate          = self
        tableView.backgroundColor   = UIColor.clearColor()
        tableView.bounces           = false
        tableView.pagingEnabled     = true
        tableView.separatorStyle    = .None
        tableView.showsVerticalScrollIndicator = false
        
        tableView.registerNib(UINib(nibName: "JTMBrowseJobsHomeTableViewCell", bundle: nil), forCellReuseIdentifier: "JTMBrowseJobsHomeTableViewCell")
    }

    func scrollToCenter(tableView: UITableView) {
        
        //Center is 1st item of 3rd copy 
        
        var indexPath:NSIndexPath
        
        if (tableView == regionJobTableView) {
            
            indexPath = NSIndexPath.init(forRow: (regionList?.count)!*2,
                                         inSection: 0)
        } else {
            
            indexPath = NSIndexPath.init(forRow: (specialRegionList?.count)!*2,
                                         inSection: 0)
        }
        
        tableView.scrollToRowAtIndexPath(indexPath,
                                         atScrollPosition: UITableViewScrollPosition.None,
                                         animated: false)
    }
    
    //tableView
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
        if( tableView == regionJobTableView) {
            return (regionList?.count)!*5
        } else {
            return (specialRegionList?.count)!*5
        }
    }
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return self.frame.height
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMBrowseJobsHomeTableViewCell") as! JTMBrowseJobsHomeTableViewCell
        cell.setUpBackground()
        
        var index:Int
        
        if(tableView == regionJobTableView) {
            index = indexPath.row % (regionList?.count)!
            cell.setCellData(regionList![index])
        } else {
            index = indexPath.row % (specialRegionList?.count)!
            cell.setCellData(specialRegionList![index])
        }
        
        return cell
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        var regionItem: BrowseJobHomeRegionsModel
        
        if(tableView == regionJobTableView) {
            regionItem = regionList![indexPath.row%(regionList?.count)!]
        } else {
            regionItem = specialRegionList![indexPath.row%(specialRegionList?.count)!]
        }
        
        delegate?.onBrowseJobsRegionTableClick(tableView, regionItem: regionItem)
    }
    
    //ScrollView
    func scrollViewDidScroll(scrollView: UIScrollView) {
        
        //Mock 5 copy of data turn to 1st item of 2nd copy when reach 1st item of last copy
        //and turn to last item of 4th copy when reach last item of 1st copy
        
        if (scrollView == regionJobTableView) {
            let currentRegionIndexPath = regionJobTableView.indexPathsForVisibleRows![0]

            if (currentRegionIndexPath.row >= (regionList?.count)!*4) {
                
                regionJobTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: regionList!.count, inSection: 0),
                    atScrollPosition: .Top,
                    animated: false)
                
            } else if (currentRegionIndexPath.row < (regionList?.count)!) {
                
                regionJobTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: regionList!.count*4-1, inSection: 0),
                    atScrollPosition: .Top,
                    animated: false)
            }
            
        } else {
            let currentSpecialIndexPath = specialRegionTableView.indexPathsForVisibleRows![0]

            if (currentSpecialIndexPath.row >= (specialRegionList?.count)!*4) {
                
                specialRegionTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: specialRegionList!.count, inSection: 0),
                    atScrollPosition: .Top,
                    animated: false)
                
            } else if (currentSpecialIndexPath.row < (specialRegionList?.count)!) {
                
                specialRegionTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: specialRegionList!.count*4-1, inSection: 0),
                    atScrollPosition: .Top,
                    animated: false)
            }
        }
    }
    
    //auto slide
    func autoSlideRegion() {
        if (regionList?.count > 0) {
            if let currentRegionIndexPath = regionJobTableView.indexPathsForVisibleRows?[0] {
                regionJobTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: currentRegionIndexPath.row+1, inSection: 0),
                                                          atScrollPosition: .Top,
                                                          animated: true)
            }
        }
        
        if(specialRegionList?.count > 0) {
            if let currentSpecialIndexPath = specialRegionTableView.indexPathsForVisibleRows?[0] {
                specialRegionTableView.scrollToRowAtIndexPath(NSIndexPath.init(
                    forItem: currentSpecialIndexPath.row+1, inSection: 0),
                                                              atScrollPosition: .Top,
                                                              animated: true)
            }
        }
    }
    
    //MARK:- Button IBAction
    @IBAction func onClickNextButton() {
        delegate?.onBrowseJobsHomeClick()
    }
    
    @IBAction func onClickRegionItem(sender: AnyObject) {
        onClickTableItem(regionJobTableView)
    }
    
    @IBAction func onClickSpecialRegionItem(sender: AnyObject) {
        onClickTableItem(specialRegionTableView)
    }
    
    func onClickTableItem(tableView:UITableView) {
        if let currentIndex = tableView.indexPathsForVisibleRows?[0] {
            self.tableView(tableView, didSelectRowAtIndexPath: currentIndex)
        }
    }
    
    //timer
    func startTimer() {
        if (timer == nil) {
            timer = NSTimer.scheduledTimerWithTimeInterval(
                5.0,
                target: self,
                selector: #selector(autoSlideRegion),
                userInfo: nil,
                repeats: true)
        }
    }
    
    func stopTimer() {
        timer?.invalidate()
        timer = nil
    }
}

protocol JTMBrowseJobHomeViewDelagate {
    func onBrowseJobsRegionTableClick(regionTableView: UITableView, regionItem: BrowseJobHomeRegionsModel)
    func onBrowseJobsHomeClick()
}

