//
//  JTMCompaniesViewController.swift
//  JobThai
//
//  Created by weerapon on 1/21/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMCompaniesViewController : JTMBaseViewController, UITableViewDelegate, UITableViewDataSource, JTMSearchViewControllerDelegate {
    
    @IBOutlet weak var companiesTableView : UITableView?
    @IBOutlet var searchView : JTMSearchViewController!
    @IBOutlet var searchViewContraintHeight : NSLayoutConstraint!
    @IBOutlet var searchMessageLabel : UILabel?
    
    var listSectionCompanies = [SectionCompanyModel]()
    var listIndex = [[String]]()
    
    var sectionSelect : Int?
    var selectIndex : Int = 0
    var selectName : String!
    var categoryKey : [String] = ["thai1" , "thai2" , "eng" , "numeric"]
    var segmentTitle : [String] = ["ก-น", "บ-ฮ", "A-Z", "0-9"]
    
    var allLetterData = [LetterCompanyModel]()
    var filterResultByLetter = [LetterCompanyModel]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.setUpSearchUI(self.segmentTitle)
        self.selectIndex = 0
        self.searchView.segment.selectedSegmentIndex = self.selectIndex
        self.companiesTableView?.sectionIndexBackgroundColor = UIColor.clearColor()
        self.companiesTableView?.tableFooterView = UIView(frame: CGRectZero)
        connectService()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("Companies viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_COMPANIES)
        
        self.searchMessageLabel?.text = applicationDelegate.getLocalizeStringForKey("SEARCH_NO_RESULT_MESSAGE")
        self.searchMessageLabel?.hidden = true
        
        self.searchView.setNoInternetView(self.noInternetTabView!)
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        self.searchView.searchController.dismissViewControllerAnimated(false, completion: nil)
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- Reachability
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        self.searchView.updateViewWhenReachabilityChange()
    }
    
    override func manageViewAfterInternetNotReachable() {
        super.manageViewAfterInternetNotReachable()
        self.searchView.updateViewWhenReachabilityChange()
    }
    
    //MARK:- On Retry Connection
    override func onRetryConnection() {
        super.onRetryConnection()
        if (listSectionCompanies.count == 0) {
            connectService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (listSectionCompanies.count == 0) {
            setAlphaShowToView(view)
        }
    }

    //MARK: Connect Service
    func connectService() {
        
        let serviceController = JTMServiceController()
        let url = getServiceCompanies()
        
        serviceController.requestService(self, urlString: url, params: nil) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonResult = response!.result.value as? NSMutableDictionary
                self.parseJsonToModel(jsonResult!)
                self.companiesTableView?.reloadData()
            }
        }
    }
    
    func parseJsonToModel(jsonObject : NSMutableDictionary) {
        let companiesObject = jsonObject.objectForKey("companies") as? NSMutableDictionary
        let indexObject = jsonObject.objectForKey("indexedList") as? NSMutableDictionary
        
        //Temp Data and Index
        let tmpListIndexDictionary = NSMutableDictionary()
        let tmpListSectionCompanies = NSMutableDictionary()
        
        for (key,value) in companiesObject! {
            let sectionObjet = SectionCompanyModel(keySectionParam: key as! String,letterObject: value as! NSMutableDictionary)
            
            //Sort LetterList and Keep Data in temp
            sectionObjet.letterList?.sortInPlace({String($0.letterKey!) < String($1.letterKey!)})
            tmpListSectionCompanies.setValue(sectionObjet, forKey: key as! String)
            
            //Keep Value with Key
            tmpListIndexDictionary.setValue((indexObject?.objectForKey(key))! as! [String], forKey: key as! String)
        }
        
        //Sort Data and Index by categoryKey
        for key in categoryKey {
            listSectionCompanies.append((tmpListSectionCompanies.objectForKey(key) as? SectionCompanyModel)!)
            listIndex.append((tmpListIndexDictionary.objectForKey(key) as? [String])!)
            
            //Get All LetterList for Search
            allLetterData = allLetterData + (tmpListSectionCompanies.objectForKey(key) as? SectionCompanyModel)!.letterList!
        }
        
        
    }
    
    //MARK: Filter Function
    func filterContentForSearchText(searchText: String) {
        var resultCount : Int = 0
        self.filterResultByLetter.removeAll()

        //Filter Search Data by Section
        for itemLetter in self.allLetterData {
            var searchLetterResult = [CompanyModel]()
            searchLetterResult = (itemLetter.listCompany?.filter({ (companiesObject : CompanyModel) -> Bool in
                let stringMatch = companiesObject.name?.lowercaseString.containsString(searchText.lowercaseString)
                return stringMatch!
            }))!
            resultCount += searchLetterResult.count
            self.filterResultByLetter.append(LetterCompanyModel(key: itemLetter.letterKey! , list: searchLetterResult))
        }
        
        self.companiesTableView?.hidden = false
        if(resultCount == 0 && searchText != ""){
            self.searchMessageLabel?.hidden = false
            self.companiesTableView?.hidden = true
        } else {
            self.searchMessageLabel?.hidden = true
        }
        self.companiesTableView?.reloadData()
    }
    
    
    // MARK : JTMSearchView Delegate
    func setUpSearchUI(categoryList : [String]){
        searchView.delegate = self
        searchView.setUpSegmentTitle(categoryList)
    }
    
    func searchBarOfJTMSearchViewCancelButtonClicked(searchbar: UISearchBar) {
        self.searchView.searchController.searchBar.text = ""
        self.searchView.hideSegment(self.view, constraintViewHeight: searchViewContraintHeight, status: false)
    }
    
    func searchBarOfJTMSearchViewResultUpdate(searchController: UISearchController) {
        let searchText = searchController.searchBar.text!
        self.filterContentForSearchText(searchText)
    }
    
    func searchBarOfJTMSearchViewBeginEditing(searchBar: UISearchBar) {
        self.searchView.hideSegment(self.view, constraintViewHeight:  searchViewContraintHeight , status: true)
    }
    
    func segmentOfJTMSearchViewIndexChange(index: Int, categoryName: String) {
        selectIndex = index
        selectName = categoryName
        
        self.companiesTableView?.reloadData()

        let scrollTopIndex = NSIndexPath(forRow: 0, inSection: 0)
        self.companiesTableView?.scrollToRowAtIndexPath(scrollTopIndex, atScrollPosition: .Top, animated: false)
    }
    
    /*
    * Mark: Delegate TableView
    */
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        if (self.searchView.searchController.searchBar.text != "") {
            return self.filterResultByLetter.count
        } else {
            if(listSectionCompanies.isEmpty) {
                return 0
            } else {
                return (listSectionCompanies[selectIndex].letterList?.count)!
            }
        }
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if (self.searchView.searchController.searchBar.text != "") {
            return self.filterResultByLetter[section].listCompany!.count
            
        } else {
            if(listSectionCompanies.isEmpty) {
                return 0
            } else {
                let companies = listSectionCompanies[selectIndex].letterList![section]
                return (companies.listCompany?.count)!
            }
        }
    }

    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMCompaniesCell", forIndexPath: indexPath)
        
        // Initialization code set Separator
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
        
        if(self.searchView.searchController.searchBar.text != "") {
            let companies = self.filterResultByLetter[indexPath.section].listCompany![indexPath.row]
            cell.textLabel?.text = companies.name
        } else {
            let companies = listSectionCompanies[selectIndex].letterList![indexPath.section]
            cell.textLabel?.text = companies.listCompany![indexPath.row].name
        }
        
        cell.selectionStyle = .None
        
        return cell
    }
    
    /*
    * Mark: IndexScroll
    */
    func sectionIndexTitlesForTableView(tableView: UITableView) -> [String]? {
        if(self.searchView.searchController.searchBar.text != "") {
            return nil
        } else {
            if listIndex.isEmpty {
                return nil
            } else {
                return listIndex[selectIndex]
            }
        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        let searchResultController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainSearchResultViewController") as? JTMMainSearchResultViewController
        if(self.searchView.searchController.searchBar.text != "") {
            let companies = self.filterResultByLetter[indexPath.section].listCompany![indexPath.row]
            searchResultController?.params = [
                PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                PARAMETER_KEYWORD_KEY               : "\"\(companies.name!)\""
            ]
                
                //PARAMETER_DEFAULT + PARAMETER_KEYWORD + "\"" + companies.name! + "\""
        } else {
            let companies = listSectionCompanies[selectIndex].letterList![indexPath.section].listCompany![indexPath.row]
            searchResultController?.params = [
                PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                PARAMETER_KEYWORD_KEY               : "\"\(companies.name!)\""
            ]
                 
                //PARAMETER_DEFAULT + PARAMETER_KEYWORD + "\"" +  companies.name! + "\""
        }
        searchResultController?.isOpenFromSearchMap = false
        self.navigationController?.pushViewController(searchResultController!, animated: true)
    }
    
    /*
    * Mark: Header Title View
    */
    func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if(self.searchView.searchController.searchBar.text != "") {
            if self.filterResultByLetter[section].listCompany!.count == 0 {
                return 0
            }
        }
        return 40
    }
    
    func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let nib = UINib(nibName: "JTMCompaniesHeaderViewCell", bundle: nil)
        let headerView = nib.instantiateWithOwner(self, options: nil)[0] as! JTMCompaniesHeaderViewCell
        
        if(self.searchView.searchController.searchBar.text != "") {
            let companies = self.filterResultByLetter[section]
            headerView.setCompaniesTableHeaderView(companies.letterKey!, companiesCount: "\((companies.listCompany?.count)!)")
//            headerView.headerTitle?.text = companies.letterKey
//            headerView.headerCompaniesCount?.text = "\((companies.listCompany?.count)!)"
        } else {
            let companies = listSectionCompanies[selectIndex].letterList![section]
//            headerView.headerTitle?.text = companies.letterKey
//            headerView.headerCompaniesCount?.text = "\((companies.listCompany?.count)!)"
            headerView.setCompaniesTableHeaderView(companies.letterKey!, companiesCount: "\((companies.listCompany?.count)!)")
        }
        return headerView
    }
    
    func tableView(tableView: UITableView, didHighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = JTM_SELECTED_CELL_BORDER_COLOR.colorWithAlphaComponent(0.4)
    }
    
    func tableView(tableView: UITableView, didUnhighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = UIColor.clearColor()
    }
    
    func scrollViewWillBeginDragging(scrollView: UIScrollView) {
        self.searchView.searchController.searchBar.resignFirstResponder()
    }
}
