//
//  JTMContactUsViewController.swift
//  JobThai
//
//  Created by weerapon on 2/9/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import MessageUI

class JTMContactUsViewController : JTMBaseViewController , UITableViewDelegate, UITableViewDataSource, MFMailComposeViewControllerDelegate {
    
    var listContact = NSMutableArray()
    
    let THINKNET_LAT = 13.72716600
    let THINKNET_LONG = 100.53137100
    
    @IBOutlet weak var contactUsTable : UITableView?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // register nib
        self.registerNib()
        // set Data
        self.setContactData()
        // set Title
        self.title = applicationDelegate.getLocalizeStringForKey("CONTACT_US_TITLE")
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("Contact us")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_CONTACT_US)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CONTACT_US, action: ACTION_VIEW, label: LABEL_VIEW_CONTACT)
        
        if let selectedIndexPath = contactUsTable?.indexPathForSelectedRow {
            contactUsTable?.deselectRowAtIndexPath(selectedIndexPath, animated: false)
        }
    }
    
    func registerNib() {
        contactUsTable?.registerNib(UINib(nibName: "JTMContactUsViewCell", bundle: nil), forCellReuseIdentifier: "JTMContactUsViewCell")
        contactUsTable?.registerNib(UINib(nibName: "JTMContactUsHeaderViewCell", bundle: nil), forCellReuseIdentifier: "JTMContactUsHeaderViewCell")
    }
    
    func setContactData() {
        if let path = NSBundle.mainBundle().pathForResource("ListContact", ofType: "plist") {
            listContact = NSMutableArray(contentsOfFile: path)!
        }
        
        self.contactUsTable?.rowHeight = UITableViewAutomaticDimension
        self.contactUsTable?.estimatedRowHeight = 44
    }
    
    func openApp(appURL : NSURL = NSURL(string: "")!, webURL : NSURL) {
        if UIApplication.sharedApplication().canOpenURL(appURL) {
            UIApplication.sharedApplication().openURL(appURL)
        } else {
            //redirect to safari because the user doesn't have Application
            UIApplication.sharedApplication().openURL(webURL)
        }
    }
 
    func callPhone() {
        let alert = UIAlertController(title: THNKNET_TEL, message: nil, preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("BUTTON_CANCEL"), style: UIAlertActionStyle.Default, handler: nil))
        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("BUTTON_CALL"), style: .Default, handler: { action in
            if let url = NSURL(string: "tel://\(THNKNET_TEL))") {
                UIApplication.sharedApplication().openURL(url)
            }
        }))
        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    func openMapView() {
        let nameKey = applicationDelegate.appSetting?.currentLanguage == .THAI ? "nameTH" : "nameEN"
        let companyString = listContact[0].objectForKey(nameKey)
        
        let separators      = applicationDelegate.appSetting?.currentLanguage == .THAI ? "จำกัด" : "Ltd."
        var companyArr      = companyString?.componentsSeparatedByString(separators)
        let companyName     = "\(companyArr![0])\(separators)"
        var companyAddrress : String = ""
        var companyAddressList = [String]()
        
        for addressLine in companyArr![1].stringByTrimmingCharactersInSet(NSCharacterSet.whitespaceAndNewlineCharacterSet()).componentsSeparatedByString("\n") {
            companyAddressList.append(addressLine)
            
            companyAddrress += addressLine + " "
        }
        
        
        let location = CLLocationCoordinate2D(latitude: THINKNET_LAT, longitude: THINKNET_LONG)
        
        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        let fullMapViewController = storyboard.instantiateViewControllerWithIdentifier("JTMFullScreenMapViewController") as! JTMFullScreenMapViewController
        fullMapViewController.isNoMap = true
        fullMapViewController.calloutShowType = mapCalloutType.CALLOUT_TYPE_ADDRESS
        fullMapViewController.companyName = companyName
        fullMapViewController.location = location
        fullMapViewController.myUserInfo = ""
        fullMapViewController.companyAddress = companyAddrress
        
        self.navigationController?.pushViewController(fullMapViewController, animated: true)
    }

    func openEmail(index : Int) {
        if((applicationDelegate.appSetting?.isNetworkReachable) == true) {
            if MFMailComposeViewController.canSendMail() {
                let mailComposeViewController = configuredMailComposeViewController(index)
                self.presentViewController(mailComposeViewController, animated: true, completion: nil)
            } else {
                self.showSendMailErrorAlert()
            }
        } else {
            self.showAlertNoNetworkIsReachable()
        }
    }
    
    func configuredMailComposeViewController(index : Int) -> MFMailComposeViewController {
        let mailComposerVC = MFMailComposeViewController()
        mailComposerVC.mailComposeDelegate = self
        
        if index == 2 {
            mailComposerVC.setToRecipients(["iPhone@jobthai.com"])
            mailComposerVC.setSubject("\(applicationDelegate.getLocalizeStringForKey("EMAIL_TO_TECHNICAL_REPORT")) \(DEVICE_NAME)")
            mailComposerVC.setMessageBody("<br><br><br><br><hr><b>Technical Information</b><br>Service Version : \(SERVICE_VERSION)<br>Device-ID : \(applicationDelegate.getDeviceUUID())<br>Platforn : \(PLATFORM) - \(DEVICE_NAME) OS, \(IOS_VERSION) <br>Conection Via : \((reachability?.currentReachabilityString)!) <br>RES' CODE : Not Available<br><br>Contact via Jobthai Mobile v\(version)", isHTML: true)
        } else {
            mailComposerVC.setToRecipients(["sale@jobthai.com"])
            mailComposerVC.setSubject("\(applicationDelegate.getLocalizeStringForKey("EMAIL_TO_SALE_REPORT"))")
            mailComposerVC.setMessageBody("<br>", isHTML: true)
        }
        return mailComposerVC
    }
    
    func showSendMailErrorAlert() {
        self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ERROR_TITLE_EMAIL"), message: applicationDelegate.getLocalizeStringForKey("ERROR_MESSAGE_EMAIL"), completionHandler: nil, actionList: nil)
    }
    
    func setGoogleAnalyticContactUs(index : Int) {
        
        var trackLabel = ""
        
        switch(index) {
        case 1:
            trackLabel = LABEL_TEL
            break
        case 2:
            trackLabel = LABEL_EMAIL_SUPPORT
            break
        case 3:
            trackLabel = LABEL_EMAIL_SALE
            break
        case 4:
            trackLabel = LABEL_MAPMAGIC
            break
        case 5:
            trackLabel = LABEL_FACEBOOK
            break
        case 6:
            trackLabel = LABEL_TWITTER
            break
        case 7:
            trackLabel = LABEL_LINKEDIN
            break
        case 8 :
            trackLabel = LABEL_INSTAGRAM
            break
        case 9 :
            trackLabel = LABEL_YOUTUBE
        case 10 :
            trackLabel = LABEL_PINTEREST
        default:
            trackLabel = LABEL_WEB
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CONTACT_US, action: ACTION_CLICK, label: trackLabel)
    }
    
    // MARK: MFMailComposeViewControllerDelegate
    func mailComposeController(controller: MFMailComposeViewController, didFinishWithResult result: MFMailComposeResult, error: NSError?) {
        controller.dismissViewControllerAnimated(true, completion: nil)
    }
    
    // MARK : Delegate Table View
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        switch(indexPath.row) {
        case 0:
            break
        case 1:
            callPhone()
            break
        case 2:
            openEmail(indexPath.row)
            break
        case 3:
            openEmail(indexPath.row)
            break
        case 4:
            openMapView()
            break
        case 5:
            let appURL = NSURL(string: APP_FB)
            let webURL = NSURL(string: URL_FB)
            openApp(appURL!, webURL: webURL!)
            break
        case 6:
            // let appURL = NSURL(string: "gplus://plus.google.com/+jobthai")
//            let webURL = NSURL(string: URL_GOOGLE_PLUS)
//            openApp(webURL: webURL!)
            let appURL = NSURL(string: APP_TWITTER)
            let webURL = NSURL(string: URL_TWITTER)
            openApp(appURL!, webURL: webURL!)
            break
        case 7:
            let appURL = NSURL(string: APP_LINKEDIN)
            let webURL = NSURL(string: URL_LINKEDIN)
            openApp(appURL!, webURL: webURL!)
            break
        case 8 :
            let appURL = NSURL(string: APP_INSTAGRAM)
            let webURL = NSURL(string: URL_INSTAGRAM)
            openApp(appURL!, webURL: webURL!)
            break
        case 9 :
            let appURL = NSURL(string: APP_YOUTUBE)
            let webURL = NSURL(string: URL_YOUTUBE)
            openApp(appURL!, webURL: webURL!)
        case 10 :
            let appURL = NSURL(string: APP_PINTEREST)
            let webURL = NSURL(string: URL_PINTEREST)
            openApp(appURL!, webURL: webURL!)
        default:
            let webURL = NSURL(string: URL_JOBTHAI)
            openApp(webURL: webURL!)
        }
        
        setGoogleAnalyticContactUs(indexPath.row)
        
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return listContact.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let item = listContact[indexPath.row] as? NSMutableDictionary
        let nameKey = (applicationDelegate.appSetting?.currentLanguage == .THAI) ? "nameTH" : "nameEN"
        
        if(indexPath.row == 0) {
            let cell = tableView.dequeueReusableCellWithIdentifier("JTMContactUsHeaderViewCell", forIndexPath: indexPath) as? JTMContactUsHeaderViewCell
            let pathIcon = item!.objectForKey("iconName") as! String
            let title = item!.objectForKey(nameKey) as? String
            cell?.setCellHeader(title!, pathIcon: pathIcon)
            return cell!
        } else {
            let cell = tableView.dequeueReusableCellWithIdentifier("JTMContactUsViewCell", forIndexPath: indexPath) as? JTMContactUsViewCell
            let pathIcon = item!.objectForKey("iconName") as! String
            let title = item!.objectForKey(nameKey) as? String
            cell?.setDataCell(title!, pathIcon: pathIcon)
//            if indexPath.row == 1 {
//                cell?.titleCell?.textColor = JTM_LINK_COLOR
//            }
            setSelectedBackgroundView(cell!)
            return cell!
        }
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
}
