//
//  JTMFavoriteJobCollectionViewCell.swift
//  JobThai
//
//  Created by kanya on 5/13/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import MHVideoPhotoGallery

class JTMFavoriteJobCollectionViewCell: UICollectionViewCell {
    
    let CELL_BORDER_WIDTH : CGFloat = JTM_CELL_BORDER_HEIGHT//2
    let CELL_SELECTED_BORDER_WIDTH : CGFloat = 2.5
    let BORDER_COLOR = JTM_CELL_SEPERATOR_COLOR//JTM_BROWSEJOB_HOME_BG_COLOR
    
    let SELECTED_COLOR = JTM_HIGHLIGHT_COLOR
    let NORMAL_COLOR = UIColor.whiteColor()
    
    @IBOutlet weak var sequenceLabel: UILabel!
    @IBOutlet weak var dateLabel: UILabel!
    @IBOutlet weak var datePrefixLabel: UILabel!
    @IBOutlet weak var companyLogoImageView: UIImageView!
    @IBOutlet weak var jobTitleLabel: UILabel!
    @IBOutlet weak var companyNameLabel: UILabel!
    @IBOutlet weak var closedJobImageView: UIImageView!
    @IBOutlet weak var workLocationLabel: UILabel!
    @IBOutlet weak var salaryLabel: UILabel!
    @IBOutlet weak var removeFavButton: UIButton!
    @IBOutlet weak var salaryImage: UIImageView!
    @IBOutlet weak var bottomBorderView: UIView!
    @IBOutlet weak var topBorderView: UIView!
    @IBOutlet weak var topBorderHeightConstraint: NSLayoutConstraint!
    @IBOutlet weak var bottomBorderHeightConstraint: NSLayoutConstraint!
    
    var index : Int?
    var delegate : JTMFavoriteJobCollectionViewCellDelegate?
    
    override func awakeFromNib() {
        setCellBorder()
//        addGestureToCellButton()
    }
    
    //MARK:- Set Border
    func setCellBorder() {
//        self.layer.borderColor = BORDER_COLOR.CGColor
//        self.layer.borderWidth = CELL_BORDER_WIDTH
        
        
        //top border
        self.topBorderView.backgroundColor = BORDER_COLOR
        self.topBorderHeightConstraint.constant = CELL_BORDER_WIDTH
//
//        //bottom border
        self.bottomBorderView.backgroundColor = BORDER_COLOR
        self.bottomBorderHeightConstraint.constant = CELL_BORDER_WIDTH
    }
    
    func setBorderWidth(isSelectedCell: Bool) {
        self.layer.borderWidth  = isSelectedCell ? CELL_SELECTED_BORDER_WIDTH : CELL_BORDER_WIDTH
    }
    
    //MARK:- Set Cell Data
    func setCellData(index: Int, jobItem: FavoriteJobItem) {
        self.index = index
        self.topBorderHeightConstraint.constant = (index == 0) ? CELL_BORDER_WIDTH : 0
        self.sequenceLabel.text = "\(index+1)"
        self.dateLabel.text = jobItem.date
        self.datePrefixLabel.text = applicationDelegate.getLocalizeStringForKey("FAVORITE_DATE_PREFIX")
        self.jobTitleLabel.text = jobItem.jobTitle
        self.companyNameLabel.text = jobItem.companyName
        
        self.salaryLabel.text = jobItem.salary
        self.salaryLabel.hidden = (jobItem.salary == "")
        self.salaryImage.hidden = (jobItem.salary == "")
        
        self.workLocationLabel.text = (jobItem.countryCode == "TH") ? jobItem.province : jobItem.national
        
        //Company Logo
        setCompanyLogoImage(jobItem.companyLogo)
        
        //Closed Job
        setClosedJobImage(jobItem.onlineStatus! == "1")
        
        //Favorite button
        setFavoriteButton()
    }
    
    func setCompanyLogoImage(logoImageUrl : NSURL?) {
        if (logoImageUrl != nil) {
            self.companyLogoImageView.hidden = false
            self.companyLogoImageView.sd_setImageWithURL(logoImageUrl, placeholderImage: UIImage())
        } else {
            self.companyLogoImageView.hidden = true
        }
    }
    
    func setClosedJobImage(isJobAvailableForApply: Bool) {
        self.closedJobImageView.hidden = isJobAvailableForApply
        self.closedJobImageView.image = UIImage(named: applicationDelegate.getLocalizeStringForKey("CLOSED_JOB_IMAGE"))
        self.closedJobImageView.accessibilityIdentifier = isJobAvailableForApply ? ID_FAVORITE_AVAILABLE_JOB_IMAGE : ID_FAVORITE_CLOSED_JOB_IMAGE
    }
    
    //MARK:- Set Favorite Button
    func setFavoriteButton() {
        //        self.removeFavButton.titleLabel?.text = applicationDelegate.getLocalizeStringForKey("REMOVE_FAVORITE")
//        setupFavoriteButtonFontSize()
        self.removeFavButton.imageView?.contentMode = UIViewContentMode.ScaleAspectFit
//        self.removeFavButton.setTitle(applicationDelegate.getLocalizeStringForKey("REMOVE_FAVORITE"), forState: .Normal)
    }
    
//    func setupFavoriteButtonFontSize() {
//        var size : CGFloat = 0
//        switch screenType {
//        case .SCREEN_3_5_INCH :
//            size = 14.0
//            break
//        case .SCREEN_4_0_INCH :
//            size = 14.0
//            break
//        case .SCREEN_4_7_INCH :
//            size = 14.0
//            break
//        case .SCREEN_5_5_INCH :
//            size = 16
//            break
//        default :
//            size = 14.0
//            break
//        }
//        
//        self.removeFavButton.titleLabel?.font = self.removeFavButton.titleLabel?.font.fontWithSize(size)
//    }
    
    @IBAction func onClickRemoveFavButton(sender: AnyObject) {
        delegate?.onClickRemoveFavoriteButton(self, index: index!)
    }
    
    // MARK:- Add Gesture
//    func addGestureToCellButton() {
//        let longTapGesture = UILongPressGestureRecognizer(target: self, action: #selector(handleTap(_:)))
//        longTapGesture.minimumPressDuration = 0
//        addFavoriteButtonView.addGestureRecognizer(longTapGesture)
//    }
    
    func onClickButton() {
        delegate?.onClickRemoveFavoriteButton(self, index: index!)
    }
    
//    func handleTap(tap:UITapGestureRecognizer) {
//        
//        if tap.state == .Began {
//            print("Began")
//            addFavoriteButtonView.backgroundColor = SELECTED_COLOR
//        } else if tap.state == .Cancelled {
//            print("Cancelled")
//        } else if tap.state == .Changed {
//            print("Changed")
//        } else if tap.state == .Failed {
//            print("Failed")
//        } else if tap.state == .Possible {
//            print("Possible")
//        }  else if tap.state == .Ended {
//            print("Ended")
//            addFavoriteButtonView.backgroundColor = NORMAL_COLOR
//            self.onClickButton()
//        }
//    }
}

protocol JTMFavoriteJobCollectionViewCellDelegate {
    func onClickRemoveFavoriteButton(cell: JTMFavoriteJobCollectionViewCell, index: Int)
}
