//
//  JTMFavoriteJobTableViewCell.swift
//  JobThai
//
//  Created by tayathorn on 8/17/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import MHVideoPhotoGallery

class JTMFavoriteJobTableViewCell: UITableViewCell {
    
    @IBOutlet weak var sequenceLabel: UILabel!
    @IBOutlet weak var dateLabel: UILabel!
    @IBOutlet weak var datePrefixLabel: UILabel!
    @IBOutlet weak var companyLogoImageView: UIImageView!
    @IBOutlet weak var jobTitleLabel: UILabel!
    @IBOutlet weak var companyNameLabel: UILabel!
    @IBOutlet weak var closedJobImageView: UIImageView!
    @IBOutlet weak var workLocationLabel: UILabel!
    @IBOutlet weak var salaryLabel: UILabel!
    @IBOutlet weak var removeFavButton: UIButton!
    @IBOutlet weak var salaryLogo: UIImageView!
    
    var index : Int?
    var delegate : JTMFavoriteJobTableViewCellDelegate?
    
    override func awakeFromNib() {
        super.awakeFromNib()
    }
    
    //MARK:- Set Cell Data
    func setCellData(index: Int, jobItem: FavoriteJobItem) {
        self.index = index
        self.sequenceLabel.text = "\(index+1)"
        self.dateLabel.text = jobItem.date
        self.datePrefixLabel.text = applicationDelegate.getLocalizeStringForKey("FAVORITE_DATE_PREFIX")
        self.jobTitleLabel.text = jobItem.jobTitle
        self.companyNameLabel.text = jobItem.companyName
        
        self.salaryLabel.text = jobItem.salary
        self.salaryLogo.hidden = (jobItem.salary == "")
        
        self.workLocationLabel.text = (jobItem.countryCode == "TH") ? jobItem.province : jobItem.national
        
        //Company Logo
        setCompanyLogoImage(jobItem.companyLogo)
        
        //Closed Job
        setClosedJobImage(jobItem.onlineStatus! == "1")
        
        //Favorite button
        setFavoriteButton()
    }
    
    func setCompanyLogoImage(logoImageUrl : NSURL?) {
        if (logoImageUrl != nil) {
            self.companyLogoImageView.hidden = false
            self.companyLogoImageView.sd_setImageWithURL(logoImageUrl, placeholderImage: UIImage())
        } else {
            self.companyLogoImageView.hidden = true
        }
    }
    
    func setClosedJobImage(isJobAvailableForApply: Bool) {
        self.closedJobImageView.hidden = isJobAvailableForApply
        self.closedJobImageView.image = UIImage(named: applicationDelegate.getLocalizeStringForKey("CLOSED_JOB_IMAGE"))
        self.closedJobImageView.accessibilityIdentifier = isJobAvailableForApply ? ID_FAVORITE_AVAILABLE_JOB_IMAGE : ID_FAVORITE_CLOSED_JOB_IMAGE
    }
    
    //MARK:- Set Favorite Button
    func setFavoriteButton() {
//        self.removeFavButton.titleLabel?.text = applicationDelegate.getLocalizeStringForKey("REMOVE_FAVORITE")
        setupFavoriteButtonFontSize()
        self.removeFavButton.imageView?.contentMode = UIViewContentMode.ScaleAspectFit
        self.removeFavButton.setTitle(applicationDelegate.getLocalizeStringForKey("REMOVE_FAVORITE"), forState: .Normal)
    }
    
    func setupFavoriteButtonFontSize() {
        var size : CGFloat = 0
        switch screenType {
        case .SCREEN_3_5_INCH :
            size = 12.0
            break
        case .SCREEN_4_0_INCH :
            size = 12.0
            break
        case .SCREEN_4_7_INCH :
            size = 12.0
            break
        case .SCREEN_5_5_INCH :
            size = 14
            break
        default :
            size = 12.0
            break
        }
        
        self.removeFavButton.titleLabel?.font = self.removeFavButton.titleLabel?.font.fontWithSize(size)
    }
    
    @IBAction func onClickRemoveFavButton(sender: AnyObject) {
        delegate?.onClickRemoveFavoriteButton(self, index: index!)
    }
}

protocol JTMFavoriteJobTableViewCellDelegate {
    func onClickRemoveFavoriteButton(cell: JTMFavoriteJobTableViewCell, index: Int)
}
