//
//  JTMFavoriteJobsTableViewController.swift
//  JobThai
//
//  Created by tayathorn on 8/17/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
//import UITableView_FDTemplateLayoutCell   // Uncomment and add pod to use this lib

class JTMFavoriteJobsTableViewController : JTMBaseMenuViewController, UITableViewDelegate, UITableViewDataSource, JTMFavoriteJobTableViewCellDelegate{
    
    let FAVORITE_CELL_IDENTIFIER = "JTMFavoriteJobTableViewCell"
    let FOOTER_CELL_IDENTIFIER = "JTMSearchResultListFooterTableViewCell"
    
    let BACK_TO_TOP_HIDE_ALPHA : CGFloat = 0
    let BACK_TO_TOP_SHOW_ALPHA : CGFloat = 0.7
    let START_BACK_TO_TOP_INDEX : Int = 10
    
    var isBrowseToJobDetail = false
    var isFavoriteListChanged = false
    var selectedJobCode : Int?
    var isHasLoadMore = false
    var favoriteJobsData : FavoriteJobModel?
    var noResultView : JTMResumeNoResultView?
    
    @IBOutlet weak var autoRemoveNoticeLabel: UILabel!
    @IBOutlet weak var backToTopButton: UIButton!
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initObserver()
        initQuickSearchNavigationItemButton()
        initNoResultView()
        registerNibForTableView()
        clearControllerData()
        setAccessibilityIdentifier()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_FAVORITE_JOBS)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FAVORITE_JOB, action: ACTION_VIEW, label: LABEL_VIEW_FAV_JOB)
        
        setShowNoResultView(false)
        noResultView?.frame = self.view.bounds
        
        isBrowseToJobDetail = false
        isFavoriteListChanged = false
    }
    
//    override func viewDidAppear(animated: Bool) {
//        tableView.reloadData()
//    }
    
    override func viewDidDisappear(animated: Bool) {
        super.viewDidDisappear(animated)
        
        if (isBrowseToJobDetail == false) {
            clearControllerData()
        }
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
        isCanUseAfterLogInOnly = true
    }
    
    override func clearControllerData() {
        isLoadingService = false
        selectedJobCode = nil
        isHasLoadMore = false
        favoriteJobsData = nil
        tableView.reloadData()
        hideAllHeaderLabel()
        setShowBackToTopButton(false)
        setShowNoResultView(false)
    }
    
    //MARK:- Manage Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if (applicationDelegate.appSetting?.isLogin)!
            && ((favoriteJobsData == nil) || tableView.visibleCells.contains({$0.isKindOfClass(JTMSearchResultListFooterTableViewCell)})) {
            callService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (favoriteJobsData == nil) {
            setAlphaShowToView(view)
        }
    }
    
    //MARK:- No Result View
    func initNoResultView() {
        noResultView = JTMResumeNoResultView(frame: self.view.bounds)
        noResultView!.setNoResultForFavoriteJob()
        self.view.addSubview(noResultView!)
    }
    
    func setShowNoResultView(isShow: Bool) {
        noResultView?.hidden = !isShow
    }
    
    //MARK:- Observer
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(onFavoriteListChanged), name: NOTI_FAVORITE_LIST_CHANGED, object: nil)
    }
    
    func onFavoriteListChanged() {
        if (checkIfControllerIsShowing() == false) {
            clearControllerData()
            isFavoriteListChanged = true
        }
    }
    
    //MARK:- Language changed observer
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_FAVORITE_JOB")
        noResultView?.setNoResultForFavoriteJob()
    }
    
    //MARK:- Call Service
    func callService() {
        let serviceController = JTMServiceController()
        let url = getServiceFavoriteJob()
        let indexStart = favoriteJobsData == nil ? 0 : (favoriteJobsData?.favoriteJobList.count)!
        let params = [
            "indexStart" : indexStart
        ]
        
        serviceController.requestService(self, urlString: url, params: params, isPresentLoadingIndicator: (indexStart == 0)) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let responseJSON = response!.result.value as! NSMutableDictionary
                if (responseJSON.objectForKey("status") as! String == "ok") {
                    self.parseJSONToObject(responseJSON)
                    self.setShowNoResultView(self.favoriteJobsData?.countAllJobs == 0)
                    self.isHasLoadMore = self.favoriteJobsData?.favoriteJobList.count < self.favoriteJobsData?.countAllJobs
                    
//                    self.performSelector(#selector(self.goToTop), withObject: nil, afterDelay: 3)
                    
                    self.tableView.reloadData()
                } else {
                    self.onConnectionError(nil)
                }
            }
        }
    }
    
    func goToTop() {
        self.tableView.reloadData()
    }
    
    //MARK:- Controller's Data
    func parseJSONToObject(favoriteJobJSONData: NSMutableDictionary) {
        if (favoriteJobsData == nil) {
            favoriteJobsData = FavoriteJobModel(favoriteJobsJSONData: favoriteJobJSONData)
            self.setHeaderLabel((self.favoriteJobsData?.countAllJobs)!) //Label is all position, so just set on first time only
        } else {
            favoriteJobsData?.appendNewPageToFavoriteJobList(favoriteJobJSONData)
        }
    }
    
    func getNumberOfCellForTableView() -> Int {
        
        if (favoriteJobsData == nil) {
            return 0
        }
        
        let favoriteJobList = favoriteJobsData?.favoriteJobList
        if (isHasLoadMore) {
            return (favoriteJobList?.count)! + 1
        } else {
            return (favoriteJobList?.count)!
        }
    }
    
    //MARK:- TableView
    func registerNibForTableView() {
        let favoriteCellNib = UINib(nibName: FAVORITE_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(favoriteCellNib, forCellReuseIdentifier: FAVORITE_CELL_IDENTIFIER)
        
        let footerCellNib = UINib(nibName: FOOTER_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(footerCellNib, forCellReuseIdentifier: FOOTER_CELL_IDENTIFIER)
        
//        tableView.estimatedRowHeight = 44.0
//        tableView.rowHeight = UITableViewAutomaticDimension
//        tableView.sectionFooterHeight = 0//.tableFooterView?.frame = CGRect.zero
        tableView.tableFooterView = UIView()
//        tableView.tableFooterView?.frame
    }
    
    //MARK:- TableView Datasource
    func numberOfSectionsInCollectionView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return getNumberOfCellForTableView()
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {

        if (indexPath.row + 1 == getNumberOfCellForTableView() && isHasLoadMore) {
            let loadMoreCell = tableView.dequeueReusableCellWithIdentifier(FOOTER_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMSearchResultListFooterTableViewCell
            loadMoreCell?.accessibilityIdentifier = ID_FAVORITE_LOAD_MORE
            print("loadmore")
            return loadMoreCell!
        } else {
            let jobItem = (favoriteJobsData?.favoriteJobList[indexPath.row])!
            let cell = tableView.dequeueReusableCellWithIdentifier(FAVORITE_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMFavoriteJobTableViewCell
            cell!.delegate = self
            cell?.setCellData(indexPath.row, jobItem: jobItem)
            print("index :: \(indexPath.row) size : \(cell?.frame.size)\n")
            return cell!
            
        }
    }
//    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        print("index \(indexPath.row) item \(self.favoriteJobsData?.favoriteJobList.count)")
        
        
            
            if (indexPath.row + 1 == getNumberOfCellForTableView() && self.isHasLoadMore) {
                return 44
            } else {
                // Uncomment for use dynamic height lib
//                return tableView.fd_heightForCellWithIdentifier(FAVORITE_CELL_IDENTIFIER, cacheByIndexPath: indexPath, configuration: { (var cellObject) in
//                    let jobItem = (self.favoriteJobsData?.favoriteJobList[indexPath.row])!
//                                        let cell = cellObject as! JTMFavoriteJobTableViewCell
//                                        cell.setCellData(indexPath.row, jobItem: jobItem)
//                                        
//                                        cellObject = cell
//                })
                return 200
        }
            
            
            
            
            
        
    }
    
    
    //MARK:- ScrollView Delegate
    func scrollViewDidScroll(scrollView: UIScrollView) {

        let offset : CGPoint = scrollView.contentOffset
        let bounds : CGRect = scrollView.bounds
        let size : CGSize = scrollView.contentSize
        let inset : UIEdgeInsets = scrollView.contentInset
        let y : CGFloat = offset.y + bounds.size.height - inset.bottom;
        let h  : CGFloat = size.height;
        
        // checkToLoadMore
        if(y > h - RELOAD_DISTANCE){
            self.callService()
        }
        
        
        self.backToTopButton?.alpha = self.BACK_TO_TOP_SHOW_ALPHA
//         checkToSetBackToTop()
    }
    
    func checkToSetBackToTop() {
        let visibleIndex = tableView.indexPathsForVisibleRows
        print("visibleIndex :: \(visibleIndex?.last?.row)")
        
        if ((visibleIndex?.last?.row)!+1 >= self.START_BACK_TO_TOP_INDEX && tableView.contentOffset.y != 0) {
            self.backToTopButton?.alpha = self.BACK_TO_TOP_SHOW_ALPHA
        } else {
            self.backToTopButton?.alpha = self.BACK_TO_TOP_HIDE_ALPHA
        }
    }
    
    
    //MARK:- Back to top
    func setShowBackToTopButton(isShow: Bool) {
        backToTopButton.alpha = isShow ? BACK_TO_TOP_SHOW_ALPHA : BACK_TO_TOP_HIDE_ALPHA
    }
    
    @IBAction func onClickBackToTop(sender: AnyObject) {
//        tableView.setContentOffset(CGPointZero, animated: true)
        
        print("onClickBackToTop")
        
        tableView.scrollToRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), atScrollPosition: .Top, animated: true)
        
//        tableView.scro
    }
    
    //MARK:- HeaderView
    func hideAllHeaderLabel() {
        autoRemoveNoticeLabel.text = ""
    }
    
    func setHeaderLabel(position: Int) {
        autoRemoveNoticeLabel.text = applicationDelegate.getLocalizeStringForKey("JOB_WILL_BE_AUTO_REMOVE")
    }
    
    //MARK:- Identifier for UITest
    func setAccessibilityIdentifier() {
        tableView.accessibilityIdentifier = ID_FAVORITE_JOB_TABLEVIEW
        noResultView?.accessibilityIdentifier = ID_FAVORITE_NO_RESULT
    }
    
    //MARK:- Favotire Cell Delegate
    func onClickRemoveFavoriteButton(cell: JTMFavoriteJobTableViewCell, index: Int) {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FAVORITE_JOB, action: ACTION_CLICK, label: LABEL_REMOVE_FAV_JOB)
        
        let cancelAction = UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("CANCEL"),
                                         style: UIAlertActionStyle.Cancel,
                                         handler: nil)
        
        self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_CONFIRM_REMOVE_FAVORITE"),
                               message: applicationDelegate.getLocalizeStringForKey("MESSAGE_CONFIRM_REMOVE_FAVORITE"),
                               completionHandler: { (UIAlertAction) -> Void in
                                self.callServiceRemoveFavorite(index)
            },
                               actionList: [cancelAction])
    }
    
    func callServiceRemoveFavorite(index: Int) {
        let jobItem = favoriteJobsData?.favoriteJobList[index]
        applicationDelegate.appSetting?.callServiceRemoveFavoriteForJobCode(self, jobCode: (jobItem?.jobCode)!, completion: { () -> Void in
            self.removeJobFromFavoriteList(index)
            self.setShowNoResultView((self.favoriteJobsData?.countAllJobs)! == 0)
//            self.checkToLoadMore()
        })
    }
    
    func removeJobFromFavoriteList(index: Int) {
        favoriteJobsData?.favoriteJobList.removeAtIndex(index)
        favoriteJobsData?.countAllJobs -= 1
        setHeaderLabel((self.favoriteJobsData?.countAllJobs)!)
        tableView.reloadData()
    }
}
