//
//  JTMIndustrialSearchViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/26/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMIndustrialSearchViewController: JTMBaseSearchViewController {
    
    enum industrialSearchCellIndex: Int {
        case INDUSTRIAL_CATEGORY
        case INDUSTRIAL_INDUSTRIALESTATE
        case INDUSTRIAL_KEYWORD
        case INDUSTRIAL_FOOTER
        case count
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func isHasAllRequireFilterListValue() -> Bool {
        if (applicationDelegate.appSetting?.searchFilterIndustrialList.count > 0) {
            return super.isHasAllRequireFilterListValue()
        }
        
        return false
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("IndustrialSearch  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_SEARCH_INDUSTRIAL_ESTATE)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE, action: ACTION_VIEW, label: LABEL_FROM_SEARCH_INDUSTRAIL_ESTATE)
        
        self.tableView.reloadData()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    //MARK: ---
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_INDUSTRIAL_ESTATES")
    }
    
    //MARK:- Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if (applicationDelegate.appSetting?.searchFilterIndustrialList.count == 0) {
            loadIndustrialListFromService()
        }
    }
    
    override func onConnectionError(errorRequestUrl: String?) {
        if (errorRequestUrl! == getServiveSearchFilter(.INDUSTRIALESTATE))
            && (applicationDelegate.appSetting?.searchFilterIndustrialList.count > 0) {
                return
        }
        
        super.onConnectionError(errorRequestUrl)
    }
    
    //MARK:- Industrial Filter
    func getKeywordIndustrialFromSaveState() -> String {
        let selectedIndustrial = parentController?.searchFilterSaveState?.selectedIndustrial
        let selectedSubIndustrial = parentController?.searchFilterSaveState?.selectedSubIndustrial
        
        return getKeywordFromSaveState(selectedIndustrial, subFilter: selectedSubIndustrial, defaultKeyword: DEFAULT_KEYWORD_INDUSTRIAL)
    }
    
    func getIndustrialParameterFromSaveState() -> String {
        if let selectedSubIndustrial = parentController?.searchFilterSaveState?.selectedSubIndustrial {
            return selectedSubIndustrial.code!
        } else if let selectedIndustrial = parentController?.searchFilterSaveState?.selectedIndustrial {
            return selectedIndustrial.code!
        } else {
            return DEFAULT_KEYWORD_FOR_NIL_TYPE
        }
    }

    //MARK: Delegate Update Save State : Selected IndustrialEstate
    func saveStateupdateSelectedIndustrialEstate(saveData: AnyObject?) {
        if(saveData?.isKindOfClass(SearchFilterCategory) == true) {
            let selectedIndustrialEstate = saveData as? SearchFilterCategory
            parentController?.searchFilterSaveState?.selectedIndustrial = selectedIndustrialEstate
            
            if (selectedIndustrialEstate?.hasSub == false) {
                parentController?.searchFilterSaveState?.selectedSubIndustrial = nil
            }
            
        } else if(saveData?.isKindOfClass(SearchFilterSubCategory) == true) {
            let selectedSubIndustrialEstate = saveData as? SearchFilterSubCategory
            parentController?.searchFilterSaveState?.selectedSubIndustrial = selectedSubIndustrialEstate
        }
        
        tableView.reloadRowsAtIndexPaths([NSIndexPath(forRow: industrialSearchCellIndex.INDUSTRIAL_INDUSTRIALESTATE.rawValue, inSection: 0)], withRowAnimation: .None)
    }
    
    //MARK:- Call Service
    override func loadRequiredFilterListFromService() {
        super.loadRequiredFilterListFromService()
        loadIndustrialListFromService()
    }
    
    func loadIndustrialListFromService() {
        let lastUpdate = (appSetting?.searchFilterIndustrialLastUpdate)!
        let urlIndustrialEstates = getServiveSearchFilter(SearchFilterCategoryType.INDUSTRIALESTATE)
        
        loadSearchFilterListFromService(urlIndustrialEstates, lastupdate: lastUpdate) { (jsonObject) -> Void in
            self.appSetting?.searchFilterIndustrialLastUpdate = (jsonObject.objectForKey("timestamps") as? Int)!
            let itemList = jsonObject.objectForKey("industrials") as? NSArray

            self.appSetting?.searchFilterIndustrialList.removeAll()
            for item in itemList! {
                self.appSetting?.searchFilterIndustrialList.append(SearchFilterCategory(item: item as! NSMutableDictionary))
            }
        }
    }
    
    //MARK:- TableView
    override func configureCellForRowAtIndexPath(cell: JTMSeachFilterTableViewCell, indexPath: NSIndexPath) -> UITableViewCell {
        
        setSelectedBackgroundView(cell)
        
        switch (industrialSearchCellIndex(rawValue: indexPath.row))! {
            case .INDUSTRIAL_CATEGORY:
                configureJobCategoryCell(cell)
                break
            case .INDUSTRIAL_INDUSTRIALESTATE:
                cell.setDescription(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_INDUSTRIAL_ESTATES"),
                    subTitle: getKeywordIndustrialFromSaveState())
                break
            case .INDUSTRIAL_FOOTER:
                cellFooter = configureCellSearchFooter()
                cellFooter?.setCellSeperatorLineHide(tableView)
                return cellFooter!
            default:
                cellSearch = configureCellSearch()
                return cellSearch!
        }

        return cell
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
//        if (industrialSearchCellIndex(rawValue: indexPath.row))! == .INDUSTRIAL_FOOTER{
            return CELL_SIZE_BIG
//        } else {
//            return CELL_SIZE_SMALL
//        }
    }

    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        print("Click\(indexPath.row)")
        
        switch (industrialSearchCellIndex(rawValue: indexPath.row))! {
            case .INDUSTRIAL_CATEGORY:
                openSearchFilter(SearchFilterCategoryType.CATEGORY)
                break
            case .INDUSTRIAL_INDUSTRIALESTATE:
                openSearchFilter(SearchFilterCategoryType.INDUSTRIALESTATE)
                break
            default:
                break
        }

        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func setSelectedBackgroundView(cell: JTMSeachFilterTableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
 
    //MARK:- On Search
    override func footerOnClickSearch() {
        
//        var action = ""
        
        let jobType = getJobTypeParamaterFromSaveState()
        let subType = getSubJobTypeParamaterFromSaveState()
        let keyword = getKeywordParameterFromSaveState()
        let industrial = getIndustrialParameterFromSaveState()
        let disability = getDisabilitySaveState().hashValue
        
        print("disability :: \(disability)")
        
//        let parameter = "?jobtype=\(jobType)&subtype=\(subType)&keyword=\(keyword)&industrial=\(industrial)&disabled_person=\(disability)"
        
        let parameter: [String:AnyObject] = [
            PARAMETER_JOB_TYPE_KEY          : jobType,
            PARAMETER_SUB_JOB_TYPE_KEY      : subType,
            PARAMETER_KEYWORD_KEY           : keyword,
            PARAMETER_INDUSTRIAL_KEY        : industrial,
            PARAMETER_DISABLED_WELCOME_KEY  : disability
        ]
        
        self.setTrackGoogleAnalyticKeyword(keyword)
        self.setTrackGoogleAnalyticDisability(disability)
        
        openSearchResult(parameter , isOpenMap: false , radiusInMeter: 0)
    }
    
    func setTrackGoogleAnalyticKeyword(keyword : String) {
        var action = ""
        
        if keyword.isEmpty {
            action = ACTION_SEARCH_NON_KEYWORD
        } else {
            action = ACTION_SEARCH_KEYWORD
        }
        
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    func setTrackGoogleAnalyticDisability(disability : Int) {
        var action = ""
        
        if disability == 1 {
            action = ACTION_SEARCH_DISABILITY
        } else {
            action = ACTION_SEARCH_NON_DISABILITY
        }
        
        // Track Google Analytic Event disability
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
}