//
//  JTMJobDetailCompanyViewController.swift
//  JobThai
//
//  Created by weerapon on 3/15/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMJobDetailCompanyViewController : JTMBaseViewController, UITableViewDataSource, UITableViewDelegate, JTMJobDetailCompanyHeaderCellDelegate, JTMFullScreenMapViewDelegate {
    
    let PAGE_INDEX: IndexJobDetailPage = .COMPANY_DETAIL_PAGE
    let TABLE_FOOTER_HEIGHT: CGFloat = 0
    let PIXEL_OF_HIDDEN_HEADER : CGFloat = 5
    let ANIMATION_DURATION = 0.3
    
    let TEXT_CELL_IDENTIFIER = "JTMJobDetailTextTableViewCell"
    let IMAGE_CELL_IDENTIFIER = "JTMJobDetailImageTableViewCell"
    let MAP_CELL_IDENTIFIER = "JTMJobDetailMapTableViewCell"
    let LIST_IMAGE_CELL_IDENTIFIER = "JTMJobDetailCompanyHeaderTableViewCell"
    
    var infoList = [CompanyInfoModel]()
    var companyName: String?
    var companyAddress : String?
    var containerViewController : JTMJobDetailViewController?
    var delegateOnScroll : JTMJobDetailMainForSubPageViewDelegate?
    var jobdetailPageViewController : JTMJobDetailPageViewController?
    var checkOnScrolling : Bool = false
    
    var mapInfoData   : NSMutableDictionary?
    var mapAnnotation : RMAnnotation?
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initTableView()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("JobDetailCompany  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_JOB_DETAIL_COMPANY)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_COMPANY_DETAIL, action: ACTION_VIEW, label: LABEL_VIEW_COMPANY)
        
        checkOnScrolling = false
        
        if (infoList.count > 0 && isPresentedBefore) {
            tableView?.scrollToRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), atScrollPosition: .Top, animated: false)
        }
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    //MARK:- Reachability change update
    override func onRetryConnection() {
        super.onRetryConnection()
        if mapInfoData != nil {  
            NSNotificationCenter.defaultCenter().postNotificationName(OBSERVER_RELOAD_MAP_CELL, object: mapInfoData)
            tableView.reloadData()
        }
    }
    
    override func setControllerFlags() {
        //Need to set flag before call super cause we need it set as true before initnointernetview
        isPresentedInOtherController = true
    }
    
    override func setPresentedBeforeFlag() {
        super.setPresentedBeforeFlag()
        
        if (infoList.count > 0) {
            tableView?.reloadData()
            self.containerViewController?.hideLoading()
        } else if (self.jobdetailPageViewController?.isLoadingService == false) {
            self.containerViewController?.hideLoading()
        }
    }
    
    //MARK:- JobInfo
    func updateData(infoList: [CompanyInfoModel], companyName: String) {
        self.infoList = infoList
        self.companyName = companyName
        
        if (isPresentedBefore) {
            tableView?.reloadData()
            tableView?.scrollToRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), atScrollPosition: .Top, animated: false)
        }
    }
    
    //MARK:- Tableview
    func initTableView() {
        registerNibForJobInfoTableView()
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 44
        tableView.tableFooterView = UIView(frame: CGRect(x: 0, y: 0, width: tableView.frame.width, height: TABLE_FOOTER_HEIGHT))
    }
    
    func registerNibForJobInfoTableView() {
        let headerNib = UINib(nibName: LIST_IMAGE_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(headerNib, forCellReuseIdentifier: LIST_IMAGE_CELL_IDENTIFIER)
        
        let textNib = UINib(nibName: TEXT_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(textNib, forCellReuseIdentifier: TEXT_CELL_IDENTIFIER)
        
        let imageNib = UINib(nibName: IMAGE_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(imageNib, forCellReuseIdentifier: IMAGE_CELL_IDENTIFIER)
        
        let mapNib = UINib(nibName: MAP_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(mapNib, forCellReuseIdentifier: MAP_CELL_IDENTIFIER)
    }
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return isPresentedBefore ? 1 : 0
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return infoList.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let infoForCell = infoList[indexPath.row]
        
        switch (infoForCell.type)! {
            case .LIST_IMAGE_CELL:
                return configureCellForFeatureImage(tableView, infoForCell: infoForCell as! CompanyInfoImageModel)
            case .TEXT_CELL:
                return configureCellForText(tableView, infoForCell: infoForCell as! CompanyInfoTextModel, isDetectForLink: false)
            case .TEXT_DETECT_CELL:
                return configureCellForText(tableView, infoForCell: infoForCell as! CompanyInfoTextModel, isDetectForLink: true)
            case .IMAGE_CELL:
                return configureCellForImage(tableView, infoForCell: infoForCell as! CompanyInfoTextModel)
            case .MAP_CELL:
                let isHasDrawMap = infoList.contains({$0.type == .IMAGE_CELL})
                return configureCellForMap(tableView, infoForCell: infoForCell as! CompanyInfoTextModel, isHasDrawMap: isHasDrawMap)
            default:
                return UITableViewCell()
        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        let infoForCell = infoList[indexPath.row]
        
        print("select :: \(infoForCell.type)")
        
        switch (infoForCell.type)! {
        case .IMAGE_CELL:
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_COMPANY_DETAIL, action: ACTION_CLICK, label: LABEL_DRAW_MAP)
            openImageForCellAtIndex(tableView, indexPath: indexPath, infoForCell: infoForCell as! CompanyInfoTextModel)
            break
        case .MAP_CELL:
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_COMPANY_DETAIL, action: ACTION_CLICK, label: LABEL_MAP_ONLINE)
            openFullMapView(self.mapAnnotation)
            break
        default:
            break
        }
    }
    
    func openImageForCellAtIndex(tableView: UITableView, indexPath:NSIndexPath, infoForCell: CompanyInfoTextModel) {
        let cell = tableView.cellForRowAtIndexPath(indexPath) as! JTMJobDetailImageTableViewCell
        openImageGallery(cell.jobMapImageView, listImageUrl: [NSURL(string: (infoForCell.valueInfo)!)!])
    }
    
    //MARK:- Configure Cell
    func configureCellForText(tableView:UITableView, infoForCell:CompanyInfoTextModel, isDetectForLink: Bool) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(TEXT_CELL_IDENTIFIER) as? JTMJobDetailTextTableViewCell
        cell!.setCellData(infoForCell, isDetectForLink: isDetectForLink)
        
        return cell!
    }
    
    func configureCellForImage(tableView:UITableView, infoForCell:CompanyInfoTextModel) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(IMAGE_CELL_IDENTIFIER) as? JTMJobDetailImageTableViewCell
        cell!.setCellData(infoForCell)
        
        return cell!
    }
    
    func configureCellForMap(tableView:UITableView, infoForCell:CompanyInfoTextModel, isHasDrawMap: Bool) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(MAP_CELL_IDENTIFIER) as? JTMJobDetailMapTableViewCell
        cell!.setCellData(infoForCell, companyName: companyName!, isHasDrawMap: isHasDrawMap)
        saveMapInfoData(infoForCell, companyName: companyName!, isHasDrawMap: isHasDrawMap)
        self.mapAnnotation = cell!.annotation
        return cell!
    }
    
    func configureCellForFeatureImage(tableView:UITableView, infoForCell:CompanyInfoImageModel) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(LIST_IMAGE_CELL_IDENTIFIER) as? JTMJobDetailCompanyHeaderTableViewCell
        cell!.setCellData(infoForCell)
        cell?.delegate = self
        
        return cell!
    }
    
    //MARK: Save Map Info Data
    func saveMapInfoData(infoForCell : CompanyInfoTextModel , companyName : String , isHasDrawMap: Bool) {
        if mapInfoData == nil {
            mapInfoData = NSMutableDictionary()
        } else {
            mapInfoData?.removeAllObjects()
        }
        mapInfoData?.setObject(infoForCell, forKey: MAP_CELL_ANNOTATION_INFO)
        mapInfoData?.setObject(companyName, forKey: MAP_CELL_ANNOTATION_COMPANY_NAME)
        mapInfoData?.setObject(isHasDrawMap, forKey: MAP_CELL_ANNOTATION_IS_HAS_DRAWN_MAP)
    }
    
    // Mark :: Delegate onScroll To hidden and show Company Title
    func scrollViewDidScroll(scrollView: UIScrollView) {
        if (scrollView != tableView) {
            return
        }
        
        let offset : CGPoint = scrollView.contentOffset;
        
        if(offset.y > self.PIXEL_OF_HIDDEN_HEADER && !checkOnScrolling){
            self.checkOnScrolling = true
            dispatch_async(dispatch_get_main_queue()) {
                self.delegateOnScroll?.onScrollView(true ,index: self.PAGE_INDEX.rawValue)
            }
        } else if(offset.y < self.PIXEL_OF_HIDDEN_HEADER  && (!isCollectionViewCantScrollToBottom() || (isCollectionViewCantScrollToBottom() && scrollView.isScrollUp(scrollView)))) {
            self.checkOnScrolling = false
            dispatch_async(dispatch_get_main_queue()) {
                self.delegateOnScroll?.onScrollView(false ,index: self.PAGE_INDEX.rawValue)
            }
        }
    }
    
    func isCollectionViewCantScrollToBottom() -> Bool {
        let collectionViewHeight = tableView?.contentSize.height
        let companyHeaderHeight = containerViewController?.headerStackView?.frame.height
        let segmentHeight = containerViewController?.segmentContainerView.frame.height
        let isCollectionViewHigherThanViewWithHideCompanyHeader = collectionViewHeight <= DEFAULT_FRAME_HEIGHT - segmentHeight!
        let isCollectionViewShorterThanViewWithShowCompanyHeader = collectionViewHeight >= DEFAULT_FRAME_HEIGHT - segmentHeight! - companyHeaderHeight!
        
        return isCollectionViewHigherThanViewWithHideCompanyHeader && isCollectionViewShorterThanViewWithShowCompanyHeader
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    //MARK: Full MapView
    func openFullMapView(annotation: RMAnnotation?) {
        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        let fullMapViewController = storyboard.instantiateViewControllerWithIdentifier("JTMFullScreenMapViewController") as! JTMFullScreenMapViewController
        fullMapViewController.delegate = self
        if(annotation != nil){
            fullMapViewController.annotations = [annotation!]
        }
    
        self.navigationController?.pushViewController(fullMapViewController, animated: true)
    }
    
    //MARK:- List Image Cell Delegate
    func onClickFeatureImage(imageView: UIImageView, listFeatureImage: [NSURL], index: Int) {
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticEvent(CAT_COMPANY_DETAIL, action: ACTION_CLICK, label: LABEL_IMAGE_COMPANY)
        openImageGallery(imageView, listImageUrl: listFeatureImage, presentIndex: index)
    }
    
    func didTapCallout(controller: JTMFullScreenMapViewController) {
        controller.navigationController?.popViewControllerAnimated(false)
    }
}
