//
//  JTMJobDetailMapTableViewCell.swift
//  JobThai
//
//  Created by kanya on 3/30/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMJobDetailMapTableViewCell: UITableViewCell, TNMapViewDelegate {

    @IBOutlet weak var titleLabel: JTMLabel!
    @IBOutlet weak var mapView: TNMapView!
    var annotation : RMAnnotation?
    
    let OBSERVER_RELOAD_MAP_CELL = "OBSERVER_RELOAD_MAP_CELL"
    
    override func awakeFromNib() {
        super.awakeFromNib()
        initSelectedBackground()
        initialMapView()
        initObserver()
    }

    override func layoutSubviews() {
        super.layoutSubviews()
//        reloadMapView()
        if(annotation != nil){
            mapView.setCenterMapToCoordinate((annotation?.coordinate)!, animated: false)
            mapView.zoomToBound(mapView.zoomRect)
        }
//        mapView.zoomToFitAllAnnotationWithOutUserLocation()
    }
    override func setSelected(selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)

        // Configure the view for the selected state
    }
    
    //MARK: Init Observer
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(reloadMapView(_:)), name: OBSERVER_RELOAD_MAP_CELL, object: nil)
    }
    
    //MARK:- Selected background
    func initSelectedBackground() {
        let selectedBackgroundView = UIView(frame: CGRectZero)
        self.selectedBackgroundView = selectedBackgroundView
    }

    //MARK:- Set Cell
    func setCellData(jobInfoForCell: JobInfoModel, companyName: String/*, jobList: [String]*/, isHasDrawMap: Bool) {
        setCellTitleLabel(jobInfoForCell.title!)
//        self.titleLabel.hidden = isHasDrawMap
        addAnnotationToCellMap(jobInfoForCell.valueInfo!, companyName: companyName)//, jobList: jobList)
    }
    
    func setCellData(jobInfoForCell: CompanyInfoTextModel, companyName: String/*, jobList: [String]*/, isHasDrawMap: Bool) {
        setCellTitleLabel(jobInfoForCell.title!)
        self.titleLabel.hidden = isHasDrawMap
        addAnnotationToCellMap(jobInfoForCell.valueInfo!, companyName: companyName)//, jobList: jobList)
    }
    
    func setCellTitleLabel(title: String) {
        self.titleLabel.text = title
    }
    
    func addAnnotationToCellMap(pinInfoString: String, companyName: String/*, jobList: [String]*/) {
//        addAnnotationToMapView(JTMJobDetailMapTableViewCell.getJobCoordinateFromCellData(coordinateString), companyName: companyName, jobList: jobList) 
        let pinData : JobDetailPinInfoModel = JobDetailPinInfoModel(pinInfoString: pinInfoString)
        addAnnotationToMapView(pinData, companyName: companyName)
    }

    //MARK:- convert data to coordinate    
//    static func getJobCoordinateFromCellData(coordinateString: String) -> [Double] {
//        let coordinate = coordinateString.componentsSeparatedByString(" , ")
//        var doubleCoordinate = [Double]()
//        for axis in coordinate {
//            doubleCoordinate.append(Double(axis)!)
//        }
//        return doubleCoordinate
//    }
    
    //MARK:- MapView
    func initialMapView() {
        setMapTileSourceLanguage()
        mapView.delegate = self
    }
    
    func setMapTileSourceLanguage() {
        let mapLanguage = applicationDelegate.appSetting?.currentLanguage == .THAI ? TNMapLanguageThai : TNMapLanguageEnglish
        mapView.setMapTileLanguage(mapLanguage)
    }
    
    func addAnnotationToMapView(coordinateData: JobDetailPinInfoModel, companyName: String/*, jobList: [String]*/) {
        mapView.removeAllAnnotation()
        
        let coordinate = CLLocationCoordinate2DMake(coordinateData.latitude!, coordinateData.longitude!)//(coordinateInDoubleValue[0], coordinateInDoubleValue[1])
//        annotation = mapView.createAnnotationWithJobList(companyName, jobList: jobList, coordinate: coordinate, userInfo: "")
        annotation = mapView.createAnnotationWithAddress(companyName, address: coordinateData.callOutInfo!, coordinate: coordinate, userInfo: "")
        if annotation != nil {
            annotation!.enabled = false
            mapView.addAnnotationsToMapView([annotation!])
            mapView.setUserInteraction(false)
        }
    }
    
    func reloadMapView(notification : NSNotification){
        if mapView.mapViewThinknet == nil {
            self.mapView.reloadMapView()
            if let mapData = notification.object as? NSMutableDictionary{
                let mapInfo = mapData.objectForKey(MAP_CELL_ANNOTATION_INFO)
                let companyName = mapData.objectForKey(MAP_CELL_ANNOTATION_COMPANY_NAME) as? String
                let isHasDrawMap = mapData.objectForKey(MAP_CELL_ANNOTATION_IS_HAS_DRAWN_MAP) as? Bool
                
                if(mapInfo != nil && companyName != nil && isHasDrawMap != nil) {
                    if mapInfo!.isKindOfClass(JobInfoModel) {
                        setCellData(mapInfo as! JobInfoModel, companyName: companyName!, isHasDrawMap: isHasDrawMap!)
                    } else {
                        setCellData(mapInfo as! CompanyInfoTextModel, companyName: companyName!, isHasDrawMap: isHasDrawMap!)
                    }
                    
                }
            }
        }
        setBackgroundMapView()
    }
    
    func setBackgroundMapView() {
        if mapView.mapViewThinknet == nil {
            self.mapView.backgroundColor = JTM_LIGHT_GRAY
            self.mapView.mapViewThinknet?.backgroundColor = JTM_LIGHT_GRAY
        }
    }
}
