//
//  JTMJobDetailPageViewController.swift
//  JobThai
//
//  Created by weerapon on 3/15/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMJobDetailPageViewController : JTMBaseViewController , UIPageViewControllerDataSource, UIPageViewControllerDelegate, JTMJobDetailMainDelegate {
    
    var isNeedToUpdateData : Bool = true
    var companyCode : String?
    var INDEX_PAGE : IndexJobDetailPage?
    var selectedJob : Int?
    var jobCode : Int?  
    var jobdetailData : JobDetailModel?
    var jobDetailMainController : JTMJobDetailViewController?
    var delegateSetMainJobdetail : JTMJobdetailPageViewDelegate?
    
    // Mark :: Instance Object In Page View
    var allPositionViewController : JTMJobDetailAllPositionViewController?
    var companyViewController : JTMJobDetailCompanyViewController?
    var infoViewController : JTMJobDetailInfoViewController?
    
    // Mark :: PageView
    var pageViewController : UIPageViewController?
    var pageTitles : Array<String> = ["ALLJOB", "COMPANIES", "DETAIL"]
    var currentIndex : NSInteger = NSNotFound
    var isWillTransition : Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()

        self.initailObject()
        self.initialPageView()
        self.initialDelegateToChildPageView()
        self.initObserver()
        self.setAccessibilityIdentifier()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        if isWillTransition {
            isWillTransition = false
            let currentPageIndex : NSInteger = getCurrentIndex((self.pageViewController?.viewControllers?.first)!)
            onClickSegment(currentPageIndex)
        }
        
//        if isNeedToUpdateData {
//            self.callService()
//        }
    }
    
    override func setControllerFlags() {
        //Need to set flag before call super cause we need it set as true before initnointernetview
        isPresentedInOtherController = true
    }
    
    //MARK:- On Retry Connection
    override func onRetryConnection() {
        super.onRetryConnection()
        if (jobdetailData == nil) || isNeedToUpdateData {
            jobdetailData = nil
            callService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (jobdetailData == nil) {
            if (view == noInternetView) {
                setAlphaShowToView((jobDetailMainController?.noInternetView)!)
            } else {
                setAlphaShowToView((jobDetailMainController?.connectionErrorView)!)
            }
        }
    }
    
    override func onConnectionSuccess() {
        super.onConnectionSuccess()
        jobDetailMainController?.onConnectionSuccess()
    }
    
    // ----------------------------------------------------------------------- Initital Controller -------------------------------------------------------------------------- //
    
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(userDidUpdateSomeJobStatus), name: JTM_LOGIN_STATE_DID_CHANGE, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(userDidUpdateSomeJobStatus), name: NOTI_FAVORITE_LIST_CHANGED, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(userDidUpdateSomeJobStatus), name: JTM_USER_JUST_APPLIED_JOB, object: nil)
    }
    
    func userDidUpdateSomeJobStatus() {
        isNeedToUpdateData = true
        if (jobDetailMainController?.view.window != nil) {
            onRetryConnection()
        }
    }
    
    func initailObject() {
        allPositionViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMJobDetailAllPositionViewController") as? JTMJobDetailAllPositionViewController
        companyViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMJobDetailCompanyViewController") as? JTMJobDetailCompanyViewController
        infoViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMJobDetailInfoViewController") as? JTMJobDetailInfoViewController
        
        allPositionViewController?.containerViewController = jobDetailMainController
        companyViewController?.containerViewController = jobDetailMainController
        infoViewController?.containerViewController = jobDetailMainController
        
        allPositionViewController?.jobdetailPageViewController = self
        companyViewController?.jobdetailPageViewController = self
        infoViewController?.jobdetailPageViewController = self
    }
    
    func initialDelegateToChildPageView() {
        // company delegate
        self.companyViewController?.delegateOnScroll = self.jobDetailMainController
        
        // allposition delegate
        self.allPositionViewController?.delegateOnScroll = self.jobDetailMainController
        self.allPositionViewController?.delegateOnClick = self.jobDetailMainController
        
        // info delegate
        self.infoViewController?.delegateOnScroll = self.jobDetailMainController
    }
    
    func initialPageView() {
        // Initail Pageview
        pageViewController = UIPageViewController(transitionStyle: .Scroll, navigationOrientation: .Horizontal, options: nil)
        pageViewController!.dataSource = self
        pageViewController?.delegate = self
        
        // Set First View
        let startingViewController: UIViewController = viewControllerAtIndex(INDEX_PAGE!.rawValue)!
        let viewControllers = [startingViewController]
        
        pageViewController!.setViewControllers(viewControllers , direction: .Forward, animated: false, completion: nil)
        pageViewController!.view.frame = CGRectMake(0, 0, view.frame.size.width, view.frame.size.height);
        
        // Add to Pageview
        self.addChildViewController(pageViewController!)
        self.view.addSubview(pageViewController!.view)
        pageViewController!.didMoveToParentViewController(self)
        enableSwipePageView(true)
    }
    
    func callService() {
        
        let serviceController = JTMServiceController()
        let url = getServiceJobDetail(companyCode!)
        
        jobDetailMainController?.showLoading()
        serviceController.requestService(self, urlString: url, params: nil, isPresentLoadingIndicator: false) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.parseJson(jsonObject!)
                self.setTitleMainJobDetail()
                self.isNeedToUpdateData = false
            }
            
            self.jobDetailMainController?.hideLoading()
        }
    }
    
    func parseJson(jsonResult : NSMutableDictionary) {
        self.jobdetailData = JobDetailModel(jsonObject: jsonResult)
        updatePageViewUI()
    }
    
    func setTitleMainJobDetail() {
        self.delegateSetMainJobdetail?.setTitleJobDetail((jobdetailData?.companiesDetail!)!)
    }
    
    func updatePageViewUI() {
        // Mark :: Update Data
        self.allPositionViewController?.updateData((self.jobdetailData?.allPosition)!)
        
        //MARK:- Update Job Info Page
        let allPositionData = self.jobdetailData?.allPosition!
        if (allPositionData!.count > 0) {
            let selectedIndex = allPositionData?.indexOf({$0.runningNumber == jobCode})

            if ((jobCode != nil) && (selectedIndex != nil)) {
                self.updataDataSelectedJob(selectedIndex!, jobcode: jobCode!)
            } else {
                self.updataDataSelectedJob(0, jobcode: (self.jobdetailData?.allPosition![0].runningNumber)!)
            }
        }
        
        //MARK:- Update Company Info Page
        self.companyViewController?.updateData((self.jobdetailData?.companiesDetail?.companyInfo)!, companyName: (self.jobdetailData?.companiesDetail?.companyName)!)
    }

    // ----------------------------------------------------------------- Update Data ------------------------------------------------------------------------ //
    
    func updataDataSelectedJob(indexList: Int, jobcode: Int) {
        self.selectedJob = indexList
        self.jobCode = jobcode
        self.infoViewController?.updateJobData((self.jobdetailData?.allPosition![selectedJob!])!, companyName: (self.jobdetailData?.companiesDetail?.companyName)!)
    }
    
    // -------------------------------------------------------------- Delegate PageView --------------------------------------------------------------------- //
    
    //Mark :: Before Page View
    func pageViewController(pageViewController: UIPageViewController, viewControllerBeforeViewController viewController: UIViewController) -> UIViewController? {
        // check index
        var index : NSInteger = getCurrentIndex(viewController)
        
        if (index == 0) || (index == NSNotFound) {
            return nil
        }
        
        index -= 1
        
        return viewControllerAtIndex(index)
    }
    
    //Mark :: After Page View
    func pageViewController(pageViewController: UIPageViewController, viewControllerAfterViewController viewController: UIViewController) -> UIViewController? {
        // check index
        var index : NSInteger = getCurrentIndex(viewController)
        
        if (index == NSNotFound) || (index + 1 == self.pageTitles.count) {
            return nil
        }
        
        index += 1
        
        return viewControllerAtIndex(index)
    }
    
    func pageViewController(pageViewController: UIPageViewController, willTransitionToViewControllers pendingViewControllers: [UIViewController]) {
        
        isWillTransition = true
        
        let destinationController = pendingViewControllers[0] as! JTMBaseViewController
        
        if (destinationController.isPresentedBefore == false) {
            jobDetailMainController?.showLoading()
        }    }
    
    // Mark :: Slide
    func pageViewController(pageViewController: UIPageViewController, didFinishAnimating finished: Bool, previousViewControllers: [UIViewController], transitionCompleted completed: Bool) {
        
        isWillTransition = false
        
        let currentController = self.pageViewController?.viewControllers?.first as! JTMBaseViewController
        self.delegateSetMainJobdetail?.setStatusCurrentSegment(self.getCurrentIndex(currentController))
        if (currentController.isPresentedBefore) {
            jobDetailMainController?.hideLoading()
        }
    }
    
    // Mark :: Select ViewController
    func viewControllerAtIndex(index: Int) -> UIViewController? {
        if self.pageTitles.count == 0 || index >= self.pageTitles.count {
            return nil
        }
        
        if IndexJobDetailPage(rawValue: index) == .JOB_INFO_PAGE {
            return infoViewController
        } else if IndexJobDetailPage(rawValue: index) == .ALL_POSITION_PAGE {
            return allPositionViewController
        } else if IndexJobDetailPage(rawValue: index) == .COMPANY_DETAIL_PAGE {
            return companyViewController
        }
        
        return nil
    }
    
    // Mark :: Get Current Index
    func getCurrentIndex(viewController : UIViewController)-> NSInteger {
        // check index
        var index : NSInteger = NSNotFound
        
        if viewController.isKindOfClass(JTMJobDetailAllPositionViewController) {
            index = (viewController as! JTMJobDetailAllPositionViewController).PAGE_INDEX.rawValue
        } else if( viewController .isKindOfClass(JTMJobDetailCompanyViewController)){
            index = (viewController as! JTMJobDetailCompanyViewController).PAGE_INDEX.rawValue
        } else if( viewController .isKindOfClass(JTMJobDetailInfoViewController)){
            index = (viewController as! JTMJobDetailInfoViewController).PAGE_INDEX.rawValue
        }
        
        return index
    }
    
    // -------------------------------------------------------------- Delegate onClick --------------------------------------------------------------------- //
   
    func onClickSegment(index: Int) {
        let controller = viewControllerAtIndex(index) as! JTMBaseViewController
        
        if(controller.isPresentedBefore == false) {
            self.jobDetailMainController?.showLoading()
        }
        
        if !isWillTransition {
            self.pageViewController!.setViewControllers([controller],
                                                        direction: .Forward,
                                                        animated: false,
                                                        completion: {(success)->Void in
                                                            self.delegateSetMainJobdetail?.setStatusCurrentSegment(index)
                                                            if (controller.isPresentedBefore) {
                                                                (self.jobDetailMainController?.hideLoading())!
                                                            }
            })
        }
        
    }
    
    //MARK:- Call Back When Application Did Active
    override func applicationDidEnterBackground() {
        if isWillTransition {
            self.isWillTransition = false
            let currentPageIndex : NSInteger = getCurrentIndex((self.pageViewController?.viewControllers?.first)!)
            onClickSegment(currentPageIndex)
        }
        
    }
    
    //MARK:- Swipe page view
    func enableSwipePageView(enable: Bool) {
        
        if enable {
            pageViewController?.dataSource = self
        }
        else {
            pageViewController?.dataSource = nil
        }
    }
    
    //MARK:- Set identifier for UITest
    func setAccessibilityIdentifier() {
        self.view.accessibilityIdentifier = ID_JOB_DETAIL_VIEW
        infoViewController?.view.accessibilityIdentifier = ID_JOB_DETAIL_INFO_VIEW
        allPositionViewController?.view.accessibilityIdentifier = ID_JOB_DETAIL_ALL_POSITION
        companyViewController?.view.accessibilityIdentifier = ID_JOB_DETAIL_COMPANY
    }
}

// Mark :: Delegate send to main controller
protocol JTMJobdetailPageViewDelegate {
    func setStatusCurrentSegment(currentIndex : NSInteger)
    func setTitleJobDetail(companyDetail : CompanyDetailModel)
}
