//
//  JTMJobNearMeViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/29/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMJobNearMeViewController: JTMBaseSearchViewController, JTMAreaJobNearMeDelegate, CLLocationManagerDelegate {

    enum nearMeSearchCellIndex: Int {
        case NEARME_CATEGORY
        case NEARME_KEYWORD
        case NEARME_RADIUS
        case NEARME_FOOTER
        case count
    }
    
    let NUMBER_OF_NEARME_CELL:CGFloat = 3.25 // 2 cell + 1 extra cell (near me area)
    
    // Variable Parameter Default
    var latitude = 0.0
    var longitude = 0.0
    
    var locationStatus : NSString = "Not Started"
    let locationManager = CLLocationManager()
    
    var waitingLocation : Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        initLocationService()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("JobNearMe viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_SEARCH_NEAR_ME)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_NEAR_ME, action: ACTION_VIEW, label: LABEL_FORM_JOB_NEAR_ME)
        
        self.tableView.reloadData()
    }
    
    override func applicationDidActive() {
        self.checkLocationEnable()
    }
    //MARK: ---
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_JOB_NEAR_ME")
    }

    // Mark :: Initial Location Service
    func initLocationService() {
        locationManager.delegate = self
        locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters
        locationManager.startUpdatingLocation()
    }
    
    // Mark :: Register File Nib
    override func registerNibForTableView() {
        super.registerNibForTableView()
        self.tableView?.registerNib(UINib(nibName: "JTMAreaJobNearMeTableViewCell", bundle: nil),
                                    forCellReuseIdentifier: "JTMAreaJobNearMeTableViewCell")
    }

    //MARK:- Near Me Filter
    func getNearMeRadiusValueFromSaveState() -> Int {
        return (parentController?.searchFilterSaveState?.nearMeRadius)!
    }
    
    func getNearMeRadiusParameterFromSaveState() -> String {
        return "\((parentController?.searchFilterSaveState!.nearMeRadius)!)"
    }
    
    func updateSelectedNearMeRadius(value:Int) {
        parentController?.searchFilterSaveState?.nearMeRadius = value
    }
    
    //MARK:- TableView
    override func initTableView() {
        super.initTableView()
//        tableHeightConstraint.constant = cellHeight! * NUMBER_OF_NEARME_CELL
    }
    
    override func configureCellForRowAtIndexPath(cell: JTMSeachFilterTableViewCell, indexPath: NSIndexPath) -> UITableViewCell {
        
        setSelectedBackgroundView(cell)
        
        switch ((nearMeSearchCellIndex(rawValue: indexPath.row))!) {
            case .NEARME_CATEGORY:
                configureJobCategoryCell(cell)
                return cell
            case .NEARME_KEYWORD:
                cellSearch = configureCellSearch()
                return cellSearch!
            case .NEARME_FOOTER:
                cellFooter = configureCellSearchFooter()
                cellFooter?.setCellSeperatorLineHide(tableView)
                return cellFooter!
            default:
                let cellArea = tableView.dequeueReusableCellWithIdentifier("JTMAreaJobNearMeTableViewCell") as? JTMAreaJobNearMeTableViewCell
                cellArea!.selectionStyle = UITableViewCellSelectionStyle.None
                cellArea?.setSliderValueTo(getNearMeRadiusValueFromSaveState())
                cellArea?.delegate = self
                return cellArea!
        }
    }

    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        if((nearMeSearchCellIndex(rawValue: indexPath.row))! == .NEARME_CATEGORY) {
            openSearchFilter(SearchFilterCategoryType.CATEGORY)
        }
        
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func setSelectedBackgroundView(cell: JTMSeachFilterTableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
//        if (nearMeSearchCellIndex(rawValue: indexPath.row))! == .NEARME_RADIUS || (nearMeSearchCellIndex(rawValue: indexPath.row))! == .NEARME_FOOTER {
            return CELL_SIZE_BIG
//        } else {
//            return CELL_SIZE_SMALL
//        }
    }

    override func footerOnClickSearch() {
        
        let status = CLLocationManager.authorizationStatus()
        
        if CLLocationManager.locationServicesEnabled() {
            switch (status) {
            case .NotDetermined :
                if CLLocationManager.locationServicesEnabled() {
                    locationManager.requestWhenInUseAuthorization()
                } else {
                    alertNoPermissionToAccess()
                }
                break
            case .Denied :
                alertNoPermissionToAccess()
                break
            default :
                if(self.latitude == 0.0 || self.longitude == 0.0){
                    self.showLoading()
                    self.waitingLocation = true
                }else{
                    openSearchResultForJobNearMe()
                }
                break
            }
        } else {
            alertNoOpenLocationService()
        }
    }
    
    func checkLocationEnable(){
        if CLLocationManager.locationServicesEnabled() {
            let status = CLLocationManager.authorizationStatus()
            if(status == .AuthorizedAlways || status == .AuthorizedWhenInUse){
                if(self.latitude == 0.0 || self.longitude == 0.0){
                    self.showLoading()
                }
            }
        }
    }
    
    func openSearchResultForJobNearMe() {
        
        var action = ""
        
        let jobType = getJobTypeParamaterFromSaveState()
        let subType = getSubJobTypeParamaterFromSaveState()
        let keyword = getKeywordParameterFromSaveState()
        let radius = getNearMeRadiusParameterFromSaveState()
        let disability = getDisabilitySaveState().hashValue
//        let params = "?jobtype=\(jobType)&subtype=\(subType)&keyword=\(keyword)&lat=\(self.latitude)&long=\(self.longitude)&radius=\(radius)&disabled_person=\(disability)"
        
        let parameter: [String:AnyObject] = [
            PARAMETER_JOB_TYPE_KEY          : jobType,
            PARAMETER_SUB_JOB_TYPE_KEY      : subType,
            PARAMETER_KEYWORD_KEY           : keyword,
            PARAMETER_LATITUDE_KEY          : self.latitude,
            PARAMETER_LONGITUDE_KEY         : self.longitude,
            PARAMETER_NEAR_ME_RADIUS        : radius,
            PARAMETER_DISABLED_WELCOME_KEY  : disability
        ]
        
        self.setTrackGoogleAnalyticKeyword(keyword)
        self.setTrackGoogleAnalyticDistance(radius)
        self.setTrackGoogleAnalyticDisability(disability)
        
        
        openSearchResult(parameter , isOpenMap: true , radiusInMeter: (radius as NSString).doubleValue)
    }
    
    func setTrackGoogleAnalyticKeyword(keyword : String) {
        var action = ""
        
        if keyword.isEmpty {
            action = ACTION_SEARCH_NON_KEYWORD
        } else {
            action = ACTION_SEARCH_KEYWORD
        }
        
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    func setTrackGoogleAnalyticDistance(radius : String) {
        
        let distance = "\((Float(radius)!/1000))" + " " + applicationDelegate.getLocalizeStringForKey("TITLE_AREA_JOB_NEAR_ME_BACK")
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_NEAR_ME, action: ACTION_CLICK_DISTANCE, label: getLabelDistance(distance))
    }
    
    func setTrackGoogleAnalyticDisability(disability : Int) {
        var action = ""
        
        if disability == 1 {
            action = ACTION_SEARCH_DISABILITY
        } else {
            action = ACTION_SEARCH_NON_DISABILITY
        }
        
        // Track Google Analytic Event disability
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    // Mark :: Delegate UISlider
    func onValueAreaChanged(value: Int) {
        updateSelectedNearMeRadius(value)
    }
        
    // Mark :: Get Lat Lon
    func locationManager(manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        let locValue:CLLocationCoordinate2D = manager.location!.coordinate
        print("locations = \(locValue.latitude) \(locValue.longitude)")
        self.longitude = locValue.longitude
        self.latitude = locValue.latitude
        if isHasAllRequireFilterListValue() {
            self.hideLoading()
        }
        if(self.waitingLocation){
            self.waitingLocation = false
            locationManager.stopUpdatingLocation()
            self.openSearchResultForJobNearMe()
        }
        
        
    }

}
