//
//  JTMLandingPageView.swift
//  JobThai
//
//  Created by ForAlL on 1/29/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMLandingPageView: UIView  , UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout{
    
    var LANDING_PAGE_CELL_SIZE = CGSize(width: UIScreen.mainScreen().bounds.size.width, height: 120)
    let DEFAULT_NUMBER_OF_DATA = 10000
    
    @IBOutlet weak var landingPageUICollectionView: UICollectionView!
    var delegate : JTMLandingPageViewDelagate?
    var landingPageListData : [LandingPageModel]?
    var numOfData = 0
    
    var timer:NSTimer?
    var view: UIView!
    
    //Load Nib File
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupUi()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupUi()
    }
    
    func setupUi(){
        view = loadViewFromNib()
        view.frame = bounds
        view.autoresizingMask = [.FlexibleWidth , .FlexibleHeight]
        addSubview(view)
        
        // Set landing page Height
        if screenType == DisplaySizeType.SCREEN_3_5_INCH { // iphone 4,4s
            LANDING_PAGE_CELL_SIZE.height = 80
        } else if screenType == DisplaySizeType.SCREEN_4_0_INCH {  // iphone 5,5s,5c
            LANDING_PAGE_CELL_SIZE.height = 80
        } else if screenType == DisplaySizeType.SCREEN_4_7_INCH {  // iphone 6
            LANDING_PAGE_CELL_SIZE.height = 100
        } else if screenType == DisplaySizeType.SCREEN_5_5_INCH {  // iphone 6 plus
            LANDING_PAGE_CELL_SIZE.height = 120
        }
        
    }
    
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMLandingPageView", bundle: bundle)
        let view = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        landingPageUICollectionView.delegate = self
        landingPageUICollectionView.dataSource = self
        
        // Initialization code
        landingPageUICollectionView?.registerNib(UINib(nibName: "JTMLandingPageCollectionViewCell", bundle: nil), forCellWithReuseIdentifier: "JTMLandingPageCollectionViewCell")
    }
    
    
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        print("Click \(indexPath.row)")
        print("Click Landing Page")
        
        if(delegate != nil) {
            let indexOnModels = indexPath.row % (self.landingPageListData?.count)!
            delegate?.onLandingPageClick(self, landingPageItem: self.landingPageListData![indexOnModels])
        }
        
    }
    
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return 1
    }
    
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return numOfData
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCellWithReuseIdentifier("JTMLandingPageCollectionViewCell", forIndexPath: indexPath) as! JTMLandingPageCollectionViewCell
        let indexOnModels = indexPath.row % (self.landingPageListData?.count)!
        cell.setImageLandingPage(self.landingPageListData![indexOnModels])
        
        return cell
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
        return LANDING_PAGE_CELL_SIZE
    }
    
    func setLandingPageModel(landingPageModel: [LandingPageModel]) {
        self.landingPageListData = landingPageModel
        if(self.landingPageListData?.count > 0) {
            numOfData = DEFAULT_NUMBER_OF_DATA
        }
        self.landingPageUICollectionView.reloadData()
        
        landingPageUICollectionView.scrollToItemAtIndexPath(
            NSIndexPath.init(forItem: DEFAULT_NUMBER_OF_DATA/2, inSection: 0),
            atScrollPosition: .CenteredHorizontally,
            animated: false)
    }
    
    func autoSlideLandingPage() {
        if (self.landingPageListData?.count > 0) {
            if landingPageUICollectionView?.indexPathsForVisibleItems().count > 0 {
                let currentLandingPagIndexPath = landingPageUICollectionView?.indexPathsForVisibleItems()[0]
                landingPageUICollectionView.scrollToItemAtIndexPath(NSIndexPath.init(
                    forItem: currentLandingPagIndexPath!.row+1, inSection: 0),
                                                                    atScrollPosition: .CenteredHorizontally,
                                                                    animated: true)
            }
        }
        
    }
    
    //timer
    func startTimer() {
        if (timer == nil) {
            timer = NSTimer.scheduledTimerWithTimeInterval(
                5.0,
                target: self,
                selector: #selector(autoSlideLandingPage),
                userInfo: nil,
                repeats: true)
        }
    }
    
    func stopTimer() {
        timer?.invalidate()
        timer = nil
    }
    
}

protocol JTMLandingPageViewDelagate {
    func onLandingPageClick(landingPageView : JTMLandingPageView, landingPageItem : LandingPageModel)
}
