//
//  JTMLoginViewController.swift
//  JobThai
//
//  Created by ForAlL on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit


class JTMLoginViewController: JTMBaseMenuViewController, JTMLoginViewDelegate {
    
    @IBOutlet weak var loginView: JTMLoginView!
    var redirectType: LogInRedirectType = .BACK_TO_HOME
    var completion: (() -> Void)?
    var isParentNeedLogInStatus: Bool = false
    
    var parentToBack : UIViewController?
    
//    var userData : UserModel?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        loginView.delegate = self
        initialObserver()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_LOGIN)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_LOGIN, action: ACTION_VIEW, label: LABEL_VIEW_LOGIN)
        
        loginView.setFilledTextField()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
    }
    
    func initialObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(applicationFinishedTryAutoLogIn), name: JTM_FINISHED_AUTO_LOG_IN, object: nil)
    }
    
    //Mark:- UIBarButton
    override func genUISlidMenuButton() -> UIBarButtonItem? {
        if (redirectType != .BACK_TO_HOME) {
            return nil
        } else {
            return super.genUISlidMenuButton()
        }
    }
    
    override func genUISearchMenuButton() -> UIBarButtonItem? {
        return nil
    }
    
    override func actionBack() {
        if isParentNeedLogInStatus {
            NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
        } else {
            super.actionBack()
        }
    }
    
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        if applicationDelegate.isTryingAutoLogin {
            showLoadingWithBlankView()
        }
    }
    
    //Mark:- LanguageChange
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_LOGIN")
        
        loginView.setUILabel()
    }
    
    
    //Mark:- Delegate
    func onClickForgotPassword() {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_LOGIN, action: ACTION_CLICK, label: LABEL_FORGOT_PASSWORD)
        
        let forgotPassword = self.storyboard?.instantiateViewControllerWithIdentifier("JTMForgotPasswordViewController") as? JTMForgotPasswordViewController
        forgotPassword?.parentToBack = parentToBack
        self.navigationController?.pushViewController(forgotPassword!, animated: true)
    }
    
    func onclickLogin() {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_LOGIN, action: ACTION_CLICK, label: LABEL_LOGIN)
        
        if (applicationDelegate.appSetting?.isNetworkReachable)! == false {
            return
        }
        
        loginUser()
    }
    
    func onClickSignUp() {
//        var actionList = [UIAlertAction]()
//        let alertAction = UIAlertAction(title: "OK", style: .Default, handler: nil)
//        actionList.append(alertAction)
//        
//        self.showAlertWithData("Sign Up", message: nil, completionHandler:nil, actionList: nil)
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_LOGIN, action: ACTION_CLICK, label: LABEL_REGISTER)
        
        let registerView = self.storyboard?.instantiateViewControllerWithIdentifier("JTMRegisterViewController") as? JTMRegisterViewController
        registerView?.parentToBack = parentToBack
        self.navigationController?.pushViewController(registerView!, animated: true)
    }
    
    //Mark:- Log in 
    func loginUser() {
        var alertTitle: String = ""
        var alertCompletion: ((UIAlertAction) -> Void)?
        
        if (loginView.usernameTextField.text == "") {
            if (loginView.passwordTextField.text == "") {
                alertTitle = applicationDelegate.getLocalizeStringForKey("ALERT_NO_USER_PASSWORD")
                alertCompletion = { (UIAlertAction) -> Void in
                    self.loginView.setfocusOnUsernameTextField()
                }
            } else {
                alertTitle = applicationDelegate.getLocalizeStringForKey("ALERT_NO_USERNAME")
                alertCompletion = { (UIAlertAction) -> Void in
                    self.loginView.setfocusOnUsernameTextField()
                }
            }
        } else if (loginView.passwordTextField.text == "") {
            alertTitle = applicationDelegate.getLocalizeStringForKey("ALERT_NO_PASSWORD_LOGIN")
            alertCompletion = { (UIAlertAction) -> Void in
                self.loginView.setfocusOnPasswordTextField()
            }
        } else {
            let param = getLogInParameter(loginView.usernameTextField.text!, password: loginView.passwordTextField.text!)
            JTMLoginViewController.connectServiceLogIn(self, parameter: param/*username: loginView.usernameTextField.text!, password: loginView.passwordTextField.text!*/, redirectType: self.redirectType, completion: completion)
            return
        }
        
        self.showAlertWithData(alertTitle, message: nil,completionHandler: alertCompletion, actionList: nil)
    }
    
    //Manage Something after call service login
    static func setUserCredentialAfterLogIn() {//username: String, password: String) {
        let userCredential = applicationDelegate.appSetting?.userCredential
        userCredential?.username = applicationDelegate.appSetting?.userData?.userName
        userCredential?.token = applicationDelegate.appSetting?.userData?.token
        userCredential?.sessionId = applicationDelegate.appSetting?.userData?.sessionId
        
        applicationDelegate.appSetting?.userCredential?.rememberUserCredential()
    }
    
    static func checkUserStatusAndRedirect(controller: JTMBaseViewController, redirectType: LogInRedirectType, completion: (() -> Void)?) {
        let userData = applicationDelegate.appSetting?.userData
        if ((userData?.isWatch)! || (userData?.isBan)!) && redirectType != .AUTO_LOGIN {
            controller.showAlertWithData((userData?.warningTitle)!,
                message: (userData?.warningMessage)!,
                completionHandler: { (alierAction) -> Void in
                    JTMLoginViewController.redirectToViewController(controller, redirectType: redirectType, completion: completion)
                }, actionList: nil)
        } else {
            JTMLoginViewController.redirectToViewController(controller, redirectType: redirectType, completion: completion)
        }
    }
    
    static func manageLogInError(controller: JTMBaseViewController,jsonObject: NSMutableDictionary?) {
        let errCode = (jsonObject?.objectForKey("errcode"))! as! String
        let errorTitle = (jsonObject?.objectForKey("titlemsg") == nil) ? "" : (jsonObject?.objectForKey("titlemsg"))! as! String
        let errorMessage = (jsonObject?.objectForKey("errmsg") == nil) ? "" : (jsonObject?.objectForKey("errmsg"))! as! String
        var completionHandler: ((UIAlertAction)->Void)?
        
        if errCode != EMAIL_NOT_VERIFIED_OLD_USER {
            
            let logInController = controller as? JTMLoginViewController
            logInController?.loginView.passwordTextField.text = ""
            
            completionHandler = {(UIAlertAction) -> Void in
                logInController?.loginView.setfocusOnUsernameTextField()
            }
//            controller.showAlertWithData(errorTitle, message: errorMessage, completionHandler: nil, actionList: nil)
            
        } else {
            let linkUrl = (jsonObject?.objectForKey("linkUrl"))! as! String
            completionHandler = { (UIAlertAction) -> Void in
                UIApplication.sharedApplication().openURL(NSURL(string: linkUrl)!)
                print("openURL")
            }
        }
        
        controller.showAlertWithData(errorTitle, message: errorMessage, completionHandler: completionHandler, actionList: nil)
    }
    
    // Call Service JSON Function
    static func connectServiceLogIn(controller:JTMBaseViewController, parameter:[String:AnyObject]/*username:String, password:String*/, redirectType: LogInRedirectType, completion: (() -> Void)?) {
        
        let serviceController = JTMServiceController()
        let url = redirectType == .AUTO_LOGIN ? getServiceAutoLogIn() : getServiceAuthentication()
        
//        let encrypt = Encryption()
//        let textEncode = encrypt.encrypt("\(username):\(password)")
//        
//        let param = [
//            "cipher"    : textEncode
//        ]
        
        serviceController.requestService(controller, urlString: url, params: parameter) { (response, status) -> Void in
            switch status {
                case .CONNECTION_SUCCESS:
                    var jsonObject: NSMutableDictionary?
                    jsonObject = response!.result.value as? NSMutableDictionary
                    
                    dump(jsonObject)
                    
                    if ((jsonObject?.objectForKey("status"))! as! String == "ok") {
                        self.convertJsonToModel(jsonObject!)
                        JTMLoginViewController.setUserCredentialAfterLogIn()//username, password: password)
                        applicationDelegate.appSetting?.changeUserLoginState(true)
                        checkUserStatusAndRedirect(controller, redirectType: redirectType, completion: completion)
                    } else {
                        if (redirectType != .AUTO_LOGIN) {
                            manageLogInError(controller, jsonObject: jsonObject)
                        }
                        completion?()
                    }
                    break
                default:
                    completion?()
                    break
            }
        }
    }
    
    func getLogInParameter(username:String, password:String) -> [String:AnyObject] {
        let encrypt = Encryption()
        let textEncode = encrypt.encrypt("\(username):\(password)")
        
        let param = [
            "cipher"    : textEncode
        ]
        
        return param
    }
    
    static func getAutoLogInParameter(token : String, sessionId : String) -> [String:AnyObject] {
        let param = [
            "token" : token,
            "sessionId" : sessionId,
        ]
        return param
    }
    
    static func redirectToViewController(controller:JTMBaseViewController?, redirectType: LogInRedirectType, completion: (() -> Void)?) {
        switch (redirectType) {
            case .BACK_TO_HOME:
                NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
                break
            case .DISMISS_CONTROLLER:
                controller?.navigationController?.popViewControllerAnimated(true)
                completion?()
                break
            case .DISMISS_CONTROLLER_NO_ANIMATED:
                controller?.navigationController?.popViewControllerAnimated(false)
                completion?()
                break
            case .AUTO_LOGIN:
                completion?()
                break
            case .STAY_IN_CONTROLLER:
                completion?()
                break
        }
    }
    
    static func convertJsonToModel(jsonObject : NSMutableDictionary) {
        applicationDelegate.appSetting!.saveUserData(UserModel(userObjectJson: jsonObject))
    }
    
    //Mark:- Logout
    static func connectServiceLogout(controller: JTMBaseViewController?, redirectType: LogInRedirectType, completion: (() -> Void)?) {
        
        let serviceController = JTMServiceController()
        let url = getServiceLogOutAuthentication()
        let param = [
            "imei" : applicationDelegate.getDeviceUUID(),
            "token" : (applicationDelegate.appSetting?.userData?.token)!
        ]
        
        serviceController.requestBackgroundService(url, params: param) { (response, status) -> Void in
            
        }
        
        applicationDelegate.appSetting!.removeUserData()
        applicationDelegate.appSetting?.changeUserLoginState(false)
        
        redirectToViewController(controller, redirectType: redirectType , completion: completion)
    }
    
    //MARK:- AutoLogIn
    func applicationFinishedTryAutoLogIn() {
        if (applicationDelegate.appSetting?.isLogin)! {
            NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
        }
        
        hideLoadingWithBlankView()
    }
    
    //MARK:- Navigation Controller
    override func navigationController(navigationController: UINavigationController, willShowViewController viewController: UIViewController, animated: Bool) {
        super.navigationController(navigationController, willShowViewController: viewController, animated: animated)
        if let destinationController = viewController as? JTMBaseMenuViewController {
            if (checkIfControllerIsShowing() && destinationController.isCanUseAfterLogInOnly) {
                destinationController.isIgnoreTokenTimeOutAlert = true
                NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
            }
        }
    }
}
