//
//  JTMNoticeViewController.swift
//  JobThai
//
//  Created by pinyo on 2/10/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import Foundation
import UIKit

class JTMNoticeViewController : JTMBaseViewController , UITableViewDataSource , UITableViewDelegate {
    
    
    @IBOutlet var noticeTableView : UITableView!
    
    let CELL_IDENTIFIER         = "JTMNoticeTableViewCell"
    var noticeList              = [NoticeModel]()
    
    let NOTICE_CELL_HEIGHT      : CGFloat   = 600.0
    let CELL_HEIGHT             : CGFloat   = 100
    var isLoadNoticeFromService : Bool      = false
    var noticeReadIDList        : NSMutableDictionary! = NSMutableDictionary()
    var delegate                : JTMNoticeDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.onLanguageChange()
        self.initSaveNotceReadIdList()
        self.initalTableView()
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        clearContentShowing()
    }
    
    //MARK:- Connection Error 
    override func onRetryConnection() {
        super.onRetryConnection()
        if !isLoadNoticeFromService {
            delegate?.updateNoticeList(self)
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if !isLoadNoticeFromService {
            setAlphaShowToView(view)
        }
    }
    
    //MARK: ---
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TEXT_TITLE_NOTIES")
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_NOTICES)
    }

    //MARK: READ & SAVE Notice Read List
    func initSaveNotceReadIdList() {
        
        self.noticeReadIDList = applicationDelegate.appSetting!.getNoticeReadIdList().mutableCopy() as! NSMutableDictionary
    }
    
    func saveNoticeReadList() {
        applicationDelegate.appSetting!.saveNoticeReadIdList(self.noticeReadIDList!)
        applicationDelegate.appSetting!.saveNoticeNewCount(self.noticeList.count - self.noticeReadIDList.count)
    }
    
    func selectNoticeCell(cell : JTMNoticeTableViewCell , index : Int) -> Bool{
        let noticeItem = self.noticeList[index]
        self.noticeReadIDList.setObject(noticeItem.title!, forKey: noticeItem.id!)
        cell.setHideNewLabel(true)
        
        noticeItem.read = true
        noticeItem.contentShowing = !noticeItem.contentShowing
        cell.showContent(noticeItem.contentShowing)
        
        return noticeItem.contentShowing
    }

    //MARK: Initial TableView
    func initalTableView(){
        self.noticeTableView.tableFooterView = UIView()
        self.noticeTableView.rowHeight = UITableViewAutomaticDimension
        self.noticeTableView.estimatedRowHeight = NOTICE_CELL_HEIGHT
        self.noticeTableView.registerNib(UINib(nibName: CELL_IDENTIFIER, bundle: nil), forCellReuseIdentifier: "JTMNoticeTableViewCell")
        self.noticeTableView.layer.borderColor = UIColor(hexString: "CCCCCC").CGColor
    }
    
    //MARK: TableView Delegate
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.noticeList.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(CELL_IDENTIFIER , forIndexPath: indexPath) as? JTMNoticeTableViewCell
        cell?.setData(self.noticeList[indexPath.row])
//        setSelectedBackgroundView(cell!)
        cell?.selectionStyle = .None
        return cell!
    }
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        let noticeItem = noticeList[indexPath.row]
        if noticeItem.contentShowing {
            return CELL_HEIGHT + noticeItem.contentHeight
        } else {
            return CELL_HEIGHT
        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        if let cell : JTMNoticeTableViewCell = tableView.cellForRowAtIndexPath(indexPath) as? JTMNoticeTableViewCell{
            tableView.deselectRowAtIndexPath(indexPath, animated: false)
            let contentShowing = selectNoticeCell(cell, index: indexPath.row)
            tableView.reloadData()
            if contentShowing {
                self.delay(0.1, closure: { () -> () in
                    tableView.scrollToRowAtIndexPath(indexPath, atScrollPosition: UITableViewScrollPosition.Top, animated: true)
                })
            }
            saveNoticeReadList()
        }
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
    func clearContentShowing() {
        for noticeItem in noticeList {
            noticeItem.contentShowing = false
        }
    }
}

protocol JTMNoticeDelegate {
    func updateNoticeList(noticeController: JTMNoticeViewController)
}
