//
//  JTMPreloadViewController.swift
//  JobThai
//
//  Created by pinyo on 6/6/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMPreloadViewController : JTMBaseViewController {
    
    @IBOutlet private var preloadImageView  : UIImageView!
    @IBOutlet private var preloadButton     : UIButton!
    
    @IBOutlet private var spaceTop          : NSLayoutConstraint?
    @IBOutlet private var spaceBottom       : NSLayoutConstraint?
    
    private var preloadItemList             = [PreloadModel]()
    private var PRELOAD_TIME : Double       = 8                 //Seconds
    private var preloadShowing              : PreloadModel?
    
    static var waitDelayAction              : Bool = true
    
    let SHADOW_WIDTH    : CGFloat   = 7
    let SHADOW_OPACITY  : Float     = 0.1
    
    override func viewDidLoad() {
        preloadStartTimer(PRELOAD_TIME)
    }
    
    //MARK: Override for remove super function
    override func viewWillAppear(animated: Bool) {
        setPreloadUI()
    }
    
    override func viewWillDisappear(animated: Bool) {
        
    }
    
    //MARK: Initial Data
    func setPreloadData(data : [PreloadModel]){
        preloadItemList = data
    }
    
    func setPreloadUI(){ 
        setupPreloadImage()
        setPreloadAccessibilityId()
    }
    
    func setupPreloadImage() {
        preloadShowing = preloadItemList.first
        
        guard let preloadImg = preloadShowing?.imageView.image else {
            guard let imageUrl = preloadShowing?.imageUrl else {
                return
            }
            
            preloadImageView.sd_setImageWithURL(NSURL(string: imageUrl)) {
                (image, error, cacheType, url) in
                self.setPreloadImageSize()
            }
            return
        }
        
        preloadImageView.image = preloadImg
        setPreloadImageSize()
    }
    
    //MARK: Calculate Preload Image Size and Set Constraint
    func setPreloadImageSize() {
        self.view.layoutIfNeeded()
        self.view.bringSubviewToFront(preloadButton)
        
        guard let imageSize : CGSize = preloadImageView?.image?.size else {
            return
        }
        
        var newFrame : CGRect = (preloadImageView?.frame)!
        newFrame.size.height = (imageSize.height/imageSize.width) * (preloadImageView?.frame.size.width)!
        
        let space = (DEFAULT_SCREEN_HEIGHT - newFrame.size.height)/2
        spaceTop?.constant    = space
        spaceBottom?.constant = space
        setPreloadShadow()
        
    }
    
    func setPreloadShadow() {
        preloadImageView?.layer.shadowOpacity = SHADOW_OPACITY
        preloadImageView?.layer.shadowColor = UIColor.whiteColor().CGColor
        preloadImageView?.layer.shadowOffset = CGSize(width: SHADOW_WIDTH, height: SHADOW_WIDTH)
    }
    
    func setPreloadAccessibilityId(){
        self.view.accessibilityIdentifier               = ID_PRELOAD_VIEW
        self.preloadImageView.accessibilityIdentifier   = ID_PRELOAD_IMAGEVIEW
        self.preloadButton.accessibilityIdentifier      = ID_PRELOAD_ENTER_APP_BUTTON
    }
    
    //MARK: Preload Click Event
    func onPreloadClickImageView(preloadItem : PreloadModel) {
        
        print("preLoad")
        // Track Google Analytic Screen
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_PRELOAD)
        
        var valCom : String?
        
        if preloadItem.type != .WEBVIEW {
            valCom = preloadItem.companyCode
            self.openJobDetail(preloadItem.companyCode!, jobcode: preloadItem.jobCode!, type: preloadItem.type!)
        } else {
            valCom = preloadItem.companyName
            self.openWebView(preloadItem.companyName!, linkUrl: preloadItem.linkUrl!)
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_PRELOAD, action: ACTION_CLICK, label: getLabelPreloadWithComCode(valCom!))
    }
    
    //MARK: Timer
    func preloadStartTimer(second : Double){
        self.performSelector(#selector(preloadDoAfterDelay), withObject: nil, afterDelay: second)
    }
    
    func preloadDoAfterDelay(){
        if(JTMPreloadViewController.waitDelayAction){
            JTMPreloadViewController.waitDelayAction = false
            goToMainViewController()
        }
    }
    
    func goToMainViewController(){
        if let mainViewController = getMainViewController() {
            applicationDelegate.window?.rootViewController = mainViewController
        }
    }
    
    func getMainViewController() -> JTMMainViewController? {
        let mainViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainViewController") as? JTMMainViewController
        return mainViewController
    }
    
    func setMainIsRootAndPushViewController(view : UIViewController){
        let mainViewController = getMainViewController()
        mainViewController?.viewControllers.append(view)
        
        JTMPreloadViewController.waitDelayAction = false
         applicationDelegate.window?.rootViewController = mainViewController
    }
    
    //MARK: OpenWebView & OpenJobDetail
    func openWebView(title : String , linkUrl : String) {
        var webView : JTMWebViewController?
        webView = self.storyboard?.instantiateViewControllerWithIdentifier("JTMWebViewController") as? JTMWebViewController
        webView?.webTitle = title
        webView?.urlString = linkUrl
        
//        if(webCanOpenURL(linkUrl)){
            setMainIsRootAndPushViewController(webView!)
//        }
    }
    
    func openJobDetail(companyCode : String, jobcode : Int, type : IndexJobDetailPage) {
        let jobDetailStoryBoard = UIStoryboard(name: "JobDetail", bundle: nil)
        let jobDetail = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMJobDetailViewController") as? JTMJobDetailViewController
        
        jobDetail?.companyCode = companyCode
        if type == .JOB_INFO_PAGE {
            jobDetail?.jobCode = jobcode
        }
        jobDetail?.openPageType = type
        
        setMainIsRootAndPushViewController(jobDetail!)
    }
    
    func webCanOpenURL(url : String) -> Bool {
        let canOpenURL = UIApplication.sharedApplication().canOpenURL(NSURL(string: url)!)
        return canOpenURL
    }
    
    //MARK: Action
    @IBAction func preloadImageClicked(gesture : UIGestureRecognizer){
        JTMPreloadViewController.waitDelayAction = false
        onPreloadClickImageView(preloadShowing!)
    }
    
    @IBAction func preloadButtonPressed(){
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_PRELOAD, action: ACTION_CLICK, label: LABEL_PRELOAD_CLICK_HOME)
        
        JTMPreloadViewController.waitDelayAction = false
        goToMainViewController()
    }
    
    //MARK: Hide Status Bar
    override func prefersStatusBarHidden() -> Bool {
        return true
    }
    
}