//
//  JTMQuickSearchViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/17/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMQuickSearchViewController: JTMBaseSearchViewController {
    
    enum quickSearchCellIndex: Int {
        case QUICK_CATEGORY
        case QUICK_LOCATION
        case QUICK_KEYWORD
        case QUICK_FOOTER
        case count
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func isHasAllRequireFilterListValue() -> Bool {
        if (applicationDelegate.appSetting?.searchFilterLocationList.count > 0) {
                return super.isHasAllRequireFilterListValue()
        }
        
        return false
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("quickSearch : viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_QUICK_SEARCH)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH, action: ACTION_VIEW, label: LABEL_FORM_JOB_SEARCH)
        
        self.tableView.reloadData()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
   
    //MARK: ---
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_QUICK_SEARCH")
    }
    
    //MARK:- Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if (applicationDelegate.appSetting?.searchFilterLocationList.count == 0) {
            loadLocationListFromService()
        }
    }
    
    override func onConnectionError(errorRequestUrl: String?) {
        if (errorRequestUrl! == getServiveSearchFilter(.LOCATION))
            && (applicationDelegate.appSetting?.searchFilterLocationList.count > 0) {
            return
        }
        
        super.onConnectionError(errorRequestUrl)
    }

    //MARK: Delegate Update Save State : JobLocation 
    func saveStateupdateSelectedJobLocation(saveData: AnyObject?) {
        
        if(saveData?.isKindOfClass(SearchFilterCategory) == true) {
            let selectedLocation = saveData as? SearchFilterCategory
            parentController?.searchFilterSaveState?.selectedJobLocation = selectedLocation
            
            if (selectedLocation?.hasSub == false) {
                parentController?.searchFilterSaveState?.selectedSubJobLocation = nil
            }
            
        } else if(saveData?.isKindOfClass(SearchFilterSubCategory) == true) {
            let selectedSubLocation = saveData as? SearchFilterSubCategory
            parentController?.searchFilterSaveState?.selectedSubJobLocation = selectedSubLocation
        }
        
        tableView?.reloadRowsAtIndexPaths([NSIndexPath(forRow: quickSearchCellIndex.QUICK_LOCATION.rawValue, inSection: 0)], withRowAnimation: .None)
    }
    
    func getKeywordLocationFromSaveState() -> String {
        let selectedLocation = parentController?.searchFilterSaveState?.selectedJobLocation
        let selectedSubLocation = parentController?.searchFilterSaveState?.selectedSubJobLocation
        
        return getKeywordFromSaveState(selectedLocation, subFilter: selectedSubLocation, defaultKeyword: DEFAULT_KEYWORD_LOCATION)
    }
    
    func getLocationParameterFromSaveState() -> String {
        if let subLocation = parentController?.searchFilterSaveState!.selectedSubJobLocation {
            return subLocation.code!
        } else if let location = parentController?.searchFilterSaveState?.selectedJobLocation {
            return location.code!
        } else {
            return DEFAULT_KEYWORD_FOR_NIL_TYPE
        }
    }
    
    //MARK:- Call Service
    override func loadRequiredFilterListFromService() {
        super.loadRequiredFilterListFromService()
        loadLocationListFromService()
    }
    
    func loadLocationListFromService() {
        let lastUpdate = (appSetting?.searchFilterLocationLastUpdate)!
        let urlLocation = getServiveSearchFilter(SearchFilterCategoryType.LOCATION)
        
        loadSearchFilterListFromService(urlLocation, lastupdate: lastUpdate) { (jsonObject) -> Void in
            self.appSetting?.searchFilterLocationLastUpdate = (jsonObject.objectForKey("timestamps") as? Int)!
            let itemList = jsonObject.objectForKey("locations") as? NSArray
            
            self.appSetting?.searchFilterLocationList.removeAll()
            for item in itemList! {
                self.appSetting?.searchFilterLocationList.append(SearchFilterCategory(item: item as! NSMutableDictionary))
            }
        }
    }
    
    //MARK:- TableView
    override func configureCellForRowAtIndexPath(cell: JTMSeachFilterTableViewCell, indexPath: NSIndexPath) -> UITableViewCell {
        
        setSelectedBackgroundView(cell)
        
        switch quickSearchCellIndex(rawValue: indexPath.row)! {
            case .QUICK_CATEGORY:
                configureJobCategoryCell(cell)
                break
            case .QUICK_LOCATION:
                cell.setDescription(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_LOCATION"),
                    subTitle: getKeywordLocationFromSaveState())
                break
            case .QUICK_FOOTER:
                cellFooter = configureCellSearchFooter()
                cellFooter?.setCellSeperatorLineHide(tableView)
                return cellFooter!
            default:
                cellSearch = configureCellSearch()
                return cellSearch!
        }

        return cell
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return quickSearchCellIndex.count.rawValue
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
//        if (quickSearchCellIndex(rawValue: indexPath.row))! == .QUICK_FOOTER{
            return CELL_SIZE_BIG
//        } else {
//            return CELL_SIZE_SMALL
//        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        print("Click\(indexPath.row)")
        
        switch quickSearchCellIndex(rawValue: indexPath.row)! {
            case .QUICK_CATEGORY:
                openSearchFilter(SearchFilterCategoryType.CATEGORY)
                break
            case .QUICK_LOCATION:
                openSearchFilter(SearchFilterCategoryType.LOCATION)
                break
            default:
                break
        }
        
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func setSelectedBackgroundView(cell: JTMSeachFilterTableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
 
    //MARK:- Search
    override func footerOnClickSearch() {
        
        let jobType = getJobTypeParamaterFromSaveState()
        let subType = getSubJobTypeParamaterFromSaveState()
        let keyword = getKeywordParameterFromSaveState()
        let location = getLocationParameterFromSaveState()
        let disability = getDisabilitySaveState().hashValue
//        let parameter = "?jobtype=\(jobType)&subtype=\(subType)&keyword=\(keyword)&location=\(location)&disabled_person=\(disability)"
        
        let parameter: [String:AnyObject] = [
            PARAMETER_JOB_TYPE_KEY          : jobType,
            PARAMETER_SUB_JOB_TYPE_KEY      : subType,
            PARAMETER_KEYWORD_KEY           : keyword,
            PARAMETER_LOCATION_KEY          : location,
            PARAMETER_DISABLED_WELCOME_KEY  : disability
        ]
        
        self.setTrackGoogleAnalyticKeyword(keyword)
        self.setTrackGoogleAnalyticDisability(disability)
        
        openSearchResult(parameter ,isOpenMap: false , radiusInMeter: 0)
    }
    
    func setTrackGoogleAnalyticKeyword(keyword : String) {
        var action = ""
        
        if keyword.isEmpty {
            action = ACTION_SEARCH_NON_KEYWORD
        } else {
            action = ACTION_SEARCH_KEYWORD
        }
        
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    func setTrackGoogleAnalyticDisability(disability : Int) {
        var action = ""
        
        if disability == 1 {
            action = ACTION_SEARCH_DISABILITY
        } else {
            action = ACTION_SEARCH_NON_DISABILITY
        }
        
        // Track Google Analytic Event disability
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    
}
