
//
//  JTMSearchFilterCategoryTransportationViewController.swift
//  JobThai
//
//  Created by pinyo on 5/16/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import Foundation
import UIKit

class JTMSearchFilterCategoryTransportationViewController : JTMBaseViewController {
    
    
    let CELL_HEIGHT:CGFloat = 50
    
    @IBOutlet var tableView : UITableView!
    let appSetting = applicationDelegate.appSetting
    var parent: JTMBaseViewController?
    
    var searchFilterCategoryType: SearchFilterCategoryType?
    
    var selectedFilterTransportation : SearchFilterCategoryTransportation?
    var selectedFilter      : SearchFilterCategory?
    var selectedSubFilter   : SearchFilterSubCategory?
    
    var indexCurrent : Int = -1
    var indexSelector: Int = -1
    var indexSubSelector: Int = -1
    
    var saveFilter : SearchFilterSaveStateModel?
    
    var transportationListItem = [SearchFilterCategoryTransportation]()
    
    let QUICK_SEARCH = "JTMQuickSearchViewController"
    let NEAR_ME_SEARCH = "JTMJobNearMeViewController"
    let TRANSPORT_SEARCH = "JTMTransportationSearchViewController"
    let INDUSTRIAL_SEARCH = "JTMIndustrialSearchViewController"
    
    var saveDelegate : JTMUpdateSaveStateDelegate?
    
    func setTitleName(title: String) {
        self.title = title
    }
    
    func setSearchFilterModel(searchFilterModel: [SearchFilterCategoryTransportation]) {
        self.transportationListItem = searchFilterModel
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        print("JTMSearchFilterCategoryTransportationViewController")
        
        self.tableView.tableFooterView = UIView()
        self.tableView.accessibilityIdentifier = ID_SEARCH_FILTER_JOB_CATEGORIES_TRANSPORT_TABLE_VIEWS
        setNumberIndexSelector()
        
        self.noInternetView?.hidden = true
    }
    
    func setSelectedFilter(selectedFilter: SearchFilterCategory?) {
        self.selectedFilter = selectedFilter
    }
    
    func setSelectedSubFilter(selectedSubFilter: SearchFilterSubCategory?) {
        self.selectedSubFilter = selectedSubFilter
    }
    
    func setSelectFilterTransportModel(selectItem : SearchFilterCategoryTransportation?){
        self.selectedFilterTransportation = selectItem
    }
    
    func setNumberIndexSelector() {
        if let selectedCurrentIndex = transportationListItem.indexOf({$0.categoryName! == selectedFilterTransportation?.categoryName}) {
            indexCurrent = selectedCurrentIndex
            if let selectedCategoryIndex = transportationListItem[selectedCurrentIndex].line.indexOf({$0.code! == selectedFilter?.code}) {
                indexSelector = selectedCategoryIndex
                if let selectedSubIndex = transportationListItem[selectedCurrentIndex].line[selectedCategoryIndex].subCategoryList.indexOf({$0.code! == selectedSubFilter?.code}) {
                    indexSubSelector = selectedSubIndex
                }
            }
            
            dispatch_async(dispatch_get_main_queue(), { () -> Void in
                self.tableView.scrollToRowAtIndexPath(NSIndexPath(forRow: selectedCurrentIndex, inSection: 0), atScrollPosition: .Top, animated: false)
            })
        }
    }
    
    func setFilterCategoryType(searchFilterCategoryType: SearchFilterCategoryType) {
        self.searchFilterCategoryType = searchFilterCategoryType
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return CELL_HEIGHT
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchFilterCategoryTransportationTableViewCell")
        
        let searchFilter = transportationListItem[indexPath.row]
        
        setSelectedBackgroundView(cell!)
        
        cell?.textLabel?.text = searchFilter.categoryName
        
        if(searchFilter.line.count > 0){
            cell?.accessoryType = .DisclosureIndicator
        }
        
        if(indexCurrent == indexPath.row) {
            cell?.textLabel?.textColor = JTM_CORE_ORANGE
        } else {
            cell?.textLabel?.textColor = UIColor.blackColor()
        }
        
        setUpCellEdgeInset(cell!)
        return cell!
    }
    
    func setUpCellEdgeInset(cell : UITableViewCell){
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return transportationListItem.count
    }
    
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
        
        let searchFilter = transportationListItem[indexPath.row]
        
        print("Select : \(searchFilter.categoryName!)")
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_TRANSPORTATION, action: ACTION_CLICK_TRANSPORT, label: getLabelRoute(searchFilter.categoryName!))
        
        if(!searchFilter.hasSub!) {
            let searchFilterSubCategory : JTMSearchFilterSubCategoryViewController = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchFilterSubCategoryViewController") as? JTMSearchFilterSubCategoryViewController)!
            
            searchFilterSubCategory.parent = self
            searchFilterSubCategory.parentBeforeView = self
            searchFilterSubCategory.saveDelegate = saveDelegate
            searchFilterSubCategory.setFilterCategoryType(searchFilterCategoryType!)
            searchFilterSubCategory.setNumberIndexSelector(indexPath.row)
            
            if(indexPath.row == indexCurrent) {
                searchFilterSubCategory.setNumberIndexSubSelector(indexSubSelector)
            } else {
                searchFilterSubCategory.setNumberIndexSubSelector(-1)
            }
            
            searchFilterSubCategory.setTitleName(searchFilter.line.first!.name!)
            searchFilterSubCategory.setSearchFilterSubModel(searchFilter.line.first!, searchFilterSubModel: searchFilter.line.first!.subCategoryList)
            searchFilterSubCategory.setSearchFilterTransportModel(transportationListItem[indexPath.row])
            self.navigationController?.pushViewController(searchFilterSubCategory, animated: true)
            
        } else if(searchFilter.hasSub!) {
            
            let searchFilterCategory = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchFilterCategoryViewController") as? JTMSearchFilterCategoryViewController)!
            
            let searchFilterMainTransportationModel = transportationListItem[indexPath.row]
            searchFilterCategory.parent = self
            searchFilterCategory.saveDelegate = saveDelegate
            searchFilterCategory.setFilterCategoryType(SearchFilterCategoryType.TRANSPORTATION)
            searchFilterCategory.setTitleName(searchFilter.categoryName!)
            
            searchFilterCategory.setSearchFilterTransportModel(searchFilterMainTransportationModel)
            searchFilterCategory.setSearchFilterModel(searchFilterMainTransportationModel.line)
            
            if(indexPath.row == indexCurrent){
                searchFilterCategory.setSelectedFilter(saveFilter?.selectedTransportation)
                searchFilterCategory.setSelectedSubFilter(saveFilter?.selectedSubTransportation)
            } else {
                searchFilterCategory.indexSelector = -1
            }
            
            if(saveFilter?.selectedTransportation == nil) {
                searchFilterCategory.indexSelector = -1
            }
            self.navigationController?.pushViewController(searchFilterCategory, animated: true)
            
        } else {
            saveDelegate?.saveStateupdateSelectedTransportation?(searchFilter)
            self.navigationController?.popViewControllerAnimated(true)
        }
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
}