//
//  JTMSearchResultMapViewController.swift
//  JobThai
//
//  Created by weerapon on 2/17/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMSearchResultMapViewController: JTMBaseViewController , TNMapViewDelegate , JTMSearchResultMapFooterControlViewDelegate{
    @IBOutlet var pageFooterControl : JTMSearchResultMapFooterControlView!
    let LOCATION_FIRST_LOAD = CLLocationCoordinate2D(latitude: 9.7421681554298427, longitude: 106.76919245678513)
    let pageIndex : SearchResultPageIndex = .MAP_VIEW
    var mapDisplayed        : Bool  = false
    var isOverViewed        : Bool  = true
    
    //Map
    var mapCountAllPage                 = 1
    var mapPageIndex                    = 1
    var radiusNearMeInMeter : Double    = 0
    var savedMapPageBeforeTryLoadService: Int?
    
    //Map Radius Circle Near Me
    let CIRCLE_NEAR_ME_FILL_COLOR : UIColor = UIColor(colorLiteralRed: 0.8, green: 0.5 , blue: 0.0 , alpha: 0.1)
    let CIRCLE_NEAR_ME_LINE_COLOR : UIColor = UIColor(colorLiteralRed: 1.0, green: 0.5 , blue: 0.0 , alpha: 0.5)
    let CIRCLE_NEAR_ME_LINE_WIDTH : CGFloat = 5.0
    
    //Location
    let locationManager           = CLLocationManager()
    
    // parent
    var searchResultController  : JTMSearchResultViewController!
    var delegateMainSeachResult : JTMSeachResultClickSubViewDelegate!
    
    //TNMapView
    @IBOutlet var mapView : TNMapView!
    
    //CopyRight
    @IBOutlet var copyRightLogoImageView : UIImageView!
    @IBOutlet var copyRightNameImageView : UIImageView!
    
    //All Annotation Bounding & Event Touch
    var zoomBoundingBox     : RMProjectedRect?
    var isCallingService    : Bool = false
    var isLongPressOnMap    : Bool = false
    var isUserTouchingOnMap : Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        //Clear Map Annotation Data
        searchResultController.mapSearchResultData = nil
        self.mapView.removeAllAnnotation()
        
        self.addMapToCurrentViewController()
        self.addCircleNearMe()
        self.pageFooterControl.delegate = self
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        if searchResultController.mapSearchResultData == nil{
            self.delegateMainSeachResult.hideSearchMainPosition(self)
        }else{
            self.delegateMainSeachResult.setSeachMainPosition(self, position: (searchResultController.mapSearchResultData?.countAllJob)!)
        }
        
        self.showUserLocation(true)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
//        initNoResultView()
        
        
        print("SearchResultMap  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_SEARCH_RESULT_MAP)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_MAP, action: ACTION_VIEW, label: LABEL_VIEW_RESULT_MAP)
    }
    
    override func setControllerFlags() {
        isPresentedInOtherController = true
    }
    
//    func initNoResultView() {
//        if (self.noResultView == nil) {
//            let mainSearchResultController = searchResultController.seachResultMainSeachResult
//            let noResultViewHeight = DEFAULT_FRAME_HEIGHT - mainSearchResultController.TAB_BAR_SHOW_HEIGHT
//            self.noResultView = JTMNoResultView(frame: CGRect(x: 0, y: 0, width: DEFAULT_FRAME_WIDTH, height: noResultViewHeight))
//            self.view.addSubview(self.noResultView!)
//        }
//    }
    
    //MARK:- Reachability change update
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        setButtonEnableStatus(true)
    }
    
    override func manageViewAfterInternetNotReachable() {
        super.manageViewAfterInternetNotReachable()
        setButtonEnableStatus(false)
    }
    
    //MARK:- On Retry Connection
    override func onConnectionSuccess() {
        super.onConnectionSuccess()
        isHasRetryButtonOnAlertConnectionError = (searchResultController.mapSearchResultData == nil) //Can retry to reload if result are nil
    }
    
    override func onRetryConnection() {
        super.onRetryConnection()
        searchResultController.seachResultMainSeachResult.connectionErrorView?.alpha = VIEW_HIDE_ALPHA
        if (searchResultController.mapSearchResultData == nil) {
            mapView.reloadMapView()
            callService(self.mapPageIndex)
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (searchResultController.mapSearchResultData == nil) {
            if (view == noInternetView) {
                searchResultController.seachResultMainSeachResult.noInternetView?.alpha = VIEW_SHOW_ALPHA
            } else {
                searchResultController.seachResultMainSeachResult.connectionErrorView?.alpha = VIEW_SHOW_ALPHA
            }
            return
        }
        
        if (savedMapPageBeforeTryLoadService != nil) {
            mapPageIndex = savedMapPageBeforeTryLoadService!
        }
    }
    
    func setButtonEnableStatus(enable: Bool) {
        pageFooterControl.setButtonEnableStatus(enable)
    }
    
    //MARK: Observer for Check Application Active
    override func applicationDidActive(){
        let currentLevel = self.mapView.getMapLevel()
        self.showUserLocation(true)
        self.mapView.setMapLevel(currentLevel)
    }
    
    override func onLanguageChange() {
        super.onLanguageChange()
//        self.setMapTileCurrentLanguage()
    }
    
    //MARK: Call Service & Parse Json Data
    func callService( mapPage : Int) {//-> Bool {
        guard !isCallingService else {
            print("Now Is Calling Service Please wait....")
            return
        }
        print("Calling Service : \(mapPage)")
        isCallingService = true
        let serviceController = JTMServiceController()
        let url = getServiceSearchResult()
        
        var params = self.searchResultController.getParamForSearch()
        params["page"] = mapPage
        params["pinlist"] = 1
        
        if (mapPage == 1 && (searchResultController.mapSearchResultData == nil)) {
            self.delegateMainSeachResult.hideSearchMainPosition(self)
        }
        
        //Zoom to All Annotation in Map and Set Center to Bangkok
        if searchResultController.mapSearchResultData == nil{
            self.setCenterMapViewFirstLoad()
        }
        
        self.delegateMainSeachResult.setButtonEnableStatus(false)
        
        serviceController.requestService(self, urlString: url, params: params) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.parseJsonToModel(jsonObject! , pageIndex:  mapPage)
                self.addAnnotationCurrentPageToMap()
                self.delegateMainSeachResult.setSeachMainPosition(self, position: (self.searchResultController.mapSearchResultData?.countAllJob)!)
                
                if(self.searchResultController.mapSearchResultData != nil){
                    self.delegateMainSeachResult.setSeachMainPosition(self, position: (self.searchResultController.mapSearchResultData?.countAllJob)!)
                }
            } else if (status == .CONNECTION_UNREACHABLE) {
                if !self.searchResultController.isFirstNotConnectInternet {
                    self.searchResultController.seachResultMainSeachResult.showAlertNoNetworkIsReachable()
                    self.searchResultController.isFirstNotConnectInternet = true
                }
            }
            
            self.delegateMainSeachResult.setButtonEnableStatus(true)
            self.isCallingService = false
        }
    }
    
    func parseJsonToModel(jsonResult : NSMutableDictionary , pageIndex : Int) {
        if (searchResultController.mapSearchResultData == nil) {
            searchResultController.mapSearchResultData = SearchResultMapModel(result: jsonResult, currentPage: pageIndex)
            self.mapCountAllPage = (searchResultController.mapSearchResultData!.countPage)!
            
//            self.noResultView?.hidden = (JTMSearchResultViewController.mapSearchResultData?.countAllJob == 0) ? false : true
            if (searchResultController.mapSearchResultData?.countAllJob == 0) {
                self.delegateMainSeachResult.displayNoResult()
            }
        }
        else{
            searchResultController.mapSearchResultData?.insertNewJobList(jsonResult, currentPage: pageIndex)
            
//            self.noResultView?.hidden = ((JTMSearchResultViewController.mapSearchResultData?.jobAllPage.objectForKey(pageIndex) as? NSMutableDictionary)?.count == 0) ? false : true
            if ((searchResultController.mapSearchResultData?.jobAllPage.objectForKey(pageIndex) as? NSMutableDictionary)?.count == 0) {
                self.delegateMainSeachResult.displayNoResult()
            }
        }
    }
    
    //MARK: Inital MapView Engine
    func addMapToCurrentViewController(){
        self.mapView.delegate = self
        if (searchResultController.mapSearchResultData?.jobAllPage.objectForKey(self.mapPageIndex) as? NSMutableDictionary) == nil{
            self.callService(self.mapPageIndex)
        }else{
            self.addAnnotationCurrentPageToMap()
        }
        print("View Frame : \(self.view.frame)")
        addCopyRightToMapView()
    }
    
    func addCopyRightToMapView(){
        setMapViewCopyRightLanguage(TNMapLanguage.init((applicationDelegate.appSetting?.currentLanguage.rawValue)!))
    }
    
    func setMapViewCopyRightLanguage(language : TNMapLanguage){
        if(self.mapView.mapMagicImg == nil){
            mapView.initialCopyRightImage()
        }
        
        copyRightLogoImageView.image = mapView.mapMagicImg
        
        if(language == TNMapLanguageThai){
            copyRightNameImageView.image = mapView.copyRightThImg
        }else{
            copyRightNameImageView.image = mapView.copyRightEnImg
        }
    }
    
    func zoomToAllAnnotation() {
        guard let zoomBound = self.zoomBoundingBox else {
            print("Can not Zoom")
            self.mapView.zoomToFitAllAnnotationWithOutUserLocation()
            return
        }
        print("Found Bounding Box :: \(zoomBound)")
        self.mapView.zoomToBound(zoomBound)
    }
    
    //for set show user location in mapview.
    func showUserLocation(show : Bool){
        let status = CLLocationManager.authorizationStatus()
        if CLLocationManager.locationServicesEnabled() {
            switch (status) {
            case .NotDetermined :
                break
            case .Denied :
                break
            default :
                //Set show user location
                self.mapView.setShowUserLocation(true)
                let currentZoomLevel = self.mapView.getMapLevel()
                self.mapView.setUserTrackingMode(RMUserTrackingModeNone)
                self.mapView.setMapLevel(currentZoomLevel)
                self.mapView.setUserLocationEnable(false)
                break
            }
        }
    }
    
    //Func Create Annotation for MapView
    func addAnnotationCurrentPageToMap(){
        self.showLoading()
        
        self.delegateMainSeachResult.setSeachMainPosition(self, position: (searchResultController.mapSearchResultData?.countAllJob)!)
        self.mapView.removeAllAnnotation()
        
        if let jobListAllPinID = (searchResultController.mapSearchResultData?.jobAllPage.objectForKey(self.mapPageIndex) as? NSMutableDictionary) {
            var allAnnotations = [RMAnnotation]()
            for ( _ ,value) in jobListAllPinID{
                if let annotation = self.createAnnotation((value as! NSMutableArray), numJobLine: (value as! NSMutableArray).count){
                    allAnnotations.append(annotation)
                }
            }
            self.mapView.addAnnotationsToMapView(allAnnotations)
            zoomBoundingBox = self.mapView.getBoundsAllAnnotation()
            self.pageFooterControl.setCurrentPageLabel(self.mapPageIndex, numOfPage: self.mapCountAllPage)
            
            //Zoom to All Annotation in Map and Set Center to Bangkok
            if searchResultController.mapSearchResultData != nil{
                self.zoomToAllAnnotation()
            }
        }
        self.hideLoading()
        self.savedMapPageBeforeTryLoadService = nil
        
        self.searchResultController.enableSwipePageView(true)
    }
    
    func convertJobItemListToString(jobList : NSMutableArray) -> [String]?{
        var stringArr = [String]()
        if let jobList = jobList as NSArray as? [JobItem]{
            stringArr = jobList.map({$0.jobTitle!})
        }
        return stringArr
    }
    
    func createAnnotation( jobList : NSMutableArray, numJobLine : Int) -> RMAnnotation?{
        //Get First Annotation
        if jobList.count > 0 {
            let firstJobItem : JobItem
            firstJobItem = jobList.objectAtIndex(0) as! JobItem
            
            //Inital Annotation and Add to MapView
            let jobStr = self.convertJobItemListToString(jobList)
            let jobCoordinate = CLLocationCoordinate2DMake(firstJobItem.latitude!, firstJobItem.longitude!)
            let jobAnnotation : RMAnnotation
            jobAnnotation = self.mapView.createAnnotationWithJobList(firstJobItem.companyName!, jobList: jobStr!, coordinate: jobCoordinate, userInfo: jobList)
            return jobAnnotation
        }
        return nil
    }
    
    func addCircleNearMe(){
        if self.mapView != nil && self.radiusNearMeInMeter > 0 {
            self.mapView.setCircleColor(fillColor: CIRCLE_NEAR_ME_FILL_COLOR, andLineColor: CIRCLE_NEAR_ME_LINE_COLOR, andLineWidth: CIRCLE_NEAR_ME_LINE_WIDTH)
            self.mapView.setCircleNearMeAnnotation(radiusInMeter: self.radiusNearMeInMeter)
        }
    }

    //MARK: PageFooterControl Delegate
    func footerControlCurrentLocationButtonClicked() {
        guard !isLongPressOnMap && !isUserTouchingOnMap else {
            print("Now you longPress on Map[\(isLongPressOnMap)].\nAnd Touching On Map [\(isUserTouchingOnMap)].\nPlease release finger on screen.\n")
            return
        }
        
        let status = CLLocationManager.authorizationStatus()
        if CLLocationManager.locationServicesEnabled() {
            switch (status) {
            case .NotDetermined :
                if CLLocationManager.locationServicesEnabled() {
                    locationManager.requestWhenInUseAuthorization()
                } else {
                    alertNoPermissionToAccess()
                }
                break
            case .Denied :
                alertNoPermissionToAccess()
                break
            default :
                self.showUserLocation(true)
                self.mapView.setUserTrackingMode(RMUserTrackingModeNone)
                self.mapView.setCenterMapToUserLocation(true)
                break
            }
        } else {
            alertNoOpenLocationService()
        }
    }
    
    func footerControlBackPageButtonClicked() {
        if(self.mapPageIndex > 1 && self.mapPageIndex <= self.mapCountAllPage && !isCallingService){
            savedMapPageBeforeTryLoadService = mapPageIndex
            self.mapPageIndex -= 1
            if (searchResultController.mapSearchResultData?.jobAllPage.objectForKey(self.mapPageIndex) as? NSMutableDictionary) == nil{
                self.callService(self.mapPageIndex)
            }
            else{
                self.addAnnotationCurrentPageToMap()
            }
        }
    }
    
    func footerControlNextPageButtonClicked() {
        if(self.mapPageIndex >= 1 && self.mapPageIndex < self.mapCountAllPage && !isCallingService){
            savedMapPageBeforeTryLoadService = mapPageIndex
            self.mapPageIndex += 1
            if (searchResultController.mapSearchResultData?.jobAllPage.objectForKey(self.mapPageIndex) as? NSMutableDictionary) == nil{
                self.callService(self.mapPageIndex)
            }
            else{
                self.addAnnotationCurrentPageToMap()
            }
        }
    }
    
    func setMapTileCurrentLanguage(){
        let mapLanguage : TNMapLanguage
        mapLanguage = getCurrentApplicationLanguage()
        
        if mapLanguage != mapView.mapLanguage {
            self.mapView.setMapTileLanguage(mapLanguage)
            self.setMapViewCopyRightLanguage(mapLanguage)
        }
        
    }
    
    func setCenterMapViewFirstLoad(){
        if(self.mapView.mapViewThinknet != nil){
            self.view.layoutIfNeeded()
            self.mapView.setCenterMapToCoordinate(self.mapView.LOCATION_BANGKOK, animated: false)
        }
    }
    
    func getCurrentApplicationLanguage() -> TNMapLanguage {
        var mapLanguage : TNMapLanguage
        if applicationDelegate.appSetting?.currentLanguage == appLanguage.THAI{
            mapLanguage = TNMapLanguageThai
        }else{
            mapLanguage = TNMapLanguageEnglish
        }
        return mapLanguage
    }
    
    //MARK: TNMapView Delegate
    func mapViewSingleTapOnMap(map: RMMapView, at point: CGPoint) {
        print("single tap on map")
    }
    
    func mapViewTapOnCalloutAccessoryControl(control: UIControl, forAnnotation annotation: RMAnnotation, onMap map: RMMapView) {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_MAP, action: ACTION_CLICK, label: LABEL_INFO_MAP)
        let selectedAnnotation = self.mapView.getSelectAnnotation()
        let selectedJobList = (selectedAnnotation.userInfo as? NSMutableArray)//?.firstObject as? JobItem
        self.delegateMainSeachResult?.onClickCallOut(self, item: selectedJobList!)//selectedJob!)
    }
    
    func mapViewLongPressOnMap(map: RMMapView, at point: CGPoint, touching: Bool) {
        print("Long Press : \(touching)")
        isLongPressOnMap = touching
    }
    
    func mapViewMovebyUser(map: RMMapView, byUser wasUserAction: Bool, userTouching: Bool) {
        isUserTouchingOnMap = userTouching
        print("User move touching : \(userTouching)")
    }
    
}
