//
//  JTMSearchViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/3/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import Foundation
import UIKit

class JTMSearchViewController : UIView , UISearchControllerDelegate , UISearchResultsUpdating , UISearchBarDelegate {
 
    let HIDE_SEGMENT_TOP_SPACE: CGFloat = 0
    let SHOW_SEGMENT_TOP_SPACE: CGFloat = 52
    let HIDE_SEGMENT_VIEW_HEIGHT: CGFloat = 44
    let SHOW_SEGMENT_VIEW_HEIGHT: CGFloat = 88
    let SEARCH_BAR_HEIGHT: CGFloat = 44
    
    let searchController = UISearchController(searchResultsController: nil)
    @IBOutlet weak var segment: UISegmentedControl!
    @IBOutlet weak var segmentTopSpace : NSLayoutConstraint!
    
    var view: UIView!
    var delegate : JTMSearchViewControllerDelegate!
    var noInternetTabView: UIView?
    
    //MARK: Load Nib File
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupUi()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupUi()
    }
    
    
    func setupUi(){
        view = loadViewFromNib()
        view.frame = bounds
        view.autoresizingMask = [.FlexibleWidth , .FlexibleHeight]
        addSubview(view)
        
        //Add SearchBar Controller
        self.initSearchController()
    }
    
    func setNoInternetView(noInternetTabView: UIView) {
        if (self.noInternetTabView == nil) {
            self.noInternetTabView = noInternetTabView
            searchController.searchBar.addSubview(self.noInternetTabView!)
        }
    }
    
    func updateViewWhenReachabilityChange() {
        if (noInternetTabView != nil) {
            searchController.searchBar.bringSubviewToFront(noInternetTabView!)
        }
    }

    func setUpSegmentTitle( titleList : [String]){
        setUpSegmentUI()
        self.segment.removeAllSegments()
        for i in 0 ..< titleList.count {
            self.segment.insertSegmentWithTitle(titleList[i], atIndex: i, animated: false)
        }
    }
    
    func setUpSegmentUI() {
        self.segment?.tintColor = JTM_SEGMENT_SELECTED
        self.segment?.setTitleTextAttributes([NSForegroundColorAttributeName:JTM_SEGMENT_TEXT_COLOR], forState: UIControlState.Normal)
        self.segment?.setTitleTextAttributes([NSForegroundColorAttributeName:JTM_SEGMENT_TEXT_COLOR], forState: UIControlState.Selected)
    }
    
    @IBAction func segmentIndexChange(sender: AnyObject) {
        let index = self.segment.selectedSegmentIndex
        let segmentTitle = self.segment.titleForSegmentAtIndex(index)
        print("segmentTitle  :: \(segmentTitle!)  index  :: \(index) ")
        delegate?.segmentOfJTMSearchViewIndexChange(index, categoryName: segmentTitle!)
        
    }
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMSearchViewController", bundle: bundle)
        let view = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    // MARK: Animation for Hidden Segment
    func hideSegment(view : UIView ,constraintViewHeight : NSLayoutConstraint , status : Bool){
        if(status){
            //Hide Segment
            self.segmentTopSpace.constant = HIDE_SEGMENT_TOP_SPACE
            constraintViewHeight.constant = HIDE_SEGMENT_VIEW_HEIGHT
            UIView.animateWithDuration(0.3, animations: { () -> Void in
                view.layoutIfNeeded()
                }, completion: { (Bool) -> Void in
                    self.segment.hidden = true
            })
        }
        else{
            //Show Segment
            self.segment.hidden = false
            self.segmentTopSpace.constant = SHOW_SEGMENT_TOP_SPACE
            constraintViewHeight.constant = SHOW_SEGMENT_VIEW_HEIGHT
            UIView.animateWithDuration(0.3){
                view.layoutIfNeeded()
            }
        }
    }
    
    // MARK: Deleate Seach and Segment
    func initSearchController() {
        searchController.dimsBackgroundDuringPresentation = false
        self.searchController.hidesNavigationBarDuringPresentation = false
        self.searchController.searchResultsUpdater = self
        self.searchController.searchBar.delegate = self
        self.searchController.searchBar.frame = CGRectMake(0,0,UIScreen.mainScreen().bounds.width,SEARCH_BAR_HEIGHT)
        self.setSearchBarLanguage()
        
        self.view.addSubview(self.searchController.searchBar)
    }
    
    func setSearchBarLanguage() {
        self.searchController.searchBar.placeholder = applicationDelegate.getLocalizeStringForKey("Search")
        self.searchController.searchBar.setValue(applicationDelegate.getLocalizeStringForKey("Cancel"), forKey: "_cancelButtonText")
    }
    
    func searchBar(searchBar: UISearchBar, selectedScopeButtonIndexDidChange selectedScope: Int) {
        
    }
    
    func searchBarCancelButtonClicked(searchBar: UISearchBar) {
        delegate?.searchBarOfJTMSearchViewCancelButtonClicked(searchBar)
    }
    
    func updateSearchResultsForSearchController(searchController: UISearchController) {
        delegate?.searchBarOfJTMSearchViewResultUpdate(searchController)
    }
    
    func searchBarShouldBeginEditing(searchBar: UISearchBar) -> Bool {
        delegate?.searchBarOfJTMSearchViewBeginEditing(searchBar)
        return true
    }
}

// MARK: Protocol
protocol JTMSearchViewControllerDelegate{
    func searchBarOfJTMSearchViewResultUpdate(searchController : UISearchController)
    func searchBarOfJTMSearchViewCancelButtonClicked(searchbar : UISearchBar)
    func searchBarOfJTMSearchViewBeginEditing(searchBar : UISearchBar)
    func segmentOfJTMSearchViewIndexChange(index : Int , categoryName : String)
}
