//
//  JTMTransportaionVeiwController.swift
//  JobThai
//
//  Created by weerapon on 1/21/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMTransportaionViewController : JTMBaseViewController , UITableViewDataSource, UITableViewDelegate ,JTMSearchViewControllerDelegate{

    let SECTION_HEADER_HEIGHT:CGFloat = 44
    let CATEGORY_CODE_ARL = "ARL"
    
    var transportationData = NSMutableDictionary()
    var categoryTransportation = [String]()
    
    @IBOutlet weak var transportationTableView : UITableView?
    @IBOutlet var searchView : JTMSearchViewController!
    @IBOutlet var searchViewContraintHeight : NSLayoutConstraint!
    @IBOutlet var searchMessageLabel : UILabel?
    
    var allLineTransportation = [TransportationLineModel]()
    var filterSearchResultBySection = NSMutableArray()
    
    var selectIndex : Int!
    var selectName : String!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.searchMessageLabel?.hidden = true
        self.transportationTableView?.tableFooterView = UIView(frame: CGRectZero)
        self.transportationTableView?.accessibilityIdentifier = ID_BROWSE_JOBS_TRANSPORTATION_TABLE_VIEW
        connectService()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("Transportaion viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_METRO_SEARCH)
        
        self.searchMessageLabel?.text = applicationDelegate.getLocalizeStringForKey("SEARCH_NO_RESULT_MESSAGE")
        self.searchView.setNoInternetView(self.noInternetTabView!)
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        self.searchView.searchController.dismissViewControllerAnimated(false, completion: nil)
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- Reachable
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        searchView.updateViewWhenReachabilityChange()
    }
    
    override func manageViewAfterInternetNotReachable() {
        super.manageViewAfterInternetNotReachable()
        searchView.updateViewWhenReachabilityChange()
    }
    
    //MARK:- On Retry Connection
    override func onRetryConnection() {
        super.onRetryConnection()
        if (transportationData.count == 0) {
            connectService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (transportationData.count == 0) {
            setAlphaShowToView(view)
        }
    }
    
    //MARK:-
    func connectService(){
        
        let serviceController = JTMServiceController()
        let url = getServiceTransportation()
        
        serviceController.requestService(self, urlString: url, params: nil) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.convertJsonToModel(jsonObject!)
                self.setUpSearchUI(self.categoryTransportation)
                
                //Init SelectIndex & Name
                self.selectIndex = 0
                self.selectName = self.categoryTransportation[0]
                self.searchView.segment.selectedSegmentIndex = self.selectIndex
                self.transportationTableView?.reloadData()
            }
        }
    }
    
    func convertJsonToModel(jsonObject : NSMutableDictionary) {
        self.categoryTransportation = jsonObject.objectForKey("categoriesTitle") as! [String]
        let transportationList = jsonObject.objectForKey("transportation") as? NSArray
        allLineTransportation.removeAll()
        transportationData.removeAllObjects()
        
        for item in transportationList!{
            let object = TransportationModel(item: (item as? NSMutableDictionary)!)
            transportationData.setValue(object, forKey: object.name!)
        }
        
        //Sort & Get All Line List
        for name in self.categoryTransportation{
            if let transportationItem = transportationData.objectForKey(name) as? TransportationModel {
                for lineItem in transportationItem.lineList{
                    lineItem.stationList.sortInPlace{$0.orderNumber < $1.orderNumber}
                }
                allLineTransportation = allLineTransportation + (transportationItem.lineList)
            }
        }
    }

    // MARK: JTMSearchView Delegate
    func setUpSearchUI(categoryList : [String]){
        searchView.delegate = self
        searchView.setUpSegmentTitle(categoryList)
    }
    
    func searchBarOfJTMSearchViewCancelButtonClicked(searchbar: UISearchBar) {
        self.searchView.searchController.searchBar.text = ""
        self.searchView.hideSegment(self.view, constraintViewHeight: searchViewContraintHeight, status: false)
    }
    func searchBarOfJTMSearchViewResultUpdate(searchController: UISearchController) {
        self.filterContentForSearchText(searchController)
    }
    
    func searchBarOfJTMSearchViewBeginEditing(searchBar: UISearchBar) {
        self.searchView.hideSegment(self.view, constraintViewHeight:  searchViewContraintHeight , status: true)
    }
    
    func segmentOfJTMSearchViewIndexChange(index: Int, categoryName: String) {
        selectIndex = index
        selectName = categoryName
        
        self.transportationTableView?.reloadData()
        
        let scrollTopIndex = NSIndexPath(forRow: 0, inSection: 0)
        self.transportationTableView?.scrollToRowAtIndexPath(scrollTopIndex, atScrollPosition: .Top, animated: false)
    }
    
    func filterContentForSearchText(searchController : UISearchController) {
        //Filter Data by Section
        var resultCount : Int = 0
        filterSearchResultBySection.removeAllObjects()
        
        for lineItem in allLineTransportation {
            let stationResult = lineItem.stationOrderList.filter({ (station : TransportationStationModel) -> Bool in
                let stringMatch = station.name?.lowercaseString.containsString(searchController.searchBar.text!.lowercaseString)
                return stringMatch!
            })
            resultCount += stationResult.count
            let line = TransportationLineModel(name: lineItem.name!, list: stationResult ,category: lineItem.categoryName!)
            filterSearchResultBySection.addObject(line)
        }
        
        self.transportationTableView?.hidden = false
        if resultCount == 0 && searchController.searchBar.text != ""{
            searchMessageLabel?.hidden = false
            self.transportationTableView?.hidden = true
        }else{
            searchMessageLabel?.hidden = true
        }
        self.transportationTableView?.reloadData()
    }
    
    /*
    * MARK: Delegate TableView
    */
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
//        self.selectionStyle = UITableViewCellSelectionStyleNone

        let cell = tableView.dequeueReusableCellWithIdentifier("JTMTransportationCell", forIndexPath: indexPath)
        self.setTableViewCellInset(cell)
        if(self.searchView.searchController.searchBar.text != ""){
            if(self.filterSearchResultBySection.count > 0 ){
                let station = (self.filterSearchResultBySection[indexPath.section] as? TransportationLineModel)?.stationList[indexPath.row]
                cell.textLabel?.text = station?.name
            }
        }else{
            let transportationLineList : [TransportationLineModel] = ((self.transportationData.objectForKey(selectName) as? TransportationModel)?.lineList)!
            let station = transportationLineList[indexPath.section].stationList[indexPath.row]
            cell.textLabel?.text = station.name
        }
        
        cell.selectionStyle = .None
        
        return cell
    }
    
    func setTableViewCellInset(cell : UITableViewCell){
        // Initialization code set Separator
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
    }
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        if(self.searchView.searchController.searchBar.text != ""){
            if self.filterSearchResultBySection.count > 0 {
                return self.filterSearchResultBySection.count
            }
        }else{
            if self.transportationData.count > 0 {
                let item = self.transportationData.objectForKey(selectName) as? TransportationModel
                if item?.lineList.count == 1{
                    return 1
                }else{
                    return (item?.lineList.count)!
                }
            }
        }
        return 0
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if(self.searchView.searchController.searchBar.text != ""){
            if(self.filterSearchResultBySection.count > 0 ){
                let stationList = (self.filterSearchResultBySection[section] as? TransportationLineModel)?.stationList
                return (stationList?.count)!
            }
        }else{
            if self.transportationData.count > 0 {
                let lineList = (self.transportationData.objectForKey(selectName) as? TransportationModel)?.lineList[section]
                return (lineList?.stationList.count)!
            }
        }
        return 1
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        let searchResultController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainSearchResultViewController") as? JTMMainSearchResultViewController
        
        if(self.searchView.searchController.searchBar.text != ""){
            if(self.filterSearchResultBySection.count > 0 ){
                let station = (self.filterSearchResultBySection[indexPath.section] as? TransportationLineModel)?.stationList[indexPath.row]
                searchResultController?.params = [
                    PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                    PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                    PARAMETER_MAIN_TRANSPORTATION_KEY   : station!.transportId!,
                    PARAMETER_SUB_TRANSPORTATION_KEY    : station!.code!,
                    PARAMETER_RADIUS_KEY                : DEFAULT_PARAMETER_RADIUS
                ]
                    
                    
                    //PARAMETER_DEFAULT + PARAMETER_MAIN_TRANSPORTATION + "\(station!.transportId!)" + PARAMETER_SUB_TRANSPORTATION + station!.code! + PARAMETER_RADIUS_DEFAULT
            }
        }else{
            let transportationLineList : [TransportationLineModel] = ((self.transportationData.objectForKey(selectName) as? TransportationModel)?.lineList)!
            let station = transportationLineList[indexPath.section].stationList[indexPath.row]
            searchResultController?.params = [
                PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                PARAMETER_MAIN_TRANSPORTATION_KEY   : station.transportId!,
                PARAMETER_SUB_TRANSPORTATION_KEY    : station.code!,
                PARAMETER_RADIUS_KEY                : DEFAULT_PARAMETER_RADIUS
            ]
            
                //PARAMETER_DEFAULT + PARAMETER_MAIN_TRANSPORTATION + "\(station.transportId!)" + PARAMETER_SUB_TRANSPORTATION + station.code! + PARAMETER_RADIUS_DEFAULT
        }
        
        searchResultController!.isOpenFromSearchMap = true
        
        self.navigationController?.pushViewController(searchResultController!, animated: true)
    }
    
    func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        
        if(self.searchView.searchController.searchBar.text != ""){
            if self.filterSearchResultBySection.count > 0 {
                let stationList = (self.filterSearchResultBySection[section] as? TransportationLineModel)?.stationList
                if stationList?.count == 0{
                    return 0
                }else{
                    return SECTION_HEADER_HEIGHT
                }
            }
        }else{
            if self.transportationData.count > 0 {
                let item = self.transportationData.objectForKey(selectName) as? TransportationModel
                if item?.lineList.count == 1{
                    return 0
                }else{
                    return SECTION_HEADER_HEIGHT
                }
            }
        }
        return 0
    }

    func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let nib = UINib(nibName: "JTMTransportationHeaderViewCell", bundle: nil)
        let headerView = nib.instantiateWithOwner(self, options: nil)[0] as? JTMTransportationHeaderViewCell
        
        if (self.searchView.searchController.searchBar.text != "") && (self.filterSearchResultBySection.count > 0) {
            let lineItem = (self.filterSearchResultBySection[section] as? TransportationLineModel)
            if((lineItem?.categoryName)! != (lineItem?.name)!){
                var headerString : String
                if(lineItem?.categoryName == CATEGORY_CODE_ARL){
                    headerString = "\((lineItem?.name)!)"
                }else{
                    headerString = "\((lineItem?.categoryName)!) \((lineItem?.name)!)"
                }
                headerView?.setTransportationTableHeaderView(headerString)
                
            }else{
                headerView?.setTransportationTableHeaderView((lineItem?.name)!)
                
            }
            return headerView
        }else{
            if self.transportationData.count > 0 {
                let lineListData = (self.transportationData.objectForKey(selectName) as? TransportationModel)?.lineList
                headerView?.setTransportationTableHeaderView(lineListData![section].name!)
                return headerView
            }
        }
        return nil
    }
    
    func tableView(tableView: UITableView, didHighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = JTM_SELECTED_CELL_BORDER_COLOR.colorWithAlphaComponent(0.4)
    }
    
    func tableView(tableView: UITableView, didUnhighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = UIColor.clearColor()
    }
    
    func scrollViewWillBeginDragging(scrollView: UIScrollView) {
        self.searchView.searchController.searchBar.resignFirstResponder()
    }
}