//
//  JTMTransportationSearchViewController.swift
//  JobThai
//
//  Created by weerapon on 3/10/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMTransportationSearchViewController: JTMBaseSearchViewController, JTMAreaTransportationDelegate {

    enum transportationSearchCellIndex: Int {
        case TRANSPORT_CATEGORY
        case TRANSPORT_TRANSPORTATION
        case TRANSPORT_KEYWORD
        case TRANSPORT_RADIUS
        case TRANSPORT_FOOTER
        case count
    }
    
    let TRANSPORT_NUMBER_OF_CELL: CGFloat = 4.25 //3 cell + 1 extra cell (transportation area)
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func isHasAllRequireFilterListValue() -> Bool {
        if (applicationDelegate.appSetting?.searchFilterTransportationList.count > 0) {
            return super.isHasAllRequireFilterListValue()
        }
        
        return false
    }
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("TransportationSearch  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_SEARCH_TRANSPORTATION)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_TRANSPORTATION, action: ACTION_VIEW, label: LABEL_FORM_JOB_SEARCH_BY_TRANSPORTATION)
        
        self.tableView.reloadData()
    }
    
    
    //MARK:- Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if (applicationDelegate.appSetting?.searchFilterTransportationList.count == 0) {
            loadTransportationListFromService()
        }
    }
    
    override func onConnectionError(errorRequestUrl: String?) {
        if (errorRequestUrl! == getServiveSearchFilter(.TRANSPORTATION))
            && (applicationDelegate.appSetting?.searchFilterTransportationList.count > 0) {
                return
        }
        
        super.onConnectionError(errorRequestUrl)
    }
    
    // Mark :: Set Title
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_TRANSPORTATION_SEARCH")
    }
    
    // Mark :: Register Nib File
    override func registerNibForTableView() {
        super.registerNibForTableView()
        tableView?.registerNib(UINib(nibName: "JTMTransportationRadiusTableViewCell", bundle: nil), forCellReuseIdentifier: "JTMTransportationRadiusTableViewCell")
    }

    //MARK: Delegate Update Save State : Selected Transportation
    func saveStateupdateSelectedTransportation(saveData: AnyObject?) {
        
        if(saveData?.isKindOfClass(SearchFilterCategoryTransportation) == true) {
            let selectMainTransportation = saveData as? SearchFilterCategoryTransportation
            parentController?.searchFilterSaveState?.selectedMainTransportation = selectMainTransportation
            
        } else if(saveData?.isKindOfClass(SearchFilterCategory) == true) {
            let selectedTransportation = saveData as? SearchFilterCategory
            parentController?.searchFilterSaveState?.selectedTransportation = selectedTransportation
            
            if (selectedTransportation?.hasSub == false) {
                parentController?.searchFilterSaveState?.selectedSubTransportation = nil
            }
            
        } else if(saveData?.isKindOfClass(SearchFilterSubCategory) == true) {
            let selectedSubTransportation = saveData as? SearchFilterSubCategory
            parentController?.searchFilterSaveState?.selectedSubTransportation = selectedSubTransportation
        }
        
        tableView.reloadRowsAtIndexPaths([NSIndexPath(forRow: transportationSearchCellIndex.TRANSPORT_TRANSPORTATION.rawValue, inSection: 0)], withRowAnimation: .None)

    }
    
    func getKeywordTransportationFromSaveState() -> String {
        let selectedMainTransportation = parentController?.searchFilterSaveState?.selectedMainTransportation
        let selectedTransportation = parentController?.searchFilterSaveState?.selectedTransportation
        let selectedSubTransportation = parentController?.searchFilterSaveState?.selectedSubTransportation
        
        return getKeywordFromSaveState(selectedTransportation, subFilter: selectedSubTransportation, defaultKeyword: DEFAULT_KEYWORD_TRANSPORTATION , transportationFilter: selectedMainTransportation)
    }

    func getTransportParamaterFromSaveState() -> String {
        let saveFilter = parentController?.searchFilterSaveState
        return (saveFilter?.selectedTransportation == nil) ? DEFAULT_KEYWORD_FOR_NIL_TYPE : (saveFilter?.selectedTransportation!.code)!
    }
    
    func getSubTransportParamaterFromSaveState() -> String {
        let saveFilter = parentController?.searchFilterSaveState
        return (saveFilter?.selectedSubTransportation == nil) ? DEFAULT_KEYWORD_FOR_NIL_TYPE : (saveFilter?.selectedSubTransportation!.code)!
    }
    
    func getTransportationRadiusValueFromSaveState() -> Int {
        return (parentController?.searchFilterSaveState?.transportationRadius)!
    }
    
    func getTransportationRadiusParameterFromSaveState() -> String {
        return "\((parentController?.searchFilterSaveState!.transportationRadius)!)"
    }

    // Mark :: Delagate TableView
    override func initTableView() {
        super.initTableView()
//        tableHeightConstraint.constant = cellHeight! * TRANSPORT_NUMBER_OF_CELL
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return transportationSearchCellIndex.count.hashValue
    }

    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
//        if (transportationSearchCellIndex(rawValue: indexPath.row))! == .TRANSPORT_RADIUS || (transportationSearchCellIndex(rawValue: indexPath.row))! == .TRANSPORT_FOOTER{
            return CELL_SIZE_BIG
//        } else {
//            return CELL_SIZE_SMALL
//        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        switch (transportationSearchCellIndex(rawValue: indexPath.row))! {
            case .TRANSPORT_CATEGORY:
                openSearchFilter(.CATEGORY)
                break
            case .TRANSPORT_TRANSPORTATION:
                openSearchFilter(.TRANSPORTATION)
                break
            default:
                break
        }
        
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    override func configureCellForRowAtIndexPath(cell: JTMSeachFilterTableViewCell, indexPath: NSIndexPath) -> UITableViewCell {
        
        setSelectedBackgroundView(cell)
        
        switch (transportationSearchCellIndex(rawValue: indexPath.row))! {
        case .TRANSPORT_CATEGORY:
            configureJobCategoryCell(cell)
            break
        case .TRANSPORT_TRANSPORTATION:
            cell.setDescription(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_TRANSPORTATION"), subTitle: getKeywordTransportationFromSaveState())
            break
        case .TRANSPORT_KEYWORD:
            cellSearch = configureCellSearch()
            return cellSearch!
        case .TRANSPORT_FOOTER:
            cellFooter = configureCellSearchFooter()
            cellFooter?.setCellSeperatorLineHide(tableView)
            return cellFooter!
        default:
            let cellArea = tableView.dequeueReusableCellWithIdentifier("JTMTransportationRadiusTableViewCell") as! JTMTransportationRadiusTableViewCell
            cellArea.selectionStyle = UITableViewCellSelectionStyle.None
            cellArea.delegate = self
            cellArea.setSliderValueTo(getTransportationRadiusValueFromSaveState())
//            setCellSeperatorLineHide(cellArea) 
            return cellArea
        }
        
        return cell
    }
    
    func setSelectedBackgroundView(cell: JTMSeachFilterTableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }

    //MARK:- Call Service
    override func loadRequiredFilterListFromService() {
        super.loadRequiredFilterListFromService()
        loadTransportationListFromService()
    }
    
    func loadTransportationListFromService() {
        let lastUpdate = (appSetting?.searchFilterTransportationLastUpdate)!
        let urlTransportation = getServiveSearchFilter(SearchFilterCategoryType.TRANSPORTATION)
        
        loadSearchFilterListFromService(urlTransportation, lastupdate: lastUpdate) { (jsonObject) -> Void in
            self.appSetting?.searchFilterTransportationLastUpdate = (jsonObject.objectForKey("timestamps") as? Int)!
            let itemList = jsonObject.objectForKey("transportations") as? NSArray

            self.appSetting?.searchFilterTransportationList.removeAll()
            for item in itemList! {
                self.appSetting?.searchFilterTransportationList.append(SearchFilterCategoryTransportation(item: item as! NSMutableDictionary))
            }
        }
    }

    // Mark :: Delegate value Slide
    func onValueAreaTransportChanged(value: Int) {
        parentController?.searchFilterSaveState?.transportationRadius = value
    }
    
    //MARK:- On Search
    override func footerOnClickSearch() {
        
        var action = ""
        
        if getTransportParamaterFromSaveState() == DEFAULT_KEYWORD_FOR_NIL_TYPE {
            showAlertWithData("", message: applicationDelegate.getLocalizeStringForKey("ALERT_SUB_TITLE_TRANSPORTATION"), completionHandler: nil, actionList: nil)
        } else {
            let jobType = getJobTypeParamaterFromSaveState()
            let subType = getSubJobTypeParamaterFromSaveState()
            let keyword = getKeywordParameterFromSaveState()
            let radius = getTransportationRadiusParameterFromSaveState()
            let mt_maintype = getTransportParamaterFromSaveState()
            let mt_subtype = getSubTransportParamaterFromSaveState()
            let transportationSelected = self.parentController?.searchFilterSaveState?.selectedTransportation
            let disability = getDisabilitySaveState().hashValue
            var parameter : [String:AnyObject] = [
                PARAMETER_JOB_TYPE_KEY              : jobType,
                PARAMETER_SUB_JOB_TYPE_KEY          : subType,
                PARAMETER_KEYWORD_KEY               : keyword,
                PARAMETER_RADIUS_KEY                : radius,
                PARAMETER_MAIN_TRANSPORTATION_KEY   : mt_maintype,
                PARAMETER_DISABLED_WELCOME_KEY      : disability
            ]
            
            if(transportationSelected!.isBusRoute){
//                parameter = "?jobtype=\(jobType)&subtype=\(subType)&keyword=\(keyword)&mt_radius=\(radius)&mt_maintype=\(mt_maintype)&busroute=\(mt_subtype)&disabled_person=\(disability)"
                parameter[PARAMETER_BUS_ROUTE_KEY] = mt_subtype
            }else{
//                parameter = "?jobtype=\(jobType)&subtype=\(subType)&keyword=\(keyword)&mt_radius=\(radius)&mt_maintype=\(mt_maintype)&mt_subtype=\(mt_subtype)&disabled_person=\(disability)"
                parameter[PARAMETER_SUB_TRANSPORTATION_KEY] = mt_subtype
            }
            
            self.setTrackGoogleAnalyticKeyword(keyword)
            self.setTrackGoogleAnalyticDistance(radius)
            self.setTrackGoogleAnalyticDisability(disability)
            
            openSearchResult(parameter , isOpenMap: true , radiusInMeter: 0)
        }
    }
    
    func setTrackGoogleAnalyticKeyword(keyword : String) {
        var action = ""
        
        if keyword.isEmpty {
            action = ACTION_SEARCH_NON_KEYWORD
        } else {
            action = ACTION_SEARCH_KEYWORD
        }
        
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_TRANSPORTATION, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    func setTrackGoogleAnalyticDisability(disability : Int) {
        var action = ""
        
        if disability == 1 {
            action = ACTION_SEARCH_DISABILITY
        } else {
            action = ACTION_SEARCH_NON_DISABILITY
        }
        
        // Track Google Analytic Event disability
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_TRANSPORTATION, action: action, label: LABEL_RESULT_JOB_SEARCH)
    }
    
    func setTrackGoogleAnalyticDistance(radius : String) {
        
        let distance = "\(radius)" + " " + applicationDelegate.getLocalizeStringForKey("TITLE_AREA_TRANSPORTATION_BACK")
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH_BY_TRANSPORTATION, action: ACTION_SELECT_DISTANCE, label: getLabelDistance(distance))
    }
}