//
//  JTMWebViewController.swift
//  JobThai
//
//  Created by kanya on 1/15/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMWebViewController: JTMBaseViewController, UIWebViewDelegate {
    
    @IBOutlet weak var webView:UIWebView!
    var urlString:String!
    var webTitle:String!
    var isErrorLoadingURL : Bool = true
    var isFinishedFirstRequest : Bool = false
    
    let APP_STORE_DOMAIN = "itunes.apple.com"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.automaticallyAdjustsScrollViewInsets = false
        
        self.webView.delegate = self
        self.webView.scrollView.bounces = false
        self.webView.scalesPageToFit = true
        self.webView.accessibilityIdentifier = ID_WEBVIEW_VIEW
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        self.title = webTitle
        self.isErrorLoadingURL = false
        self.hideLoading()
        
//        self.setNavigationTitleForWeb()
//        self.loadUrl()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    //MARK:- Manage Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if !isFinishedFirstRequest {
            loadUrl()
        } else if isErrorLoadingURL {
            webView.reload()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if isErrorLoadingURL {
            setAlphaShowToView(view)
        } else if webView.loading {
            webView.stopLoading()
        }
    }
    
    func loadUrl() {
        print("load \(webView.loading)")
//        if (applicationDelegate.appSetting?.isNetworkReachable == true) {
            let url:NSURL = NSURL(string: urlString)!
            let request:NSURLRequest = NSURLRequest(URL: url)
            self.webView.loadRequest(request)
//        } else {
//            self.showAlertNoNetworkIsReachable()
//        }
    }

    func webView(webView: UIWebView, didFailLoadWithError error: NSError?) {
        print("fail \(webView.loading) \(error.debugDescription)")
        if error?.code != NSURLErrorCancelled {
            self.hideLoading()
        }
        isErrorLoadingURL = true
    }
    
    func webView(webView: UIWebView, shouldStartLoadWithRequest request: NSURLRequest, navigationType: UIWebViewNavigationType) -> Bool {
        if (!webView.loading) {
            self.showLoading()
        }
        if navigationType == UIWebViewNavigationType.LinkClicked {
            let application = UIApplication.sharedApplication()
            dump(request.URL)
            if let phoneURL = request.URL where (phoneURL.absoluteString.rangeOfString("tel:") != nil) {
                if application.canOpenURL(phoneURL) {
                    let phoneNumber = "\(phoneURL)"
                    let numberString = String(phoneNumber.characters.dropFirst(4))
                    let alert = UIAlertController(title: "\(numberString)", message: nil, preferredStyle: UIAlertControllerStyle.Alert)
//                    alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("BUTTON_CANCEL"), style: UIAlertActionStyle.Default, handler: nil))
                    alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("BUTTON_CANCEL"), style: UIAlertActionStyle.Default, handler: { (action) in
                        self.hideLoading()
                    }))
                    alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("BUTTON_CALL"), style: .Default, handler: { action in
                        application.openURL(phoneURL)
                        self.hideLoading()
                    }))
                    self.presentViewController(alert, animated: true, completion: nil)
//                    self.hideLoading()
                    
                    return false
                }
                self.hideLoading()
            } else if let mailURL = request.URL where (mailURL.absoluteString.rangeOfString("mailto:") != nil) {
                self.hideLoading()
            } else if request.URLString.containsString(APP_STORE_DOMAIN) {
                self.hideLoading()
            }
        }
        
        return true
    }
    
    func webViewDidFinishLoad(webView: UIWebView) {
        isFinishedFirstRequest = true
        isErrorLoadingURL = false
        self.hideLoading()
    }
    
    func webViewDidStartLoad(webView: UIWebView) {
    }
    
    func setNavigationTitleForWeb() {
        let titleLabel = UILabel(frame: CGRectMake(0,0,200,50))
        titleLabel.text = self.webTitle
        titleLabel.textColor = UIColor.whiteColor()
        titleLabel.textAlignment = .Center
        
        let view = UIView(frame: titleLabel.bounds)
        view.addSubview(titleLabel)
        
        self.navigationItem.titleView = view
    }
}
