//
//  TNMapView.swift
//  JobThai
//
//  Created by pinyo on 3/21/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import Foundation
import UIKit

class TNMapView : UIView , RMMapViewDelegate , CLLocationManagerDelegate{
    //Map Setting
    var mapViewThinknet : RMMapView?
    private var radiusNearMeInMeter : Double = 0
    let LOCATION_BANGKOK = CLLocationCoordinate2D(latitude: 13.724442, longitude: 100.352916)
    
    var delegate                            : TNMapViewDelegate!
    private let TNMAP_CALLOUT_JOBLIST_KEY   : String        = "TNMapCalloutView"
    private let TNMAP_CALLOUT_ADDRESS_KEY   : String        = "TNMapCalloutViewAddress"
    private var MAP_COLOR_TINT              : UIColor       = UIColor.redColor()
    var mapLanguage                         : TNMapLanguage = TNMapLanguageThai
    
    //Map Radius Circle Near Me
    private var circleNearMe : RMPolygonAnnotation?
    private let CIRCLE_ANNOTATION_KEY : String = "CIRCLE_ANNOTATION_TYPE"
    private var circleNearMeFillColor : UIColor = UIColor(colorLiteralRed: 0.8, green: 0.5 , blue: 0.0 , alpha: 0.1)
    private var circleNearMeLineColor : UIColor = UIColor(colorLiteralRed: 1.0, green: 0.5 , blue: 0.0 , alpha: 0.5)
    private var circleNearMeLineWidth : CGFloat = 5.0
    
    private let DEFAULT_ZOOM_LEVEL : Float = 4.02863
    
    //Location
    private let locationManager = CLLocationManager()
    var userLocation : CLLocationCoordinate2D?
    
    //CopyRight & MapMagic Logo
    var LeftImageView       : UIImageView?
    var RightImageView      : UIImageView?
    var mapMagicImg         : UIImage?
    var copyRightThImg      : UIImage?
    var copyRightEnImg      : UIImage?
    
    //Tile Source
    static var urlTnTileSource     : String?
    static var startMapRange       : Int        = Int(URL_START_RANGE_DEFAULT)
    static var stopMapRange        : Int        = Int(URL_STOP_RANGE_DEFAULT)
    static var maxZoom             : Float      = 1
    static var minZoom             : Float      = 19
    static var mapLanguageInit     : TNMapLanguage  =  TNMapLanguageThai
    
    //View 
    private var loaded      : Bool = false
    
    //Zoom Bounding
    var zoomRect : RMProjectedRect?

    //MARK: Initial View
    override init(frame: CGRect) {
        super.init(frame: frame)
        print("frame")
        self.addMapToCurrentViewController()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        print("codeer")
        self.addMapToCurrentViewController()
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        print("layout sub")
        if(!loaded){
            loaded = true
            self.initialCopyRight()
        }
    }
    
    //MARK: Location Manager
    ///TNMapView : for request location when can not get user location.
    func requestLocationManager(){
        if(CLLocationManager.locationServicesEnabled()){
            self.locationManager.requestAlwaysAuthorization()
            self.locationManager.requestWhenInUseAuthorization()
            self.locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters
            self.locationManager.delegate = self
            self.locationManager.startUpdatingLocation()
        }
    }
    
    func locationManager(manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        print("up location :\(manager.location?.coordinate)")
        if self.circleNearMe == nil && manager.location?.coordinate != nil{
            self.userLocation = manager.location?.coordinate
            self.addCircleAnnotation(self.radiusNearMeInMeter, coordinate: self.userLocation!)
            self.stopUpdateUserLocation()
        }
    }
    
    ///TNMapView : for start update user location.
    func startUpdateUserLocation(){
        self.locationManager.startUpdatingLocation()
    }
    
    ///TNMapView : for stop update user location.
    func stopUpdateUserLocation(){
        self.locationManager.stopUpdatingLocation()
    }
    
    //MARK: Inital MapView Engine
    ///TNMapView : for initial MapView of THiNKNET for cache mapView.
    static func initialMapViewEngine(completion : (()->(Void))?) {
        //Token for Mapbox Engine
        RMConfiguration.sharedInstance().accessToken = "pk.eyJ1IjoidGhlcGlueW8iLCJhIjoiY2loaGFwdzhxMDR0cHRyajdtdDNuODB4NyJ9.gGXr1uJ9EEkFygoLuj-cGg"
        
        let tnTileMap = TNMapView.getMapTile(TNMapTypeStandard, andLanguage: TNMapLanguageThai)//RMMapboxSource.init(mapType: TNMapTypeStandard, andLanguage: TNMapLanguageThai)
        _ = RMMapView.init(frame: UIScreen.mainScreen().bounds, andTilesource: tnTileMap)
        
        if let callBack = completion{
            callBack()
        }
    }
    
    ///TNMapView : Clear all tile images from the caching system
    static func clearMapTileCache(){
        let mapView = RMMapView.init()
        mapView.removeAllCachedImages()
    }
    
    ///TNMapView : for set show user location in mapview.
    func setShowUserLocation(show : Bool){
////        let status = CLLocationManager.authorizationStatus()
////        if CLLocationManager.locationServicesEnabled() {
////            switch (status) {
////            case .NotDetermined :
////                break
////            case .Denied :
////                break
////            default :
////                //Set show user location
////                self.mapViewThinknet?.showsUserLocation = true
////                let currentZoomLevel = self.mapViewThinknet!.zoom
////                self.mapViewThinknet?.userTrackingMode = RMUserTrackingModeNone
////                self.mapViewThinknet?.zoom = currentZoomLevel
////                
////                if self.mapViewThinknet?.userLocation != nil{
////                    self.mapViewThinknet?.userLocation.enabled = false
////                }
////                break
////            }
////        }
        if let mapView = mapViewThinknet {
            mapView.showsUserLocation = show
        }
    }
    
//    func setUserLocationRequestShow(show : Bool){
//        self.mapViewThinknet?.showsUserLocation = show
//    }
    
    func userLocationIsShow() -> Bool {
        if(mapViewThinknet != nil) {
            return (mapViewThinknet?.showsUserLocation)!
        }
        return false
    }
    
    func setUserLocationEnable(enable : Bool){
        if(mapViewThinknet?.userLocation != nil){
            mapViewThinknet?.userLocation.enabled = enable
        }
    }
    
    //MARK: AccuracyCircleAnnotation
    /** For Set/Get AccuracyCircleAnnotation Layer*/
    func setAccuracyCircleAnnotationLayer(circleLayer : RMCircle){
        self.mapViewThinknet?.setAccuracyCircleAnnotationLayer(circleLayer)
    }
    
    func setAccuracyCircleAnnotationLayer(fillColor circleFillColor : UIColor ,lineColor circleLineColor : UIColor ,andLineWidth circleLineWidth : CGFloat){
        self.circleNearMeFillColor = circleFillColor
        self.circleNearMeLineColor = circleLineColor
        self.circleNearMeLineWidth = circleLineWidth
        self.mapViewThinknet?.setAccuracyCircleAnnotationLayer(circleFillColor, andLineColor: circleLineColor, andLineWidth: circleLineWidth)
    }
    
    
    //MARK: Add MapView to self view
    ///TNMapView : for add MapView to subView of current ViewController
    private func addMapToCurrentViewController(){
        //Initial Map Properties
        let mapTile = TNMapView.getMapTile(TNMapTypeStandard, andLanguage: TNMapView.mapLanguageInit)
        self.mapViewThinknet = RMMapView.init(frame: CGRect(origin: CGPointZero, size: self.frame.size))
        self.mapViewThinknet?.tileSource        = mapTile
        self.mapViewThinknet?.adjustTilesForRetinaDisplay = true
        self.mapViewThinknet?.showLogoBug       = false
        self.mapViewThinknet?.hideAttribution   = true
        self.mapViewThinknet?.zoom              = self.DEFAULT_ZOOM_LEVEL
        self.mapViewThinknet?.tintColor         = self.tintColor
        self.mapViewThinknet?.centerCoordinate  = LOCATION_BANGKOK
        self.mapViewThinknet?.delegate          = self
        self.mapViewThinknet?.autoresizingMask  = [.FlexibleWidth ,.FlexibleHeight]
        self.mapViewThinknet?.maxZoom           = TNMapView.maxZoom
        
        if(mapViewThinknet != nil){
            self.addSubview(self.mapViewThinknet!)
            self.sendSubviewToBack(self.mapViewThinknet!)
        }
    }
    
    func reloadMapView() {
        if mapViewThinknet == nil {
            self.addMapToCurrentViewController()
        }
    }
    
    ///TNMapView : for set map tile language 
    func setMapTileLanguage(language : TNMapLanguage){
        if self.mapViewThinknet != nil{
            let mapTile = TNMapView.getMapTile(TNMapTypeStandard, andLanguage: language)
            self.mapViewThinknet?.tileSource = mapTile
            self.mapViewThinknet?.maxZoom = TNMapView.maxZoom
//            self.mapViewThinknet?.minZoom = TNMapView.minZoom
        }
//        setCopyRightLanguage(language)
    }
    
    static func getMapTile(mapTileType : TNMapType , andLanguage mapTileLanguage : TNMapLanguage) -> RMMapboxSource? {
        var mapTile : RMMapboxSource? = nil
        if TNMapView.urlTnTileSource != nil {
            mapTile = RMMapboxSource.init(mapType: mapTileType, language: mapTileLanguage, url: TNMapView.urlTnTileSource, startRange: Int32(TNMapView.startMapRange), andStopRange: Int32(TNMapView.stopMapRange))
        } else {
            mapTile = RMMapboxSource.init(mapType: mapTileType, andLanguage: mapTileLanguage)
        }
        return mapTile
    }
    
    func setMapTintColor(color : UIColor){
        if let mapView = self.mapViewThinknet {
            mapView.tintColor = color
        }
    }
    
    ///TNMapView : for set enable/disable user interaction.
    func setUserInteraction(enable : Bool){
        if let mapView = self.mapViewThinknet {
            mapView.userInteractionEnabled = enable
        }
    }
    
    ///TNMapView : for set enable/disable double-tap on mapview by one finger.
    func getDoubleTabOnMapEnabled() -> Bool{
        if self.mapViewThinknet != nil {
            return (self.mapViewThinknet?.doubleTapOnMapEnabled)!
        }
        return true
    }
    func setDoubleTabOnMapEnabled(doubleTapEnabled : Bool){
        if self.mapViewThinknet != nil {
            self.mapViewThinknet?.doubleTapOnMapEnabled = doubleTapEnabled;
        }
    }
    
    ///TNMapView : for set user location tracking mode.
    func setUserTrackingMode( userMode : RMUserTrackingMode){
        if self.mapViewThinknet != nil {
            self.mapViewThinknet?.userTrackingMode = userMode
        }
    }
    
    ///TNMapView : for set mapView use Open Street Map Tile Source.
    func useOSMTile(){
        //OpenStreetMap
        let mapBoxMapID : String = "mapbox.streets"
        let mapTile = RMMapboxSource.init(mapID: mapBoxMapID)
        self.mapViewThinknet?.tileSource = mapTile
    }
    
    ///TNMapView : for create annotation with companyName[Title] and jobList[Description] in calloutView.
    func createAnnotationWithJobList( companyName : String ,jobList : [String], coordinate : CLLocationCoordinate2D , userInfo : AnyObject?) -> RMAnnotation{
        if mapViewThinknet == nil {
            mapViewThinknet = RMMapView.init()
        }
        //Get First Annotation
        if jobList.count > 0 {
            //Inital Annotation and Add to MapView
            if mapViewThinknet != nil {
                let annotation : RMAnnotation = RMAnnotation.init(mapView: self.mapViewThinknet!, coordinate: coordinate, andTitle: companyName)
                annotation.userInfo = userInfo
                
                let calloutView  = self.createCalloutViewJobList(companyName ,andDescription: jobList)
                annotation.calloutView = calloutView
                return annotation
            }
        }
        return RMAnnotation()
    }
    
    //TNMapView : Create Annotation for Company Address 
    func createAnnotationWithAddress( companyName : String ,address : String , coordinate : CLLocationCoordinate2D , userInfo : AnyObject?) -> RMAnnotation?{
        //Inital Annotation and Add to MapView
        if mapViewThinknet == nil {
            mapViewThinknet = RMMapView.init()
        }
        
        if mapViewThinknet != nil {
            let annotation : RMAnnotation = RMAnnotation.init(mapView: self.mapViewThinknet, coordinate: coordinate, andTitle: companyName)
            annotation.userInfo = userInfo
            
            let calloutView  = self.createCalloutViewAddress(companyName ,andDescription: address)
            annotation.calloutView = calloutView
            return annotation
        }
        return nil
    }
    
    ///TNMapView : Create CalloutView for THiNKNET Annotation.
    private func createCalloutViewJobList(companyName : String, andDescription jobList : [String]) -> TNMapCalloutView{
        let calloutView = NSBundle.mainBundle().loadNibNamed(TNMAP_CALLOUT_JOBLIST_KEY, owner: nil, options: nil).first as? TNMapCalloutView
        calloutView?.setTitleAndDescription(companyName, andDescriptionList: jobList)
        return calloutView!
    }
    
    ///TNMapView : Create CalloutView for THiNKNET Address Type
    private func createCalloutViewAddress(companyName : String, andDescription detail : String ) -> TNMapCalloutViewAddress{
        let calloutView = NSBundle.mainBundle().loadNibNamed(TNMAP_CALLOUT_ADDRESS_KEY, owner: nil, options: nil).first as? TNMapCalloutViewAddress
        calloutView?.setTitleAndAddress(companyName, andDrescription: detail)
        return calloutView!
    }
    
    
    //MARK: Add Annotation & Circle Near Me
    ///TNMapView : for add annotation to MapView.
    func addAnnotationToMapView(annotation : RMAnnotation){
        if self.mapViewThinknet != nil {
            self.mapViewThinknet?.addAnnotation(annotation)
        }
    }
    
    ///TNMapView : for add annotations array to MapView.
    func addAnnotationsToMapView(annotations : [RMAnnotation]){
        if self.mapViewThinknet != nil {
            if annotations.count > 0 {
                zoomRect = self.mapViewThinknet?.getAnnotationsBoundsSouthAndNorth(NSMutableArray(array: annotations))
            }
            self.mapViewThinknet?.addAnnotations(annotations)
        }
    }
    
    ///TNMapView : for remove annotation in mapview.
    func removeAnnotation(annotation : RMAnnotation){
        if self.mapViewThinknet != nil {
            self.mapViewThinknet?.removeAnnotation(annotation)
        }
    }
    
    ///TNMapView : for clear all annotation in mapview
    func removeAllAnnotation(){
        if self.mapViewThinknet != nil {
            self.mapViewThinknet?.removeAllAnnotations()
        }
    }
    
    ///TNMapView : for add circle annotation to mapview.
    private func addCircleAnnotation(radiusInMeter : Double , coordinate : CLLocationCoordinate2D ){
        if (radiusInMeter > 0){
            if(self.circleNearMe == nil){
                self.circleNearMe = createPolygonCircleFromCoordinate(coordinate, withMeterRadius: radiusInMeter)
                setCircleColor(fillColor: circleNearMeFillColor, andLineColor: circleNearMeLineColor, andLineWidth: circleNearMeLineWidth)
                self.circleNearMe!.enabled = false
                
            }else if self.circleNearMe != nil{
                self.mapViewThinknet?.removeAnnotation(self.circleNearMe)
                self.circleNearMe?.coordinate = coordinate
            }
            self.radiusNearMeInMeter = radiusInMeter
            self.mapViewThinknet?.addAnnotation(self.circleNearMe)
        }
    }
    
    ///TNMapview : for add circle annotation to mapview and set color of cirlce.
    private func addCircleAnnotation(radiusInMeter : Double , coordinate : CLLocationCoordinate2D , fillColor : UIColor , lineColor : UIColor , lineWidth : CGFloat){
        self.circleNearMeFillColor = fillColor
        self.circleNearMeLineColor = lineColor
        self.circleNearMeLineWidth = lineWidth
        self.addCircleAnnotation(radiusInMeter, coordinate: coordinate)
    }
    
    ///TNMapView : for set circle after viewDidLoad
    func setCircleNearMeAnnotation(radiusInMeter radius : Double){
        if let userLocation = self.mapViewThinknet?.userLocation{
            let latitudeFloat : Float = Float(userLocation.coordinate.latitude)
            let longtitudeFloat : Float = Float(userLocation.coordinate.longitude)
            if latitudeFloat != FLT_MAX && longtitudeFloat != FLT_MAX && mapViewThinknet != nil {
                self.addCircleAnnotation(radius, coordinate: self.mapViewThinknet!.userLocation.coordinate)
            }
            else{
                self.requestLocationManager()
            }
        }else{
            self.requestLocationManager()
        }
        self.radiusNearMeInMeter = radius
    }

    ///TNMapView : for set Circle Near me color
    func setCircleColor(fillColor _fillColor: UIColor , andLineColor _lineColor : UIColor , andLineWidth _lineWidth: CGFloat){
        self.circleNearMeFillColor = _fillColor
        self.circleNearMeLineColor = _lineColor
        self.circleNearMeLineWidth = _lineWidth
        
        if(self.circleNearMe != nil){
            self.circleNearMe?.fillColor = _fillColor
            self.circleNearMe?.lineColor = _lineColor
            self.circleNearMe?.lineWidth = _lineWidth
        }
    }
    
    ///TNMapView : for set circle near me radius and color.
    func setCircleNearMeAnnotation(radiusInMeter radius : Double , andFillColor _fillColor: UIColor , andLineColor _lineColor : UIColor , andLineWidth _lineWidth: CGFloat){
        self.circleNearMeFillColor = _fillColor
        self.circleNearMeLineColor = _lineColor
        self.circleNearMeLineWidth = _lineWidth
        self.setCircleColor(fillColor: _fillColor, andLineColor: _lineColor, andLineWidth: _lineWidth)
        self.setCircleNearMeAnnotation(radiusInMeter: radius)
    }
    
    ///TNMapView : for set circle near me show/hide.
    func setHiddenCircleNearMe(hideCircle : Bool){
        if self.circleNearMe != nil{
            self.circleNearMe?.layer.hidden = hideCircle
        }
    }
    
    //MARK: Create Circle by Polygon
    private func setPolygonColor(polygon : RMPolygonAnnotation ,andFillColor fillColor:UIColor , andLineColor lineColor:UIColor , andLineWidth lineWidth:CGFloat){
        polygon.fillColor = fillColor
        polygon.lineColor = lineColor
        polygon.lineWidth = lineWidth
    }
    
    func createPolygonCircleFromCoordinate(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) -> RMPolygonAnnotation{
        let coordinateList = generateCoordinateListFromCoordinate(coordinate, withMeterRadius: withMeterRadius)
        let polygon = RMPolygonAnnotation(mapView: mapViewThinknet, points: coordinateList)
        polygon.enabled = false
        polygon.isUserLocationAnnotation = true
        return polygon
    }
    
    private func generateCoordinateListFromCoordinate(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) -> [CLLocation]{
        let degreesBetweenPoints = 2.0//8.0
        let numberOfPoints = floor(360.0 / degreesBetweenPoints)
        let distRadians: Double = withMeterRadius / 6371000.0
        let centerLatRadians: Double = coordinate.latitude * M_PI / 180
        let centerLonRadians: Double = coordinate.longitude * M_PI / 180
        var coordinates = [CLLocation]()
        
        
        for index in 0 ..< Int(numberOfPoints) {
            let degrees: Double = Double(index) * Double(degreesBetweenPoints)
            let degreeRadians: Double = degrees * M_PI / 180
            let pointLatRadians: Double = asin(sin(centerLatRadians) * cos(distRadians) + cos(centerLatRadians) * sin(distRadians) * cos(degreeRadians))
            let pointLonRadians: Double = centerLonRadians + atan2(sin(degreeRadians) * sin(distRadians) * cos(centerLatRadians), cos(distRadians) - sin(centerLatRadians) * sin(pointLatRadians))
            let pointLat: Double = pointLatRadians * 180 / M_PI
            let pointLon: Double = pointLonRadians * 180 / M_PI
            let point: CLLocation = CLLocation(latitude: pointLat, longitude: pointLon)
            coordinates.append(point)
        }
        return coordinates
    }
    
    func reFillColorCircleNearMe(){
        self.setCircleColor(fillColor: self.circleNearMeFillColor, andLineColor: self.circleNearMeLineColor, andLineWidth: self.circleNearMeLineWidth)
    }
    
    
    ///TNMapView : for get all annotations in mapView.
    func getAllAnnotation() -> [AnyObject]?{
        return self.mapViewThinknet?.annotations
    }
    
    func getSelectAnnotation() -> RMAnnotation!{
        return self.mapViewThinknet?.selectedAnnotation
    }
    ///TNMapView : for copy annotations to new object.
    func copyAnnotations(_annotationArr : [RMAnnotation])->[RMAnnotation]?{
        var annotationList = [RMAnnotation]()
        for annotation in _annotationArr{
            if(annotation.calloutView.isKindOfClass(TNMapCalloutView)){
                let calloutView = annotation.calloutView as? TNMapCalloutView
                let newAnnotation = self.createAnnotationWithJobList((calloutView?.title)!, jobList: (calloutView?.descriptionList)!, coordinate: annotation.coordinate, userInfo: annotation.userInfo)
                annotationList.append(newAnnotation)
            }else if(annotation.calloutView.isKindOfClass(TNMapCalloutViewAddress)){
                let calloutView = annotation.calloutView as? TNMapCalloutViewAddress
                let newAnnotation = self.createAnnotationWithAddress((calloutView?.title)!, address: (calloutView?.detail)!, coordinate: annotation.coordinate, userInfo: annotation.userInfo)
                if newAnnotation != nil {
                    annotationList.append(newAnnotation!)
                }
            }
        }
        return annotationList
    }
    
    ///TNMapView : check annotation in mapView
    func annotationIsInMapView(annotation : RMAnnotation) -> Bool{
        let annotationsArr = self.mapViewThinknet?.annotations as? [RMAnnotation]
        return (annotationsArr?.contains(annotation))!
    }
    
    //MARK: Show All Annotation, Map Level, Center Map
    ///TNMapView : for zoom to all annotation in mapView and show without group user location.
    func zoomToFitAllAnnotationWithOutUserLocation(){
        if self.mapViewThinknet?.annotations?.count > 0{
            
            guard let zoomBound = zoomRect else {
                self.mapViewThinknet?.zoomToFitAllAnnotationsWithOutUserLocationAnimated(false)
                self.mapViewThinknet?.zoom = (self.mapViewThinknet?.zoom)! - 1
                print("[TNMapView | zoomToFitAll] : Not Found Zoom Bounding Rect.")
                return
            }
            
            self.mapViewThinknet?.setProjectedBounds(zoomBound, animated: false)
            self.mapViewThinknet?.zoom = (self.mapViewThinknet?.zoom)! - 1
        }
    }
    
    func zoomToBound(boundRect : RMProjectedRect?) {
        if self.mapViewThinknet?.annotations?.count > 0{
            guard let zoomBound = boundRect else {
                self.mapViewThinknet?.zoomToFitAllAnnotationsWithOutUserLocationAnimated(false)
                self.mapViewThinknet?.zoom = (self.mapViewThinknet?.zoom)! - 1
                print("[TNMapView | zoomToBound] : Not Found Zoom Bounding Rect.")
                return
            }
            
            self.mapViewThinknet?.setProjectedBounds(zoomBound, animated: false)
            self.mapViewThinknet?.zoom = (self.mapViewThinknet?.zoom)! - 1
        }

    }
    
    func getBoundsAllAnnotation() -> RMProjectedRect? {
        guard let boundingBox = self.zoomRect else {
            return nil
        }
        return boundingBox
    }
    
    ///TNMapView : for get mapview zoom level.
    func getMapLevel()->Float{
        return self.mapViewThinknet?.zoom ?? 0
    }
    
    ///TNMapView : for set mapview zoom level.
    func setMapLevel(level : Float){
        if let mapView = mapViewThinknet {
            mapView.zoom = level
        }
    }
    
    func setMaxZoomLevel(level : Float){
        if let mapView = mapViewThinknet {
            mapView.maxZoom = level
        }
    }
    
    func setMinZoomLevel(level : Float){
        if let mapView = mapViewThinknet {
            mapView.minZoom = level
        }
    }
    
    ///TNMapView : for set center of mapview to coordinate.
    func setCenterMapToCoordinate(coordinate : CLLocationCoordinate2D , animated _animate : Bool){
        self.mapViewThinknet?.setCenterCoordinate(coordinate, animated: _animate)
    }
    
    ///TNMapView : for set center of mapview to userlocation
    func setCenterMapToUserLocation(animated : Bool){
        if let userPosition = self.mapViewThinknet?.userLocation{
            print("USer Position")
            if Float(userPosition.coordinate.latitude) != FLT_MAX && Float(userPosition.coordinate.longitude) != FLT_MAX{
                self.mapViewThinknet?.setCenterCoordinate(userPosition.coordinate, animated: animated)
            }
        }
    }
    
    ///TNMapView : for set center of mapView to Bangkok coordinate(latitude: 13.736717, longitude: 100.523186).
    func setCenterMapViewToBangkok(){
        if(mapViewThinknet != nil){
            self.mapViewThinknet?.centerCoordinate = LOCATION_BANGKOK
//            self.mapViewThinknet!.zoomToFitAllAnnotationsWithOutUserLocationAnimated(false)
//            self.mapViewThinknet!.zoom = self.mapViewThinknet!.zoom - 1
        }
    }
    
    ///TNMapView : For check visible annotation in mapview without user location.
    func haveAnnotationInMapView() -> Bool {
        if((self.userLocationIsShow() && self.getAllAnnotation()?.count > 3) || (!self.userLocationIsShow() && self.getAllAnnotation()?.count > 0)) {
            return true
        }
        return false
    }
    
    ///TNMapView : for set mapview to select annotation, calloutView will show affer selected.
    func selectAnnotation(annotation : RMAnnotation , showAnimate : Bool){
        self.mapViewThinknet?.selectAnnotation(annotation, animated: showAnimate)
    }
    
    //MARK: MapView Delegate
    ///TNMapView : for return marker layer of annotation to mapView.
    func mapView(mapView: RMMapView!, layerForAnnotation annotation: RMAnnotation!) -> RMMapLayer! {
        if annotation.isUserLocationAnnotation {
            print("User Location")
            return nil
        }

        let imagePin = UIImage(named: "pin")
        let marker : RMMarker = RMMarker.init(UIImage: imagePin!)
        marker.canShowCallout = true
        return marker
    }
    
//    //MapView Sort Layer
//    ///TNMapView : for sort layer of annotation , Layer : [0-CircleNearMe , 1-UserLocation , 2-Annotation].
//    func annotationSortingComparatorForMapView(mapView: RMMapView!) -> NSComparator! {
//        let sort : NSComparator = { (annotationLeft : AnyObject, annotationRight : AnyObject) -> NSComparisonResult in
//            let annotation1 = annotationLeft as? RMAnnotation
//            let annotation2 = annotationRight as? RMAnnotation
//            
//            //Move Other Annotation to over CircleNearMeAnnotation
//            if(annotation1 == self.circleNearMe){
//                return NSComparisonResult.OrderedAscending
//            }else if(annotation2 == self.circleNearMe){
//                return NSComparisonResult.OrderedDescending
//            }else if annotation1!.isUserLocationAnnotation && !annotation2!.isUserLocationAnnotation{
//                return NSComparisonResult.OrderedAscending
//            }else if !annotation1!.isUserLocationAnnotation && annotation2!.isUserLocationAnnotation{
//                return NSComparisonResult.OrderedDescending
//            }
//            //Other Annotation is Equal Priority
//            return NSComparisonResult.OrderedSame
//        }
//        return sort
//    }

    //MARK: CopyRight & MapMagic Logo
    private func initialCopyRight(){
        if(copyRightThImg == nil){
            initialCopyRightImage()
        }
        
//        addImageToBottomLeft(mapMagicImg!)
//        addImageToBottomRight(copyRightThImg!)
    }
    
    func setCopyRightLanguage(view : UIView , language : TNMapLanguage){
        mapLanguage = language
        if(mapMagicImg == nil){
            initialCopyRightImage()
            addImageToBottomLeft(mapMagicImg!)
        }
        
        if(language == TNMapLanguageThai){
            addImageToBottomRight(view , image: copyRightThImg!)
        }else{
            addImageToBottomRight(view , image: copyRightEnImg!)
        }
    }
    
    func initialCopyRightImage(){
        let copyRightThImgName  = "ic_copy_right_thai"
        let copryRightEnImgName = "ic_copy_right_eng"
        let mapMagicImgName     = "ic_mapmagic"
        
        mapMagicImg             = UIImage(named: mapMagicImgName)
        copyRightThImg          = UIImage(named: copyRightThImgName)
        copyRightEnImg          = UIImage(named: copryRightEnImgName)
    }
    
    private func addImageToBottomRight(view : UIView , image : UIImage){
        let paddingSpace    : CGFloat   = 8
        let bottomSpace     : CGFloat   = 5
        var rightImageRect  = CGRect()
        rightImageRect.origin.x    = self.frame.size.width - (image.size.width) - paddingSpace
        rightImageRect.origin.y    = self.frame.size.height - (image.size.height) - bottomSpace
        rightImageRect.size.width  = (image.size.width)
        rightImageRect.size.height = (image.size.height)
        
        if(RightImageView == nil){
            RightImageView              = UIImageView(frame: rightImageRect)
            RightImageView?.contentMode = .ScaleAspectFit
        }else{
            RightImageView?.removeFromSuperview()
        }
        RightImageView?.image = image
        self.addSubview(RightImageView!)
    }
    
    private func addImageToBottomLeft(image : UIImage){
        let paddingSpace    : CGFloat   = 8
        let logoBottomSpace : CGFloat   = 2
        
        var leftImageRect = CGRect()
        leftImageRect.origin.x    = paddingSpace
        leftImageRect.origin.y    = self.frame.size.height - (image.size.height) - paddingSpace - logoBottomSpace
        leftImageRect.size.width  = (image.size.width)
        leftImageRect.size.height = (image.size.height)
        
        if(LeftImageView == nil){
            LeftImageView               = UIImageView(frame: leftImageRect)
            LeftImageView?.contentMode  = .ScaleAspectFit
        }else{
            LeftImageView?.removeFromSuperview()
        }
        LeftImageView?.image = image
        self.addSubview(LeftImageView!)
    }
    
    //MARK: MapView Delegate
    @objc internal func mapViewRegionDidChange(mapView: RMMapView!) {
        if( delegate != nil ){
            delegate?.mapViewRegionDidMove?(mapView)
        }
    }
    
    //for handle when tap on callout.
    @objc internal func tapOnCalloutAccessoryControl(control: UIControl!, forAnnotation annotation: RMAnnotation!, onMap map: RMMapView!){
        if(delegate != nil) {
            guard let selected = annotation else {
                return
            }
            delegate?.mapViewTapOnCalloutAccessoryControl?(control, forAnnotation: selected, onMap: self.mapViewThinknet!)
        }
    }
    
    @objc internal func singleTapOnMap(map: RMMapView!, at point: CGPoint) {
        if(delegate != nil) {
            delegate?.mapViewSingleTapOnMap?(map, at: point)
        }
    }
    
    @objc internal func longPressOnMap(map: RMMapView!, at point: CGPoint, currentState state: UIGestureRecognizerState, touching touch: Bool) {
        if(delegate != nil && (state == UIGestureRecognizerState.Began || state == UIGestureRecognizerState.Ended)) {
            delegate?.mapViewLongPressOnMap?(map, at: point, touching: touch)
        }
    }
    
    @objc internal func beforeMapMove(map: RMMapView!, byUser wasUserAction: Bool) {
        if(delegate != nil) {
            delegate?.mapViewMovebyUser?(map, byUser: wasUserAction, userTouching: true)
        }
    }
    
    @objc internal func afterMapMove(map: RMMapView!, byUser wasUserAction: Bool) {
        if(delegate != nil) {
            delegate?.mapViewMovebyUser?(map, byUser: wasUserAction, userTouching: false)
        }
    }
}

@objc protocol TNMapViewDelegate{
    optional func mapViewTapOnCalloutAccessoryControl(control: UIControl ,forAnnotation annotation: RMAnnotation, onMap map: RMMapView)
    optional func mapViewSingleTapOnMap(map: RMMapView,at point: CGPoint)
    optional func mapViewRegionDidMove(map: RMMapView)
    optional func mapViewLongPressOnMap(map: RMMapView,at point: CGPoint,touching: Bool)
    optional func mapViewMovebyUser(map: RMMapView,byUser wasUserAction: Bool,userTouching: Bool)
}


extension UIColor {
    convenience init(hexString: String) {
        let hex = hexString.stringByTrimmingCharactersInSet(NSCharacterSet.alphanumericCharacterSet().invertedSet)
        var int = UInt32()
        NSScanner(string: hex).scanHexInt(&int)
        let a, r, g, b: UInt32
        switch hex.characters.count {
        case 3: // RGB (12-bit)
            (a, r, g, b) = (255, (int >> 8) * 17, (int >> 4 & 0xF) * 17, (int & 0xF) * 17)
        case 6: // RGB (24-bit)
            (a, r, g, b) = (255, int >> 16, int >> 8 & 0xFF, int & 0xFF)
        case 8: // ARGB (32-bit)
            (a, r, g, b) = (int >> 24, int >> 16 & 0xFF, int >> 8 & 0xFF, int & 0xFF)
        default:
            (a, r, g, b) = (1, 1, 1, 0)
        }
        self.init(red: CGFloat(r) / 255, green: CGFloat(g) / 255, blue: CGFloat(b) / 255, alpha: CGFloat(a) / 255)
    }
}

