//
//  JTMForgotPasswordTests.swift
//  JobThai
//
//  Created by weerapon on 6/1/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import XCTest

class JTMForgotPasswordTests: XCTestCase {
    
    private final let SLEEP_LAUNCH_APP : UInt32 = 3
    private final let SLEEP_FINISHED_APP : UInt32 = 1
    private final let SLEEP_WAITING_APP : UInt32 = 3
    
    let app = XCUIApplication()
    
    // Mark-: Case ---> Not Typing
    func testNoEmail() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        
        let boolArrage = getLocalizeStringForKeyForUITests(alert.label, key: "ALERT_NO_ENTER_EMAIL")
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // Mark-: Case ---> Email @example.com :: Service
    func testEnterEmailExample() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("abc@example.com")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ข้อมูลอีเมลไม่ถูกต้อง อีเมล \"@example.com\" เป็นอีเมลตัวอย่าง"
        let textAlertEN = "Your email address is incorrect. \"@example.com\" is only a sample."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // Mark-: Case ---> Email @jobthai.com :: Service
    func testEnterEmailJobthai() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("abc@jobthai.com")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ข้อมูลอีเมลไม่ถูกต้อง อีเมล \"@jobthai.com\" เป็นอีเมลตัวอย่าง"
        let textAlertEN = "Your email address is incorrect. \"@jobthai.com\" is only a sample."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // Mark-: Case ---> Email Invalid :: Fail
    func testEnterEmailInValid() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("abcd@a.x")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        
        let boolArrage = getLocalizeStringForKeyForUITests(alert.label, key: "ALERT_EMAIL_NOT_CORRECT_FORMAT")
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // Mark-: Case ---> Email Invalid :: Success :: Service
    func testEnterEmailValid() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("abc@abc.be")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ระบบได้ส่งลิงค์เปลี่ยนรหัสผ่าน ไปยังอีเมลของคุณเรียบร้อยแล้ว กรุณาตรวจสอบอีเมลเพื่อตั้งรหัสผ่านใหม่"
        let textAlertEN = "We have already sent you an email with a link to reset your password.\n Please check your email to set up a new password."
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // Mark-: Case ---> Email HaveInSystem :: Success :: Service
    func testEnterEmailHaveInSystem() {
        self.baseUI()
        
        let emailTextField = app.textFields["forgotPasswordTextField"]
        emailTextField.tap()
        emailTextField.typeText("kanya@think.net")
        
        let sendButton = app.buttons["forgotSendButton"]
        sendButton.tap()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ไม่พบอีเมลนี้ในระบบ กรุณากรอกข้อมูลใหม่อีกครั้ง"
        let textAlertEN = "Your email address is incorrect. Please try again."
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
}

extension JTMForgotPasswordTests {
    func baseUI() {
        app.launch()
        
        sleep(self.SLEEP_WAITING_APP)
        
        // Logout and Login
        let mainMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        if (mainMenu.cells.count == 8) {
            self.logOut()
        }
        
        app.navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic slide menu"].tap()
        
        mainMenu.cells.elementBoundByIndex(1).tap()
        
        app.buttons["ForgotPasswordButton"].tap()
    }
    
    func logOut() {
        let mainMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        app.navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic slide menu"].tap()
        mainMenu.cells.elementBoundByIndex(7).tap()
        let alertLogOut = app.alerts[ID_ALERT_LOGOUT]
        let alertButtonLogOut = alertLogOut.collectionViews.buttons
        let LogOutButton = alertButtonLogOut.elementBoundByIndex(1)
        LogOutButton.tap()
    }
    
}
