//
//  JTMRegisterTests.swift
//  JobThai
//
//  Created by tayathorn on 6/3/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import XCTest

class JTMRegisterTests: XCTestCase {
    
    let SLEEP_LAUNCH_APP : UInt32 = 3
    let SLEEP_FINISHED_APP : UInt32 = 2
    let SLEEP_WAITING : UInt32 = 3
    
    let VALID_EMAIL = "vaiohkae+01@gmail.com"
    let VALID_PASSWORD = "testuser"
    let app = XCUIApplication()
    
    // MARK :- Case --> did not type in textField (4 cases)
    func testNoEmail() {
        self.baseUI()
        
        self.enterEmail("")
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_EMAIL"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    func testNoConfirmEmail() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_CONFIRM_EMAIL"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    func testNoPassword() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_PASS"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    func testNoConfirmPassword() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_CONFIRM_PASS"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Invalid Email [INVALID EMAIL]
    func testEnterInvalidEmail() {
        
        let emailList = ["yok.com", "test@abc.a"]
        var round = 0
        
        self.baseUI()
        
        for email in emailList {
            round++
            print("Enter Email : \(email)")
            self.enterEmail(email)
            self.enterConfirmEmail(email)
            if round == 1 {
                self.enterPassword(VALID_PASSWORD)
                self.enterConfirmPassword(VALID_PASSWORD)
            }
            
            self.clickRegisterButton()
            
            XCTAssertTrue(self.checkAlert("ERROR_EMAIL_FORMAT"))
            sleep(SLEEP_WAITING)
            
            if round != emailList.count {
                self.clickAlertOKButton()
                
                self.clearEmail()
                self.clearConfirmEmail()
            }
        }
    }
    
    // MARK :- Case --> Invalid Email [VALID EMAIL]
    func testEnterValidEmail() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
//        let alert = app.alerts["alert"]
//        
//        if alert.exists {
//            XCTFail("Still have some error message")
//        }
        
        if (self.checkAlert("ERROR_EMAIL_FORMAT")) {
            XCTFail()
        }
        
//        XCTAssertTrue(!self.checkAlert("ERROR_EMAIL_FORMAT"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Email @example.com :: Service
    func testEnterEmailExample() {
        self.baseUI()
        
        self.enterEmail("test@example.com")
        self.enterConfirmEmail("test@example.com")
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ข้อมูลอีเมลไม่ถูกต้อง อีเมล \"@example.com\" เป็นอีเมลตัวอย่าง"
        let textAlertEN = "Your email address is incorrect. \"@example.com\" is only a sample."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)

        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Email @jobthai.com :: Service
    func testEnterEmailJobthai() {
        self.baseUI()
        
        self.enterEmail("test@jobthai.com")
        self.enterConfirmEmail("test@jobthai.com")
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "ข้อมูลอีเมลไม่ถูกต้อง อีเมล \"@jobthai.com\" เป็นอีเมลตัวอย่าง"
        let textAlertEN = "Your email address is incorrect. \"@jobthai.com\" is only a sample."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Email != ConfirmEmail
    func testEmailNotMatch() {
        self.baseUI()
        
        self.enterEmail("test@email.com")
        self.enterConfirmEmail("testja@email.com")
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_EMAIL_NOT_MATCH"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Password != ConfirmPassword
    func testPasswordNotMatch() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword("testtest")
        
        self.clickRegisterButton()
        
        XCTAssertTrue(self.checkAlert("ERROR_PASS_NOT_MATCH"))
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Password < 6 char :: Service
    func testEnterPasswordLessThan6() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword("test")
        self.enterConfirmPassword("test")
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "กรุณากรอกรหัสผ่านอย่างน้อย 6 หลัก"
        let textAlertEN = "Please enter a password at least 6 characters long."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Password > 15 char :: Service
    func testEnterPasswordMoreThan15() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword("testtest123456789")
        self.enterConfirmPassword("testtest123456789")
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "กรุณากรอกรหัสผ่าน ไม่เกิน 15 หลัก"
        let textAlertEN = "Please enter a password. (No more than 15 characters)"
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Password invalid format :: Service
    func testEnterInvalidPassword() {
        
        let passwordList = ["กขคง123", "abc$1234", "abc defg"]
        var round = 0
        
        self.baseUI()
        
        for password in passwordList {
            round++
            print("Enter Email : \(password)")
            if round == 1 {
                self.enterEmail(VALID_EMAIL)
                self.enterConfirmEmail(VALID_EMAIL)
            }
            
            self.enterPassword(password)
            self.enterConfirmPassword(password)
            
            self.clickRegisterButton()
            
            let alert = app.alerts["alert"]
            let textAlertTH = "กรุณากรอกรหัสผ่าน เป็นตัวอักษรภาษาอังกฤษ ตัวเลข หรือเครื่องหมาย_เท่านั้น"
            let textAlertEN = "Please enter a password with alphanumeric characters or underscore (_) only."
            
            let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
            XCTAssertTrue(boolArrage)
            
            sleep(SLEEP_WAITING)
            
            if round != passwordList.count {
                self.clickAlertOKButton()
                
                self.clearPassword()
                self.clearConfirmPassword()
            }
        }
    }
    
    // MARK :- Case --> Password repeat number (555555) :: Service
    func testEnterPasswordRepeatNumber() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword("555555")
        self.enterConfirmPassword("555555")
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "เพื่อความปลอดภัยของข้อมูล ไม่ควรใช้รหัสผ่านที่เป็นเลขซ้ำกัน"
        let textAlertEN = "For security reason, a password should not be a repeating numbers."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Password sequential number (123456) :: Service
    func testEnterPasswordSequenceNumber() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword("123456")
        self.enterConfirmPassword("123456")
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "เพื่อความปลอดภัยของข้อมูล ไม่ควรใช้รหัสผ่านที่เป็นเลขเรียงกัน"
        let textAlertEN = "For security reason, a password should not be a sequential numbers."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Existed email in service :: Service
    func testExistedEmail() {
        self.baseUI()
        
        self.enterEmail("vaiohkae@gmail.com")
        self.enterConfirmEmail("vaiohkae@gmail.com")
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "อีเมลนี้มีผู้ใช้งานแล้ว กรุณาเลือกใช้อีเมลอื่น"
        let textAlertEN = "This email address is already registered, please choose another email address."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
    // MARK :- Case --> Register Success :: Service
    func testRegisterSuccess() {
        self.baseUI()
        
        self.enterEmail(VALID_EMAIL)
        self.enterConfirmEmail(VALID_EMAIL)
        self.enterPassword(VALID_PASSWORD)
        self.enterConfirmPassword(VALID_PASSWORD)
        
        self.clickRegisterButton()
        
        let alert = app.alerts["alert"]
        let textAlertTH = "กรุณาตรวจสอบอีเมลของคุณ เพื่อยืนยันการสมัครสมาชิก"
        let textAlertEN = "Please check your email to verify your registration."
        
        let boolArrage = compareString(alert.label, localTH: textAlertTH, localEN: textAlertEN)
        XCTAssertTrue(boolArrage)
        
        sleep(SLEEP_FINISHED_APP)
    }
    
}

extension JTMRegisterTests {
    func baseUI() {
        app.launch()
        
        sleep(self.SLEEP_LAUNCH_APP)
        
        // Logout and Login
        let mainMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        if (mainMenu.cells.count == 8) {
            self.logOut()
        }
        
        app.navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic slide menu"].tap()
        
        mainMenu.cells.elementBoundByIndex(1).tap()
        
        app.buttons["RegisterButton"].tap()
    }
    
    func logOut() {
        let mainMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        app.navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic slide menu"].tap()
        mainMenu.cells.elementBoundByIndex(7).tap()
        let alertLogOut = app.alerts[ID_ALERT_LOGOUT]
        let alertButtonLogOut = alertLogOut.collectionViews.buttons
        let LogOutButton = alertButtonLogOut.elementBoundByIndex(1)
        LogOutButton.tap()
    }
    
    func clickRegisterButton() {
        let sendButton = app.buttons["registerSendButton"]
        
        repeat {
            app.swipeUp()
        } while(!sendButton.exists)
        
        sendButton.tap()
    }
    
    func checkAlert(alertKey : String) -> Bool {
        let alert = app.alerts["alert"]
        
        if alert.exists {
            let boolArrage = getLocalizeStringForKeyForUITests(alert.label, key: alertKey)
//            XCTAssertTrue(boolArrage)
            return boolArrage
            
        } else {
            XCTFail("Not found Alert")
        }
        
        return false
    }
    
    func clickAlertOKButton() {
        let alert = app.alerts["alert"]
        let alertButton = alert.collectionViews.buttons
        let alertOKButton = alertButton.elementBoundByIndex(0)
        
        alertOKButton.tap()
    }
    
    // MARK :- Type text
    func enterEmail(email:String) {
        let emailTextField = app.textFields["registerEmailTextField"]
        emailTextField.tap()
        emailTextField.typeText(email)
    }
    
    func enterConfirmEmail(email:String) {
        let confirmEmailTextField = app.textFields["registerConfirmEmailTextField"]
        confirmEmailTextField.tap()
        confirmEmailTextField.typeText(email)
    }
    
    func enterPassword(password:String) {
        let passwordTextField = app.secureTextFields["registerPasswordTextField"]
        passwordTextField.tap()
        passwordTextField.typeText(password)
    }
    
    func enterConfirmPassword(password:String) {
        let confirmPasswordTextField = app.secureTextFields["registerConfirmPasswordTextField"]
        confirmPasswordTextField.tap()
        confirmPasswordTextField.typeText(password)
    }
    
    // MARK :- Clear Text
    func clearEmail() {
        let emailTextField = app.textFields["registerEmailTextField"]
        emailTextField.tap()
        emailTextField.clearText()
    }
    
    func clearConfirmEmail() {
        let confirmEmailTextField = app.textFields["registerConfirmEmailTextField"]
        confirmEmailTextField.tap()
        confirmEmailTextField.clearText()
    }
    
    func clearPassword() {
        let passwordTextField = app.secureTextFields["registerPasswordTextField"]
        passwordTextField.tap()
        passwordTextField.clearText()
    }
    
    func clearConfirmPassword() {
        let confirmPasswordTextField = app.secureTextFields["registerConfirmPasswordTextField"]
        confirmPasswordTextField.tap()
        confirmPasswordTextField.clearText()
    }
}
