//
//  JTMSearchResultGridTests.swift
//  JobThai
//
//  Created by kanya on 5/25/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import XCTest

class JTMSearchResultGridTests: XCTestCase {
    
    let SLEEP_FOR_UI_UPDATE : UInt32 = 3
    let SLEEP_FOR_LOADING_SERVICE : UInt32 = 8
    let WAIT_TIME_OUT : NSTimeInterval = 20
    let LONG_PRESS_TIME : NSTimeInterval = 1
    
    let app = XCUIApplication()
    let searchResultList = XCUIApplication().collectionViews[ID_SEARCH_RESULT_LIST_COLLECTION_VIEWS]
    let searchResultGrid = XCUIApplication().collectionViews[ID_SEARCH_RESULT_GRID_COLLECTION_VIEWS]
    
    override func setUp() {
        super.setUp()
        continueAfterFailure = false
        XCUIApplication().launch()
        
        //Go to search result
        
        //Tap search navigation bar button on nav bar
        sleep(SLEEP_FOR_UI_UPDATE)
        let navigationBar = app.navigationBars[ID_MAIN_NAVIGATION_BAR]
        let searchBarButton = navigationBar.buttons[ID_HOME_SEARCH_BUTTON]
        searchBarButton.tap()
        
        //Select 1st search typr (quick search)
        let searchFilterTableView = app.tables[ID_SEARCH_FILTER_TYPE_TABLE_VIEWS]
        searchFilterTableView.cells.elementBoundByIndex(0).tap()
        
        //Tap search button
        sleep(SLEEP_FOR_UI_UPDATE)
        let searchFilterButton = app.buttons[ID_SEARCH_FILTER_QUICK_SEARCH_BUTTON]
        searchFilterButton.tap()
        
        //Wait Search Result List Appear
        waitForElementToAppear(searchResultList.cells.element, timeout: WAIT_TIME_OUT)
        searchResultList.swipeLeft()
        waitForElementToAppear(searchResultGrid.cells.element, timeout: WAIT_TIME_OUT)
    }
    
    override func tearDown() {
        super.tearDown()
    }

    //MARK:- Test Case
    func testLoadMoreCanCallMoreService() {
        var isFoundLoadMore = false
        
        while searchResultGrid.cells.element.exists {
            let loadMoreCell = searchResultGrid.cells.elementMatchingType(.Cell, identifier: ID_LOAD_MORE)
            
            if (loadMoreCell.exists) {
                isFoundLoadMore = true
                sleep(SLEEP_FOR_LOADING_SERVICE)
                searchResultGrid.swipeUp()
                let countVisibleCell = searchResultGrid.cells.count
                let lastCell = searchResultGrid.cells.elementBoundByIndex(countVisibleCell - 1)
                let sequenceLabel = lastCell.staticTexts.elementMatchingType(.Any, identifier: ID_SEARCH_RESULT_GRID_INDEX_LABEL)
                XCTAssertGreaterThanOrEqual(Int(sequenceLabel.label)!, 21, "After load more should have more than 20 cell")
                break
            }
            
            searchResultGrid.swipeUp()
        }
        
        if !(isFoundLoadMore) {
            XCTFail("Not Found Load More Cell")
        }
    }
    
    func testPressBackToTopWillGoToIndexZero() {
        var isFoundBackToTop = false
       
        while searchResultGrid.cells.element.exists {
            let backToTopButton = app.buttons[ID_SEARCH_RESULT_GRID_BACK_TO_TOP_BUTTON]
            if (backToTopButton.hittable) {
                isFoundBackToTop = true
                backToTopButton.tap()
                let firstCell = searchResultGrid.cells.elementBoundByIndex(0)
                let sequenceLabel = firstCell.staticTexts.elementMatchingType(.Any, identifier: ID_SEARCH_RESULT_GRID_INDEX_LABEL)
                XCTAssertEqual(Int(sequenceLabel.label), 1, "After back to top should show first cell")
                break
            }
            
            searchResultGrid.swipeUp()
        }
        
        if !(isFoundBackToTop) {
            XCTFail("Not Found Back To Top Button")
        }
    }
    
    func testTapSortFilterViewWillShowPicker() {
        let sortFilterHeaderCell = searchResultGrid.otherElements[ID_SORT_FILTER_HEADER_CELL]
        sortFilterHeaderCell.tap()
        XCTAssert(app.pickerWheels.element.exists)
    }
}
