//
//  JTMSearchResultListTests.swift
//  JobThai
//
//  Created by kanya on 5/24/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import XCTest

class JTMSearchResultListTests: XCTestCase {
    
    let SLEEP_FOR_UI_UPDATE : UInt32 = 3
    let SLEEP_FOR_LOADING_SERVICE : UInt32 = 8
    let WAIT_TIME_OUT : NSTimeInterval = 20
    let LONG_PRESS_TIME : NSTimeInterval = 1
    
    let app = XCUIApplication()
    let searchResultList = XCUIApplication().collectionViews[ID_SEARCH_RESULT_LIST_COLLECTION_VIEWS]
    
    //MARK:- Global case
    override func setUp() {
        super.setUp()
        continueAfterFailure = false
        app.launch()
        
        //Go to search result
        
        //Tap search navigation bar button on nav bar
        sleep(SLEEP_FOR_UI_UPDATE)
        let navigationBar = app.navigationBars[ID_MAIN_NAVIGATION_BAR]
        let searchBarButton = navigationBar.buttons[ID_HOME_SEARCH_BUTTON]
        searchBarButton.tap()
        
        //Select 1st search typr (quick search)
        let searchFilterTableView = app.tables[ID_SEARCH_FILTER_TYPE_TABLE_VIEWS]
        searchFilterTableView.cells.elementBoundByIndex(0).tap()
        
        //Tap search button
        sleep(SLEEP_FOR_UI_UPDATE)
        let searchFilterButton = app.buttons[ID_SEARCH_FILTER_QUICK_SEARCH_BUTTON]
        searchFilterButton.tap()
        
        //Wait Search Result List Appear
        waitForElementToAppear(searchResultList.cells.element, timeout: WAIT_TIME_OUT)
    }
    
    override func tearDown() {
        super.tearDown()
    }
    
    //MARK:- Test case
    func testLoadMoreCanCallMoreService() {
        var isFoundLoadMore = false
        
        while searchResultList.cells.element.exists {
            let loadMoreCell = searchResultList.cells.elementMatchingType(.Cell, identifier: ID_LOAD_MORE)
            
            if (loadMoreCell.exists) {
                isFoundLoadMore = true
                sleep(SLEEP_FOR_LOADING_SERVICE)
                searchResultList.swipeUp()
                let countVisibleCell = searchResultList.cells.count
                let lastCell = searchResultList.cells.elementBoundByIndex(countVisibleCell - 1)
                let sequenceLabel = lastCell.staticTexts.elementMatchingType(.Any, identifier: ID_SEARCH_RESULT_LIST_INDEX_LABEL)
                XCTAssertGreaterThanOrEqual(Int(sequenceLabel.label)!, 21, "After load more should have more than 20 cell")
                break
            }
            
            searchResultList.swipeUp()
        }
        
        if !(isFoundLoadMore) {
            XCTFail("Not Found Load More Cell")
        }
    }
    
    func testPressBackToTopWillGoToIndexZero() {
        var isFoundBackToTop = false

        while searchResultList.cells.element.exists {
            let backToTopButton = app.buttons[ID_SEARCH_RESULT_LIST_BACK_TO_TOP_BUTTON]
            if (backToTopButton.hittable) {
                isFoundBackToTop = true
                backToTopButton.tap()
                let firstCell = searchResultList.cells.elementBoundByIndex(0)
                let sequenceLabel = firstCell.staticTexts.elementMatchingType(.Any, identifier: ID_SEARCH_RESULT_LIST_INDEX_LABEL)
                XCTAssertEqual(Int(sequenceLabel.label), 1, "After back to top should show first cell")
                break
            }
            
            searchResultList.swipeUp()
        }
        
        if !(isFoundBackToTop) {
            XCTFail("Not Found Back To Top Button")
        }
    }
    
    func testTapSortFilterViewWillShowPicker() {
        let sortFilterHeaderCell = searchResultList.otherElements[ID_SORT_FILTER_HEADER_CELL]
        sortFilterHeaderCell.tap()
        XCTAssert(app.pickerWheels.element.exists)
    }
    
    func testSelectPickerWillUpdateSortFilter() {
        let sortFilterHeaderCell = searchResultList.otherElements[ID_SORT_FILTER_HEADER_CELL]
        sortFilterHeaderCell.tap()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        app.pickerWheels.element.pressForDuration(LONG_PRESS_TIME, thenDragToElement: app.pickerWheels.otherElements.elementBoundByIndex(1))
        app.pickerWheels
        
        let pickerToolBar = app.toolbars
        pickerToolBar.buttons.elementBoundByIndex(1).tap()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        waitForElementToAppear(searchResultList.cells.element, timeout: WAIT_TIME_OUT)
        let destinationFilterType = "More to less competitive jobs"
        let newSelectedSortFilterType = sortFilterHeaderCell.staticTexts.elementMatchingType(.Any, identifier: ID_SORT_FILTER_TYPE_LABEL).label
        XCTAssertEqual(newSelectedSortFilterType, destinationFilterType, "Sort Filter not change")
    }
}
